/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: crtc.h
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libhard\crtc
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.3
// DATE			: August, 14th 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Functions to initialize the graphic output,
//				  monitor, TV or TFT and manage the graphic
//				  surfaces.
// ------------------------------------------------------------
// VARIABLES	: HWCRTC_GraphicsModeTable
//				  HWCRTC_WaterMarkTable
//				  HWCRTC_DefaultPalette
//				  HWCRTC_pucTextModeDefaultPalette
//				  HWCRTC_pucFont8x15
//				  HWCRTC_TextModeCRTCTable
//				  HWCRTC_TextModeAttributTable
// ------------------------------------------------------------
// MACROS		: HWCRTC_GRAPHICSMODE_ID
//				  HWCRTC_GRAPHICSMODE
//				  HWCRTC_WATERMARK_ID
//				  HWCRTC_WATERMARK
//				  HWCRTC_OUTPUT
//				  HWCRTC_CONTROLER
//				  HWCRTC_MODEL
//				  HWCRTC_DCLK_...
//				  HWCRTC_OUTPUT_...
//				  HWCRTC_ERROR_...
//				  HWCRTC_CONTROLER_MONITOR
//				  HWCRTC_ENDTABLE
//				  HWCRTC_GETWIDTHFROMID
//				  HWCRTC_GETHEIGHTFROMID
// ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_Set256ColorsDefaultPalette
//				  HWCRTC_SetLinearGammaCorrection
//				  HWCRTC_SetDotClock
//				  HWCRTC_SetRefreshRate
//				  HWCRTC_GetNumberOfModes
//				  HWCRTC_GetFirstMode
//				  HWCRTC_GetNextMode
//				  HWCRTC_GetModeInfo
//				  HWCRTC_InitGraphicMode
//				  HWCRTC_GetBase
//				  HWCRTC_GetPitch
//				  HWCRTC_GetPixelDepth
//				  HWCRTC_GetScreenWidth
//				  HWCRTC_GetScreenHeight
//				  HWCRTC_GetOffsetX
//				  HWCRTC_GetOffsetY
//				  HWCRTC_FlipSurface
//				  HWCRTC_FlipBase
//				  HWCRTC_ScrollSurface
//				  HWCRTC_ChangePaletteType
//				  HWCRTC_SetPaletteEntry3Bytes
//				  HWCRTC_SetPaletteEntry4Bytes
//				  HWCRTC_SetPaletteEntryBitmap
//				  HWCRTC_WaitForVerticalRetrace
//				  HWCRTC_WaitForVerticalBlank
//				  HWCRTC_StopCRTC
//				  HWCRTC_StartCRTC
//				  HWCRTC_InitializeAdditionnalController
//				  HWCRTC_SetTextModeDefaultPalette
//				  HWCRTC_Load8xNTextFont
//				  HWCRTC_SetTextMode
//				  HWCRTC_GetCurrentDotClock
//				  HWCRTC_SelectCurrentDotClock
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.3
// DATE			: August, 14th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Modify the HWCRTC_SetWaterMark to interpolate
//				  the water mark values when the resolution is
//				  not in the table
//				  Added HWCRTC_GetFirstMode, HWCRTC_GetNextMode
//				  and HWCRTC_GetNumberOfMode functions
// ------------------------------------------------------------
// REVISION		: Rev 1.2
// DATE			: July, 18th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Add new functions to get screen parameters,
//				  HWCRTC_GetScreenWidth, HWCRTC_GetScreenHeight
//				  Contend of HWCRTC_InitGraphics mode have been 
//				  re-organize and split into different functions.
//				  Add support to shifted screen like in NTSC
//				  (16 lines missing in the top of screen)
//				  Add support of external 27MHz clock for TV.
// ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: March, 27th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Add support of 80x25 text mode and modify the
//				  HWCRTC_InitGraphicMode function to extract
//				  common part with text mode.
//				  New functions : 
//				  HWCRTC_InitializeAdditionnalController,
//				  HWCRTC_SetTextModeDefaultPalette,
//				  HWCRTC_Load8xNTextFont,
//				  HWCRTC_SetTextMode
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

#ifndef CRTC_H
#define CRTC_H

/* include file */
#include "host.h"
#include "framebuf.h"
#include "stv.h"
#include "tft.h"

/* ------------------------------------------------------------
// DEFINE		: HWCRTC_ENDTABLE 
// ------------------------------------------------------------
// DESCRIPTION	: ID for the end of HWCRTC_sGraphicsMode table and
//				  HWCRTC_sWaterMarkTable tables
// ------------------------------------------------------------*/
#define HWCRTC_ENDTABLE						0xffffffff

/* ------------------------------------------------------------
// DEFINE		: HWCRTC_ERROR_... 
// ------------------------------------------------------------
// DESCRIPTION	: Error message returned by CRTC function
// ------------------------------------------------------------*/
#define HWCRTC_ERROR_INVALIDPIXELDEPTH			-1
#define HWCRTC_ERROR_INVALIDGRAPHICSMODE		-2
#define HWCRTC_ERROR_UNSUPPORTEDGRAPHICSMODE	-3

/* ------------------------------------------------------------
// DEFINE		: HWCRTC_CONTROLER_...
// ------------------------------------------------------------
// DESCRIPTION	: Different type of output controller
// ------------------------------------------------------------
// HWCRTC_CONTROLER_MONITOR				0
// HWCRTC_CONTROLER_TV					1
// HWCRTC_CONTROLER_TFT					2
// ------------------------------------------------------------*/
#define HWCRTC_CONTROLER_VGA				1
#define HWCRTC_CONTROLER_TV					2
#define HWCRTC_CONTROLER_TFT				3

/* ------------------------------------------------------------
// DEFINE		: HWCRTC_CONTROLER_...
// ------------------------------------------------------------
// DESCRIPTION	: Different type of output controller
// ------------------------------------------------------------
// HWCRTC_DCLK_INTERNAL	: Use intenal selectable clock
// HWCRTC_DCLK_EXTERNAL	: Use external clock, low byte of
//						  the flag is the number of the external
//						  clock to select
// HWCRTC_DCLK_FIXED	: Use a fixed inter clock, 2 low bytes
//						  are the PLL values to set in
//						  DCLK registers
// ------------------------------------------------------------*/
#define HWCRTC_DCLK_INTERNAL				0x00000000
#define HWCRTC_DCLK_EXTERNAL				0x80000000
#define HWCRTC_DCLK_FIXED					0x40000000
#define HWCRTC_DCLK_27MHZ					0x1

/* ------------------------------------------------------------
// DEFINE		: HWCRTC_FORMAT_...
// ------------------------------------------------------------
// DESCRIPTION	: Different format of screen connected to VGA
// ------------------------------------------------------------
// ------------------------------------------------------------*/
#define HWCRTC_FORMAT_MONITOR				1
#define HWCRTC_FORMAT_100HZPAL				2
#define HWCRTC_FORMAT_VGANTSC				3

/* ------------------------------------------------------------
// DEFINE		: HWCRTC_OUTPUT_...
// ------------------------------------------------------------
// DESCRIPTION	: Different type of output, composed of a
//				  controler and a model
//				  HWCRTC_OUTPUT build the ID
//				  HWCRTC_CONTROLER return the controler from an ID
//				  HWCRTC_MODEL return the model from an ID
// ------------------------------------------------------------
// HWCRTC_OUTPUT_MONITOR			: VGA monitor
// HWCRTC_OUTPUT_TVPAL				: PAL TV
// HWCRTC_OUTPUT_TVNTSC				: NTSC TV
// HWCRTC_OUTPUT_TFTPHILIPSHLD1201	: Philips 800x600 TFT screen
// HWCRTC_OUTPUT_TFTLGPHILIPSLM151X2: LG-Philips 1024x768 TFT screen
// ------------------------------------------------------------*/
#define HWCRTC_OUTPUT(control, model)		(control + ((long)model << 16))
#define HWCRTC_CONTROLER(output)			(output & 0xffff)
#define HWCRTC_MODEL(output)				(output >> 16)

#define HWCRTC_OUTPUT_MONITOR				HWCRTC_OUTPUT(HWCRTC_CONTROLER_VGA, HWCRTC_FORMAT_MONITOR)
#define HWCRTC_OUTPUT_100HZPAL				HWCRTC_OUTPUT(HWCRTC_CONTROLER_VGA, HWCRTC_FORMAT_100HZPAL)
#define HWCRTC_OUTPUT_VGANTSC				HWCRTC_OUTPUT(HWCRTC_CONTROLER_VGA, HWCRTC_FORMAT_VGANTSC)
#define HWCRTC_OUTPUT_TVPAL					HWCRTC_OUTPUT(HWCRTC_CONTROLER_TV, HWSTV_FORMAT_PAL)
#define HWCRTC_OUTPUT_TVNTSC				HWCRTC_OUTPUT(HWCRTC_CONTROLER_TV, HWSTV_FORMAT_NTSC)
#define HWCRTC_OUTPUT_TVNTSC269				HWCRTC_OUTPUT(HWCRTC_CONTROLER_TV, HWSTV_FORMAT_NTSC269)
#define HWCRTC_OUTPUT_TVNTSCINTERNAL		HWCRTC_OUTPUT(HWCRTC_CONTROLER_TV, HWSTV_FORMAT_NTSCINTERNAL)
#define HWCRTC_OUTPUT_TVPALSVHS				HWCRTC_OUTPUT(HWCRTC_CONTROLER_TV, HWSTV_FORMAT_PALSVHS)
#define HWCRTC_OUTPUT_TVNTSCSVHS			HWCRTC_OUTPUT(HWCRTC_CONTROLER_TV, HWSTV_FORMAT_NTSCSVHS)
#define HWCRTC_OUTPUT_TVNTSC269SVHS			HWCRTC_OUTPUT(HWCRTC_CONTROLER_TV, HWSTV_FORMAT_NTSC269SVHS)
#define HWCRTC_OUTPUT_TFTPHILIPSHLD1201		HWCRTC_OUTPUT(HWCRTC_CONTROLER_TFT, HWTFT_FORMAT_PHILIPSHLD1201)
#define HWCRTC_OUTPUT_TFTLGPHILIPSLM151X2	HWCRTC_OUTPUT(HWCRTC_CONTROLER_TFT, HWTFT_FORMAT_LGPHILIPSLM151X2)

/* ------------------------------------------------------------
// DEFINE		: HWCRTC_PALETTE_...
// ------------------------------------------------------------
// DESCRIPTION	: Different type of palette format
// ------------------------------------------------------------
// ------------------------------------------------------------*/
#define HWCRTC_PALETTE_18BITS	0x1
#define HWCRTC_PALETTE_24BITS	0x2
#define HWCRTC_PALETTE_BYPASS	0x3

/* ------------------------------------------------------------
// STRUCTURE	: HWCRTC_GRAPHICSMODE
// ------------------------------------------------------------
// DESCRIPTION	: One entry of the table to find the CRTC
//				  timings to initialize
// ------------------------------------------------------------
// ulTimingsID			: ID to identify the concerned graphic
//						  mode filled with HWCRTC_GRAPHICSMODE_ID
//						  macro
// ulModel				: Concerned model of output
//						  (PAL, NTSC, one model of TFT...)
// ulSupportedHardware	: Hardware which support this mode
//						  should be a HWGENERIC_HAS_... value
// pucTimings			: Table of CRTC registers to initialize
// ulPixelClockFlag		: Type of DCLK (HWCRTC_DCLK_...)
// ucMiscReg			: Misc output register (control the hsync
//						  and vsync polarity)
// ------------------------------------------------------------*/
#define HWCRTC_GRAPHICSMODE_ID(width, height)	(((unsigned long)(width) & 0xffff) + (((unsigned long)(height) & 0xffff) << 16))
#define HWCRTC_GETWIDTHFROMID(id)				( (id) & 0xffff )
#define HWCRTC_GETHEIGHTFROMID(id)				( (id) >> 16  )

typedef struct tagHWCRTC_GRAPHICSMODE
{
	unsigned long ulOutput;
	unsigned long ulGraphicsModeID;
	unsigned long ulSupportedHardware;
	unsigned char *pucTimings;
	unsigned long ulPixelClockFlag;
	unsigned char ucMiscReg;
}HWCRTC_GRAPHICSMODE;

/* ------------------------------------------------------------
// STRUCTURE	: HWCRTC_WATERMARK_ID
// ------------------------------------------------------------
// DESCRIPTION	: One entry of the table to find the water
//				  marks to initialize
// ------------------------------------------------------------
// ulWaterMarkID		: ID to identify the concerned graphic
//						  mode filled with HWCRTC_WATERMARK_ID
//						  macro
// ulSupportedHardware	: Hardware which support this mode
//						  should be a HWGENERIC_HAS_... value
// ucLowWaterMark		: Low water mark value to initialize
// ucHighWaterMark		: High water mark value to initialize
// ------------------------------------------------------------*/
#define HWCRTC_WATERMARK_ID(width, height, bpp)			( (unsigned long)(width) * (unsigned long)(height) * (unsigned long)(bpp) )

typedef struct tagHWCRTC_WATERMARK
{
	unsigned long ulWaterMarkID;
	unsigned long ulSupportedHardware;
	unsigned char ucLowWaterMark;
	unsigned char ucHighWaterMark;
}HWCRTC_WATERMARK;

/* ------------------------------------------------------------
// STRUCTURE	: HWCRTC_ADDITONNALCONTROLLER
// ------------------------------------------------------------
// DESCRIPTION	: Parameters to initialize an additionnal
//				  controller as TFT or TV Output
// ------------------------------------------------------------
// ulWaterMarkID		: ID to identify the concerned graphic
//						  mode filled with HWCRTC_WATERMARK_ID
//						  macro
// ulSupportedHardware	: Hardware which support this mode
//						  should be a HWGENERIC_HAS_... value
// ucLowWaterMark		: Low water mark value to initialize
// ucHighWaterMark		: High water mark value to initialize
// ------------------------------------------------------------*/
typedef struct tagHWCRTC_ADDITONNALCONTROLLER
{
	unsigned long ulController;
	void (*pfInitFunction)(unsigned long);
}HWCRTC_ADDITONNALCONTROLLER;

/* ------------------------------------------------------------
// VARIABLE		: HWCRTC_psGraphicsModeTable 
// ------------------------------------------------------------
// DESCRIPTION	: Table to identify the CRTC timings to 
//				  initialize
// ------------------------------------------------------------*/
extern HWCRTC_GRAPHICSMODE HWCRTC_psGraphicsModeTable[];

/* ------------------------------------------------------------
// VARIABLE		: HWCRTC_psWaterMarkTable 
// ------------------------------------------------------------
// DESCRIPTION	: Table to identify the water marks to 
//				  initialize
// ------------------------------------------------------------*/
extern HWCRTC_WATERMARK HWCRTC_psWaterMarkTable[];

/* ------------------------------------------------------------
// VARIABLE		: HWCRTC_psControllerTable 
// ------------------------------------------------------------
// DESCRIPTION	: Table of additionnal controller to initialize
//				  as TFT or TV output
// ------------------------------------------------------------*/
extern HWCRTC_ADDITONNALCONTROLLER HWCRTC_psControllerTable[];

/* ------------------------------------------------------------
// VARIABLE		: HWCRTC_pucDefaultPalette 
// ------------------------------------------------------------
// DESCRIPTION	: Default color palette for 256 colors graphic
//				  mode
// ------------------------------------------------------------*/
extern unsigned char HWCRTC_pucDefaultPalette[];

/* ------------------------------------------------------------
// VARIABLE		: HWCRTC_pucFont8x15 
// ------------------------------------------------------------
// DESCRIPTION	: Default 9x15 font for text mode
// ------------------------------------------------------------*/
extern unsigned char HWCRTC_pucFont8x15[];

/* ------------------------------------------------------------
// VARIABLE		: HWCRTC_TextModeDefaultPalette 
// ------------------------------------------------------------
// DESCRIPTION	: Default color palette for text mode
// ------------------------------------------------------------*/
extern unsigned char HWCRTC_pucTextModeDefaultPalette[];

/* ------------------------------------------------------------
// VARIABLE		: HWCRTC_pucTextModeCRTCTable 
// ------------------------------------------------------------
// DESCRIPTION	: CRTC settings for the graphic mode
// ------------------------------------------------------------*/
extern unsigned char HWCRTC_pucTextModeCRTCTable[];

/* ------------------------------------------------------------
// VARIABLE		: HWCRTC_pucTextModeAttributTable 
// ------------------------------------------------------------
// DESCRIPTION	: Attribut controller settings for the graphic mode
// ------------------------------------------------------------*/
extern unsigned char HWCRTC_pucTextModeAttributTable[];

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_Set256ColorsDefaultPalette
// ------------------------------------------------------------
// DESCRIPTION	: Set the default color palette in 256 color 
//				  mode. This functions take default values from 
//				  the paldata.c file
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_Set256ColorsDefaultPalette(void);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_SetLinearGammaCorrection
// ------------------------------------------------------------
// DESCRIPTION	: Set a linear gamma correction for 64K and 16M 
//				  colors graphic mode (palette is used as gamma 
//				  correction in this graphics modes)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_SetLinearGammaCorrection(void);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_GetCurrentDotClock
// ------------------------------------------------------------
// DESCRIPTION	: Get number of the current used DCLK
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Current DCLK number
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWCRTC_GetCurrentDotClock(void);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_SelectCurrentDotClock
// ------------------------------------------------------------
// DESCRIPTION	: Select number of the current used DCLK
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulClock		: DCLK number between 0 and 4
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_SelectCurrentDotClock(unsigned long ulClock);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_SetDotClock
// ------------------------------------------------------------
// DESCRIPTION	: Set the CRTC pixel clock
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulClock		: Number of the clock to modify
// lFrequency	: Clock frequency in Hz
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_SetDotClock(unsigned long ulClock, long lFrequency);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_SetRefreshRate
// ------------------------------------------------------------
// DESCRIPTION	: Set the screen refresh rate (set the DCLK PLL
//				  to the required screen refresh rate 
//				  multiplied by the number of pixels in a 
//				  screen including blank signal)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulRefreshRate: Screen refresh rate in Hz
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_SetRefreshRate(unsigned long ulRefreshRate);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_GetNumberOfModes
// ------------------------------------------------------------
// DESCRIPTION	: Get number of graphics modes available for
//				  the current hardware
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Number of modes
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWCRTC_GetNumberOfModes(void);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_GetFirstMode
// ------------------------------------------------------------
// DESCRIPTION	: Get informations about the first graphics mode
//				  available in the current hardware
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// pulWidth		: Value to be filled with the width, NULL if not used
// pulHeight	: Value to be filled with the height, NULL if not used
// pulOutput	: Value to be filled with the type of output, NULL if not used
// pulRefresh	: Value to be filled with the possibility of refresh 
//				  rate, NULL if not used
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// pulWidth		: Value filled with the width
// pulHeight	: Value filled with the height
// pulOutput	: Value filled with the type of output
// pulRefresh	: Value filled with the possibility of refresh rate
//				  HWCRTC_DCLK_... flags
// ------------------------------------------------------------
// RETURN		: 0 if no graphics mode availables
//				  <> 0 if values are valid
// ------------------------------------------------------------
// MODIFIED		: HWCRTC_ulCurrentOffset
// ------------------------------------------------------------*/
unsigned long HWCRTC_GetFirstMode(unsigned long *pulWidth, unsigned long *pulHeight, unsigned long *pulOutput, unsigned long *pulRefresh);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_GetNextMode
// ------------------------------------------------------------
// DESCRIPTION	: Get informations about the next graphics mode
//				  available in the current hardware. This functions
//				  must be called only after a HWCRTC_GetFirstMode
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// pulWidth		: Value to be filled with the width, NULL if not used
// pulHeight	: Value to be filled with the height, NULL if not used
// pulOutput	: Value to be filled with the type of output, NULL if not used
// pulRefresh	: Value to be filled with the possibility of refresh 
//				  rate, NULL if not used
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// pulWidth		: Value filled with the width
// pulHeight	: Value filled with the height
// pulOutput	: Value filled with the type of output
// pulRefresh	: Value filled with the possibility of refresh rate
//				  HWCRTC_DCLK_... flags
// ------------------------------------------------------------
// RETURN		: 0 if no more graphics mode availables
//				  <> 0 if values are valid
// ------------------------------------------------------------
// MODIFIED		: HWCRTC_ulCurrentOffset
// ------------------------------------------------------------*/
unsigned long HWCRTC_GetNextMode(unsigned long *pulWidth, unsigned long *pulHeight, unsigned long *pulOutput, unsigned long *pulRefresh);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_GetNextMode
// ------------------------------------------------------------
// DESCRIPTION	: Get informations about the next graphics mode
//				  available in the current hardware. This functions
//				  must be called only after a HWCRTC_GetFirstMode
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulNumber		: Give info about the ulNumber graphics mode of
//				  the list (take care only of the mode available)
//				  in the current hardware.
// pulWidth		: Value to be filled with the width, NULL if not used
// pulHeight	: Value to be filled with the height, NULL if not used
// pulOutput	: Value to be filled with the type of output, NULL if not used
// pulRefresh	: Value to be filled with the possibility of refresh 
//				  rate, NULL if not used
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// pulWidth		: Value filled with the width
// pulHeight	: Value filled with the height
// pulOutput	: Value filled with the type of output
// pulRefresh	: Value filled with the possibility of refresh rate
//				  HWCRTC_DCLK_... flags
// ------------------------------------------------------------
// RETURN		: 0 if no graphics mode availables
//				  <> 0 if values are valid
// ------------------------------------------------------------
// MODIFIED		: HWCRTC_ulCurrentOffset
// ------------------------------------------------------------*/
unsigned long HWCRTC_GetModeInfo(unsigned long ulNumber, unsigned long *pulWidth, unsigned long *pulHeight, unsigned long *pulOutput, unsigned long *pulRefresh);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_InitGraphicMode
// ------------------------------------------------------------
// DESCRIPTION	: Initialize a graphic mode and a the 
//				  associated output (monitor, TV, TFT...)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// psSurface	: Graphic pixels to display (resolution and location)
// ulOutput		: Type of output : HWCRTC_OUTPUT_... parameter 
// ulRefreshRate: Output refresh rate in Hz
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: >= 0 if initialized
//				  HWCRTC_ERROR_... if error
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
long HWCRTC_InitGraphicMode(HWFRAMEBUF_SURFACE *psSurface, unsigned long ulOutput, unsigned long ulRefreshRate);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_InitTextMode
// ------------------------------------------------------------
// DESCRIPTION	: Initialize a text mode and a the 
//				  associated output (monitor, TV, TFT...)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// psSurface	: Text resolution to display (resolution and location)
// ulOutput		: Type of output : HWCRTC_OUTPUT_... parameter 
// ulRefreshRate: Output refresh rate in Hz
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: >= 0 if initialized
//				  HWCRTC_ERROR_... if error
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
long HWCRTC_InitTextMode(HWFRAMEBUF_SURFACE *psSurface, unsigned long ulOutput, unsigned long ulRefreshRate);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_GetBase
// ------------------------------------------------------------
// DESCRIPTION	: Get the base address of screen pixels
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// psSurface	: Graphic pixels to display (resolution and location)
// ulOutput		: Type of output : HWCRTC_OUTPUT_... parameter 
// ulRefreshRate: Output refresh rate in Hz
//                (Locked to 50 Hz for TV output)
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Base address of screen pixels
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWCRTC_GetBase(void);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_GetPitch
// ------------------------------------------------------------
// DESCRIPTION	: Get the number of bytes between two screen
//				  lines
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// psSurface	: Graphic pixels to display (resolution and location)
// ulOutput		: Type of output : HWCRTC_OUTPUT_... parameter 
// ulRefreshRate: Output refresh rate in Hz
//                (Locked to 50 Hz for TV output)
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Pitch of screen
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWCRTC_GetPitch(void);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_GetPixelDepth
// ------------------------------------------------------------
// DESCRIPTION	: Get the current graphic mode pixel depth
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Current pixel depth
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWCRTC_GetPixelDepth(void);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_GetScreenWidth
// ------------------------------------------------------------
// DESCRIPTION	: Get width in pixels of the current displayed
//                screen
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Width in pixels
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWCRTC_GetScreenWidth(void);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_GetScreenHeight
// ------------------------------------------------------------
// DESCRIPTION	: Get height in pixels of the current displayed
//                screen
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Height in pixels
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWCRTC_GetScreenHeight(void);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_GetOffsetX
// ------------------------------------------------------------
// DESCRIPTION	: Get the offset between CRTC base register
//				  and the first displayed pixel
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Offset in pixels
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWCRTC_GetOffsetX(void);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_GetOffsetY
// ------------------------------------------------------------
// DESCRIPTION	: Get the offset between CRTC base register
//				  and the first displayed pixel
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Offset in pixels
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWCRTC_GetOffsetY(void);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_SetPixelDepth
// ------------------------------------------------------------
// DESCRIPTION	: Set the graphic mode pixel depth
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBpp		: Number of bits per pixels
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -1 if the format is supported
//				  0 if the format is not supported
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWCRTC_SetPixelDepth(unsigned long ulBpp);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_FlipSurface
// ------------------------------------------------------------
// DESCRIPTION	: Change the location of displayed pixels
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBase		: New base address of screen pixels
// ulPitch		: Number of bytes between two screen lines 
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_FlipSurface(unsigned long ulBase, unsigned long ulPitch);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_FlipBase
// ------------------------------------------------------------
// DESCRIPTION	: Change the location of displayed pixels
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBase		: New base address of screen pixels
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_FlipBase(unsigned long ulBase);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_ScrollSurface
// ------------------------------------------------------------
// DESCRIPTION	: Change the location of displayed pixels
//				  inside a virtual desktop
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// psSurface	: The whole desktop
// ulX			: x coordinate of the rectangle to display
// ulY			: y coordinate of the rectangle to display
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_ScrollSurface(HWFRAMEBUF_SURFACE *psSurface, unsigned long ulX, unsigned long ulY);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_ChangePaletteType
// ------------------------------------------------------------
// DESCRIPTION	: Change the type of palette
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulFlag		: Type of palette : HWCRTC_PALETTE_...
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_ChangePaletteType(unsigned long ulFlag);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_SetPaletteEntry4Bytes
// ------------------------------------------------------------
// DESCRIPTION	: Modify palette entry
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// psEntry		: Table of RGB values, 4 bytes per entry
//				  4th values is unused 
// ulFirstEntry	: Number of the first entry to modify (0 - 256)
// ulNumEntries	: Number of entries to modify (up to 256)
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_SetPaletteEntry3Bytes(unsigned char *pucEntry, unsigned long ulFirstEntry, unsigned long ulNumEntries);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_SetPaletteEntryBitmap
// ------------------------------------------------------------
// DESCRIPTION	: Modify palette entry from a .bmp style
//				  R and B value are inverted compare to hardware
//				  palette
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// psEntry		: Table of RGB values, 4 bytes per entry
//				  4th values is unused 
// ulFirstEntry	: Number of the first entry to modify (0 - 256)
// ulNumEntries	: Number of entries to modify (up to 256)
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_SetPaletteEntryBitmap(unsigned char *pucEntry, unsigned long ulFirstEntry, unsigned long ulNumEntries);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_SetPaletteEntry4Bytes
// ------------------------------------------------------------
// DESCRIPTION	: Modify palette entry
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// psEntry		: Table of RGB values, 4 bytes per entry
//				  4th values is unused 
// ulFirstEntry	: Number of the first entry to modify (0 - 256)
// ulNumEntries	: Number of entries to modify (up to 256)
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_SetPaletteEntry4Bytes(unsigned char *pucEntry, unsigned long ulFirstEntry, unsigned long ulNumEntries);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_WaitForVerticalRetrace
// ------------------------------------------------------------
// DESCRIPTION	: Wait for the beginning of next graphic
//				  frame by polling
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_WaitForVerticalRetrace(void);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_WaitForVerticalBlank
// ------------------------------------------------------------
// DESCRIPTION	: Wait for the beginning of next vertical blank
//				  by polling
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_WaitForVerticalBlank(void);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_StopCRTC
// ------------------------------------------------------------
// DESCRIPTION	: Switch of the screen display
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_StopCRTC(void);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_StartCRTC
// ------------------------------------------------------------
// DESCRIPTION	: Switch on the screen display
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_StartCRTC(void);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_SetTextModeDefaultPalette
// ------------------------------------------------------------
// DESCRIPTION	: Set the default color palette for text mode.
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_SetTextModeDefaultPalette(void);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_InitializeAdditionnalController
// ------------------------------------------------------------
// DESCRIPTION	: Set the low and high water mark to assure
//				  no glitch will appear on screen
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulOutput		: Type of graphic output (HWCRTC_OUTPUT_...)
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_InitializeAdditionnalController(unsigned long ulOutput);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_Load8xNTextFont
// ------------------------------------------------------------
// DESCRIPTION	: Load text font into frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// pucFontData	: Font data, 256 blocks (one by ASCII code)
//				  of ulFontHeight bytes (one by character lines)
// ulFontHeight	: Number of lines for a character
// ulFontBlock	: Number of the font block to load (0 to 4) as
//				  VGA standard support 4 blocks of fonts
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_Load8xNTextFont(unsigned char *pucFontData, unsigned long ulFontHeight, unsigned long ulFontBlock);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_SetTextMode
// ------------------------------------------------------------
// DESCRIPTION	: Initialize text mode
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulOutput		: Type of output : HWCRTC_OUTPUT_... parameter 
// ulRefreshRate: Output refresh rate in Hz
//                (Locked to 50 Hz for TV output)
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_SetTextMode(unsigned long ulOutput, unsigned long ulRefreshRate);

#endif
