/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: crtcdata.c
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libhard\crtc
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.2
// DATE			: August, 25th 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Tables with timings and water marks to
//				  initialize the differents graphics mode
// ------------------------------------------------------------
// VARIABLES	: HWCRTC_psGraphicsModeTable
//				  HWCRTC_psWaterMarkTable
// ------------------------------------------------------------
// MACROS		: -
// ------------------------------------------------------------
// FUNCTIONS	: -
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.2
// DATE			: August, 25th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Fixed pixel clock is a value in KHz in place
//				  of the registers value. This modification
//				  have been required to support HCMOS7 PLLs
//				  where registers value are different
// ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: July, 18th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Change the HWCRTC_psCRTCTimingsTable into 
//				  HWCRTC_psGraphicsModeTable with more
//				  informations about the graphics mode
//				  (Pixel clock initialization and sync polarity)
//				  Add a new table HWCRTC_psControllerTable
//				  Add more graphics mode
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

/* include file */
#include "generic.h"
#include "crtc.h"
#include "tft.h"
#include "stv.h"

extern unsigned char HWCRTC_pucMonitor640x400[];
extern unsigned char HWCRTC_pucMonitor640x480[];
extern unsigned char HWCRTC_pucMonitor800x600[];
extern unsigned char HWCRTC_pucMonitor1024x768[];
extern unsigned char HWCRTC_pucMonitor1280x1024[];
extern unsigned char HWCRTC_pucMonitor1600x1200[];
extern unsigned char HWCRTC_pucExternalTVPAL[];
extern unsigned char HWCRTC_pucExternalTVNTSC[];
extern unsigned char HWCRTC_pucTVPAL[];
extern unsigned char HWCRTC_pucTVPAL320x200[];
extern unsigned char HWCRTC_pucTVPALFull[];
extern unsigned char HWCRTC_pucTVNTSC[];
extern unsigned char HWCRTC_pucTVNTSC27MHZ[];
extern unsigned char HWCRTC_pucTVNTSC320x200[];
extern unsigned char HWCRTC_pucTVNTSC640x480[];
extern unsigned char HWCRTC_pucTVNTSCFull[];
extern unsigned char HWCRTC_pucTVNTSCFull27MHZ[];
extern unsigned char HWCRTC_pucVGANTSC480x234[];
extern unsigned char HWCRTC_pucVGANTSC400x234[];
extern unsigned char HWCRTC_puc100HZPAL[];

/* ------------------------------------------------------------
// VARIABLE		: HWCRTC_GRAPHICSMODETable
// ------------------------------------------------------------
// DESCRIPTION	: Table to identify the CRTC timings to
//				  initialize
// ------------------------------------------------------------*/
HWCRTC_GRAPHICSMODE HWCRTC_psGraphicsModeTable[] =
{
	{ HWCRTC_OUTPUT_MONITOR,           HWCRTC_GRAPHICSMODE_ID(640,  400),  HWGENERIC_HAS_VGA,   HWCRTC_pucMonitor640x400,   HWCRTC_DCLK_INTERNAL, 0x2b },
	{ HWCRTC_OUTPUT_MONITOR,           HWCRTC_GRAPHICSMODE_ID(640,  480),  HWGENERIC_HAS_VGA,   HWCRTC_pucMonitor640x480,   HWCRTC_DCLK_INTERNAL, 0x2b },
	{ HWCRTC_OUTPUT_MONITOR,           HWCRTC_GRAPHICSMODE_ID(800,  600),  HWGENERIC_HAS_VGA,   HWCRTC_pucMonitor800x600,   HWCRTC_DCLK_INTERNAL, 0x2b },
	{ HWCRTC_OUTPUT_MONITOR,           HWCRTC_GRAPHICSMODE_ID(1024, 768),  HWGENERIC_HAS_VGA,   HWCRTC_pucMonitor1024x768,  HWCRTC_DCLK_INTERNAL, 0x2b },
	{ HWCRTC_OUTPUT_MONITOR,           HWCRTC_GRAPHICSMODE_ID(1280, 1024), HWGENERIC_HAS_VGA,   HWCRTC_pucMonitor1280x1024, HWCRTC_DCLK_INTERNAL, 0x2b },
	{ HWCRTC_OUTPUT_MONITOR,           HWCRTC_GRAPHICSMODE_ID(1600, 1200), HWGENERIC_HAS_VGA,   HWCRTC_pucMonitor1600x1200, HWCRTC_DCLK_INTERNAL, 0x2b },
	{ HWCRTC_OUTPUT_TVPAL,             HWCRTC_GRAPHICSMODE_ID(640,  480),  HWGENERIC_HAS_EXTTV, HWCRTC_pucExternalTVPAL,    HWCRTC_DCLK_EXTERNAL | HWCRTC_DCLK_27MHZ, 0x2b },
	{ HWCRTC_OUTPUT_TVNTSC,            HWCRTC_GRAPHICSMODE_ID(640,  430),  HWGENERIC_HAS_EXTTV, HWCRTC_pucExternalTVNTSC,   HWCRTC_DCLK_EXTERNAL | HWCRTC_DCLK_27MHZ, 0x2b },
	{ HWCRTC_OUTPUT_TVPAL,             HWCRTC_GRAPHICSMODE_ID(320,  200),  HWGENERIC_HAS_IOTV | HWGENERIC_HAS_MEMTV, HWCRTC_pucTVPAL320x200,  HWCRTC_DCLK_EXTERNAL | HWCRTC_DCLK_27MHZ, 0x2b },
	{ HWCRTC_OUTPUT_TVPAL,             HWCRTC_GRAPHICSMODE_ID(640,  480),  HWGENERIC_HAS_IOTV | HWGENERIC_HAS_MEMTV, HWCRTC_pucTVPAL,  HWCRTC_DCLK_EXTERNAL | HWCRTC_DCLK_27MHZ, 0x2b },
	{ HWCRTC_OUTPUT_TVPAL,             HWCRTC_GRAPHICSMODE_ID(720,  576),  HWGENERIC_HAS_IOTV | HWGENERIC_HAS_MEMTV, HWCRTC_pucTVPALFull,  HWCRTC_DCLK_EXTERNAL | HWCRTC_DCLK_27MHZ, 0x2b },
	{ HWCRTC_OUTPUT_TVNTSC,            HWCRTC_GRAPHICSMODE_ID(640,  430),  HWGENERIC_HAS_NTSC27MHZ, HWCRTC_pucTVNTSC27MHZ, HWCRTC_DCLK_EXTERNAL | HWCRTC_DCLK_27MHZ, 0x2b },
	{ HWCRTC_OUTPUT_TVNTSC,            HWCRTC_GRAPHICSMODE_ID(640,  430),  HWGENERIC_HAS_IOTV | HWGENERIC_HAS_MEMTV, HWCRTC_pucTVNTSC, HWCRTC_DCLK_EXTERNAL | HWCRTC_DCLK_27MHZ, 0x2b },
	{ HWCRTC_OUTPUT_TVNTSC269,         HWCRTC_GRAPHICSMODE_ID(640,  430),  HWGENERIC_HAS_IOTV | HWGENERIC_HAS_MEMTV, HWCRTC_pucTVNTSC, HWCRTC_DCLK_EXTERNAL | HWCRTC_DCLK_27MHZ, 0x2b },
	{ HWCRTC_OUTPUT_TVNTSC269,         HWCRTC_GRAPHICSMODE_ID(720,  480),  HWGENERIC_HAS_IOTV | HWGENERIC_HAS_MEMTV, HWCRTC_pucTVNTSCFull, HWCRTC_DCLK_EXTERNAL | HWCRTC_DCLK_27MHZ, 0x2b },
	{ HWCRTC_OUTPUT_TVNTSC,            HWCRTC_GRAPHICSMODE_ID(720,  480),  HWGENERIC_HAS_NTSC27MHZ, HWCRTC_pucTVNTSCFull27MHZ, HWCRTC_DCLK_EXTERNAL | HWCRTC_DCLK_27MHZ, 0x2b },
	{ HWCRTC_OUTPUT_TVNTSC,            HWCRTC_GRAPHICSMODE_ID(720,  480),  HWGENERIC_HAS_IOTV | HWGENERIC_HAS_MEMTV, HWCRTC_pucTVNTSCFull, HWCRTC_DCLK_EXTERNAL | HWCRTC_DCLK_27MHZ, 0x2b },
	{ HWCRTC_OUTPUT_TVNTSC269,         HWCRTC_GRAPHICSMODE_ID(720,  480),  HWGENERIC_HAS_IOTV | HWGENERIC_HAS_MEMTV, HWCRTC_pucTVNTSCFull, HWCRTC_DCLK_EXTERNAL | HWCRTC_DCLK_27MHZ, 0x2b },
	{ HWCRTC_OUTPUT_TVNTSC,            HWCRTC_GRAPHICSMODE_ID(320,  200),  HWGENERIC_HAS_IOTV | HWGENERIC_HAS_MEMTV, HWCRTC_pucTVNTSC320x200, HWCRTC_DCLK_EXTERNAL | HWCRTC_DCLK_27MHZ, 0x2b },
	{ HWCRTC_OUTPUT_TVNTSC269,         HWCRTC_GRAPHICSMODE_ID(320,  200),  HWGENERIC_HAS_IOTV | HWGENERIC_HAS_MEMTV, HWCRTC_pucTVNTSC320x200, HWCRTC_DCLK_EXTERNAL | HWCRTC_DCLK_27MHZ, 0x2b },
	{ HWCRTC_OUTPUT_TVNTSC,            HWCRTC_GRAPHICSMODE_ID(640,  480),  HWGENERIC_HAS_IOTV | HWGENERIC_HAS_MEMTV, HWCRTC_pucTVNTSC640x480, HWCRTC_DCLK_EXTERNAL | HWCRTC_DCLK_27MHZ, 0x2b },
	{ HWCRTC_OUTPUT_TVNTSC269,         HWCRTC_GRAPHICSMODE_ID(640,  480),  HWGENERIC_HAS_IOTV | HWGENERIC_HAS_MEMTV, HWCRTC_pucTVNTSC640x480, HWCRTC_DCLK_EXTERNAL | HWCRTC_DCLK_27MHZ, 0x2b },
	{ HWCRTC_OUTPUT_TFTPHILIPSHLD1201, HWCRTC_GRAPHICSMODE_ID(800,  600),  HWGENERIC_HAS_TFT, HWCRTC_pucMonitor800x600,         HWCRTC_DCLK_INTERNAL, 0x2b },
	{ HWCRTC_OUTPUT_TFTLGPHILIPSLM151X2, HWCRTC_GRAPHICSMODE_ID(1024,  768),  HWGENERIC_HAS_TFT, HWCRTC_pucMonitor1024x768,         HWCRTC_DCLK_INTERNAL, 0x2b },
	{ HWCRTC_OUTPUT_VGANTSC,    HWCRTC_GRAPHICSMODE_ID(480,  234),  HWGENERIC_HAS_VGA, HWCRTC_pucVGANTSC480x234,     HWCRTC_DCLK_INTERNAL | HWCRTC_DCLK_FIXED | 9695, 0xa3 },
	{ HWCRTC_OUTPUT_VGANTSC,    HWCRTC_GRAPHICSMODE_ID(400,  234),  HWGENERIC_HAS_VGA, HWCRTC_pucVGANTSC400x234,     HWCRTC_DCLK_INTERNAL | HWCRTC_DCLK_FIXED | 8254, 0xab },
	{ HWCRTC_OUTPUT_100HZPAL,          HWCRTC_GRAPHICSMODE_ID(720,  576),  HWGENERIC_HAS_VGA,  HWCRTC_puc100HZPAL,          HWCRTC_DCLK_INTERNAL | HWCRTC_DCLK_FIXED | 27000, 0x23 },

	{ HWCRTC_ENDTABLE, 0, 0, (unsigned char *)0 }
};

/* ------------------------------------------------------------
// VARIABLE		: HWCRTC_WaterMarkTable
// ------------------------------------------------------------
// DESCRIPTION	: Table to identify the water marks to
//				  initialize
// ------------------------------------------------------------*/
HWCRTC_WATERMARK HWCRTC_psWaterMarkTable[] =
{
	{ HWCRTC_WATERMARK_ID(640, 430, 8),   HWGENERIC_HAS_EDO, 0x39, 0xd0 },
	{ HWCRTC_WATERMARK_ID(640, 430, 16),  HWGENERIC_HAS_EDO, 0x69, 0xd0 },
	{ HWCRTC_WATERMARK_ID(640, 430, 24),  HWGENERIC_HAS_EDO, 0x99, 0xd0 },
	{ HWCRTC_WATERMARK_ID(640, 480, 8),   HWGENERIC_HAS_EDO, 0x3B, 0xd0 },
	{ HWCRTC_WATERMARK_ID(640, 480, 16),  HWGENERIC_HAS_EDO, 0x6B, 0xd0 },
	{ HWCRTC_WATERMARK_ID(640, 480, 24),  HWGENERIC_HAS_EDO, 0x9B, 0xd0 },
	{ HWCRTC_WATERMARK_ID(800, 600, 8),   HWGENERIC_HAS_EDO, 0x5B, 0xd0 },
	{ HWCRTC_WATERMARK_ID(800, 600, 16),  HWGENERIC_HAS_EDO, 0xAB, 0xd0 },
	{ HWCRTC_WATERMARK_ID(800, 600, 24),  HWGENERIC_HAS_EDO, 0xD3, 0xd0 },
	{ HWCRTC_WATERMARK_ID(1024, 768, 8),  HWGENERIC_HAS_EDO, 0x93, 0xd0 },
	{ HWCRTC_WATERMARK_ID(1024, 768, 16), HWGENERIC_HAS_EDO, 0xD3, 0xd0 },

	{ HWCRTC_WATERMARK_ID(640, 430, 8),   HWGENERIC_HAS_SDRAM, 0x39, 0xb8 },
	{ HWCRTC_WATERMARK_ID(640, 430, 16),  HWGENERIC_HAS_SDRAM, 0x69, 0xb8 },
	{ HWCRTC_WATERMARK_ID(640, 430, 24),  HWGENERIC_HAS_SDRAM, 0x99, 0xb8 },
	{ HWCRTC_WATERMARK_ID(640, 480, 8),   HWGENERIC_HAS_SDRAM, 0x3B, 0xb8 },
	{ HWCRTC_WATERMARK_ID(640, 480, 16),  HWGENERIC_HAS_SDRAM, 0x6B, 0xb8 },
	{ HWCRTC_WATERMARK_ID(640, 480, 24),  HWGENERIC_HAS_SDRAM, 0x9B, 0xb8 },
	{ HWCRTC_WATERMARK_ID(800, 600, 8),   HWGENERIC_HAS_SDRAM, 0x5B, 0xb8 },
	{ HWCRTC_WATERMARK_ID(800, 600, 16),  HWGENERIC_HAS_SDRAM, 0xAB, 0xb8 },
	{ HWCRTC_WATERMARK_ID(800, 600, 24),  HWGENERIC_HAS_SDRAM, 0xb8, 0xb8 },
	{ HWCRTC_WATERMARK_ID(1024, 768, 8),  HWGENERIC_HAS_SDRAM, 0x93, 0xb8 },
	{ HWCRTC_WATERMARK_ID(1024, 768, 16), HWGENERIC_HAS_SDRAM, 0xb8, 0xb8 },

	{ HWCRTC_ENDTABLE, 0, 0, 0 }
};

/* ------------------------------------------------------------
// VARIABLE		: HWCRTC_psControllerTable 
// ------------------------------------------------------------
// DESCRIPTION	: Table of additionnal controller to initialize
//				  as TFT or TV output
// ------------------------------------------------------------*/
HWCRTC_ADDITONNALCONTROLLER HWCRTC_psControllerTable[] =
{
	{ HWCRTC_CONTROLER_TV,  HWSTV_Init },
	{ HWCRTC_CONTROLER_TFT, HWTFT_Init },

	{ HWCRTC_ENDTABLE, NULL }
};

unsigned char HWCRTC_pucMonitor640x400[] =
{
	0x00, 0x5F,
	0x01, 0x4F,
	0x02, 0x50,
	0x03, 0x82,
	0x04, 0x55,
	0x05, 0x81,
	0x06, 0xBF,
	0x07, 0x1F,
	0x09, 0x4F,
	0x10, 0x9C,
	0x11, 0x0E,
	0x12, 0x8F,
	0x15, 0x96,
	0x16, 0xB9,
	0x25, 0x00,
	0xff
};

unsigned char HWCRTC_pucMonitor640x480[] =
{
	0x00, 0x5F,
	0x01, 0x4F,
	0x02, 0x50,
	0x03, 0x82,
	0x04, 0x54,
	0x05, 0x80,
	0x06, 0x0B,
	0x07, 0x3E,
	0x09, 0x40,
	0x10, 0xEA,
	0x11, 0x0C,
	0x12, 0xDF,
	0x15, 0xE7,
	0x16, 0x04,
	0x25, 0x00,
	0xff
};

unsigned char HWCRTC_pucMonitor800x600[] =
{
	0x00, 0x7F,
	0x01, 0x63,
	0x02, 0x64,
	0x03, 0x82,
	0x04, 0x6B,
	0x05, 0x1B,
	0x06, 0x72,
	0x07, 0xF0,
	0x09, 0x60,
	0x10, 0x59,
	0x11, 0x0D,
	0x12, 0x57,
	0x15, 0x57,
	0x16, 0x73,
	0x25, 0x00,
	0xff
};

unsigned char HWCRTC_pucMonitor1024x768[] =
{
	0x00, 0x99,
	0x01, 0x7F,
	0x02, 0x80,
	0x03, 0x9C,
	0x04, 0x83,
	0x05, 0x19,
	0x06, 0x2F,
	0x07, 0xF5,
	0x09, 0x60,
	0x10, 0x01,
	0x11, 0x09,
	0x12, 0xFF,
	0x15, 0xFF,
	0x16, 0x30,
	0x25, 0x00,
	0xff
};

unsigned char HWCRTC_pucMonitor1280x1024[] =
{
	0x00, 0xCE,
	0x01, 0x9F,
	0x02, 0xA0,
	0x03, 0x91,
	0x04, 0xA6,
	0x05, 0x14,
	0x06, 0x28,
	0x07, 0x52,
	0x09, 0x40,
	0x10, 0x01,
	0x11, 0x04,
	0x12, 0xFF,
	0x15, 0x01,
	0x16, 0x00,
	0x1A, 0x3c,
	0x25, 0x0D,
	0xff
};

unsigned char HWCRTC_pucMonitor1600x1200[] =
{
	0x00, 0xFF,
	0x01, 0xC7,
	0x02, 0xC8,
	0x03, 0x81,
	0x04, 0xD0,
	0x05, 0x18,
	0x06, 0x12,
	0x07, 0x11,
	0x09, 0x40,
	0x10, 0xB1,
	0x11, 0x04,
	0x12, 0xAF,
	0x15, 0xB0,
	0x16, 0x0A,
	0x1A, 0x38,
	0x25, 0x1F,
	0xff
};

unsigned char HWCRTC_pucExternalTVPAL[] =
{
	0x00, 0x67,
	0x01, 0x4F,
	0x02, 0x53,
	0x03, 0x8A,
	0x04, 0x58,
	0x05, 0x80,
	0x06, 0x6F,
	0x07, 0xBA,
	0x09, 0x40,
	0x10, 0x01,
	0x11, 0x0C,
	0x12, 0xDF,
	0x15, 0xF7,
	0x16, 0x57,
	0x25, 0x00,
	0x40, 0xB9,
	0x41, 0xB8,
	0x42, 0x00,
	0x43, 0x15,
	0x44, 0x00,
	0xff
};

unsigned char HWCRTC_pucExternalTVNTSC[] =
{
	0x00, 0x67,
	0x01, 0x4F,
	0x02, 0x55,
	0x03, 0x8A,
	0x04, 0x58,
	0x05, 0x80,
	0x06, 0x0B,
	0x07, 0x3E,
	0x09, 0x40,
	0x10, 0xCE,
	0x11, 0x09,
	0x12, 0xDF,
	0x15, 0xAD,
	0x16, 0x05,
	0x25, 0x00,
	0x40, 0xE1,
	0x41, 0xAF,
	0x42, 0x00,
	0x43, 0x15,
	0x44, 0x00,
	0xff
};

unsigned char HWCRTC_pucTVPAL[] =
{
	0x00, 0x67,
	0x01, 0x4F,
	0x02, 0x53,
	0x03, 0x8A,
	0x04, 0x58,
	0x05, 0x80,
	0x06, 0x6F,
	0x07, 0xBA,
	0x09, 0x00,
	0x10, 0x01,
	0x11, 0x0C,
	0x12, 0xDF,
	0x15, 0xF7,
	0x16, 0x57,
	0x25, 0x00,
	0x40, 0xE1,
	0x41, 0xB7,
	0x42, 0x00,
	0x43, 0x15,
	0x44, 0x80,
	0xff
};

unsigned char HWCRTC_pucTVPAL320x200[] =
{
	0x00, 0x67,
	0x01, 0x4F,
	0x02, 0x53,
	0x03, 0x8A,
	0x04, 0x58,
	0x05, 0x80,
	0x06, 0x6F,
	0x07, 0xBA,
	0x09, 0x41,/**/
	0x10, 0x01,
	0x11, 0x0C,
	0x12, 0x8F,/**/
	0x15, 0xF7,
	0x16, 0x57,
	0x25, 0x00,
	0x40, 0xE1,
	0x41, 0xB7,
	0x42, 0x08,/**/
	0x43, 0x15,
	0x44, 0x80,
	0xff
};

unsigned char HWCRTC_pucTVPALFull[] =
{
	0x00, 0x67,
	0x01, 0x59,
	0x02, 0x5a,
	0x03, 0x8a,
	0x04, 0x5b,
	0x05, 0x80,
	0x06, 0x6F,
	0x07, 0xf0,
	0x09, 0x60,
	0x10, 0x4f,
	0x11, 0x0c,
	0x12, 0x3f,
	0x15, 0x42,
	0x16, 0x6e,
	0x25, 0x00,
	0x40, 0xE1,
	0x41, 0x80,
	0x42, 0x00,
	0x43, 0x15,
	0x44, 0x80,
	0xff
};

unsigned char HWCRTC_pucTVNTSC[] =
{
	0x00, 0x66,
	0x01, 0x4F,
	0x02, 0x55,
	0x03, 0x8A,
	0x04, 0x58,
	0x05, 0x80,
	0x06, 0x0B,
	0x07, 0x3E,
	0x09, 0x00,
	0x10, 0xCE,
	0x11, 0x09,
	0x12, 0xDF,
	0x15, 0xAD,
	0x16, 0x05,
	0x25, 0x00,
	0x40, 0xF9,
	0x41, 0xAF,
	0x42, 0x00,
	0x43, 0x15,
	0x44, 0x80,
	0xff
};

unsigned char HWCRTC_pucTVNTSC640x480[] =
{
	0x00, 0x66,
	0x01, 0x4F,
	0x02, 0x55,
	0x03, 0x8A,
	0x04, 0x58,
	0x05, 0x80,
	0x06, 0x0B,
	0x07, 0x3E,
	0x09, 0x00,
	0x10, 0xE8,
	0x11, 0x05,
	0x12, 0xE0,
	0x15, 0xE0,
	0x16, 0x05,
	0x25, 0x00,
	0x40, 0xF9,
	0x41, 0xAF,
	0x42, 0x00,
	0x43, 0x15,
	0x44, 0x80,
	0xff
};

unsigned char HWCRTC_pucTVNTSC320x200[] =
{
	0x00, 0x67,
	0x01, 0x4F,
	0x02, 0x55,
	0x03, 0x8A,
	0x04, 0x58,
	0x05, 0x80,
	0x06, 0x0B,
	0x07, 0x3E,
	0x09, 0x41,
	0x10, 0xCE,
	0x11, 0x09,
	0x12, 0x8F,
	0x15, 0xAE,
	0x16, 0x05,
	0x25, 0x00,
	0x40, 0xF9,
	0x41, 0xB1,
	0x42, 0x00,
	0x43, 0x15,
	0x44, 0x80,
	0xff
};

unsigned char HWCRTC_pucTVNTSC27MHZ[] =
{
	0x00, 0x66,
	0x01, 0x4F,
	0x02, 0x55,
	0x03, 0x8A,
	0x04, 0x58,
	0x05, 0x80,
	0x06, 0x0B,
	0x07, 0x3E,
	0x09, 0x00,
	0x10, 0xCE,
	0x11, 0x09,
	0x12, 0xDF,
	0x15, 0xAD,
	0x16, 0x05,
	0x25, 0x00,
	0x3f, 0xF0,
	0x40, 0xF9,
	0x41, 0x6d,
	0x42, 0x28,
	0x43, 0x15,
	0x44, 0x80,
	0x80, 0x00,
	0x81, 0xa0,
	0x82, 0xb4,
	0x83, 0x1e,
	0xff
};

unsigned char HWCRTC_pucVGANTSC480x234[] =
{
	0x00, 0x48,
	0x01, 0x3B,
	0x02, 0x40,
	0x03, 0x89,
	0x04, 0x3E,
	0x05, 0x07,
	0x06, 0x09,
	0x07, 0x11,
	0x09, 0x40,
	0x10, 0xF4,
	0x11, 0x0C,
	0x12, 0xE9,
	0x15, 0xEC,
	0x16, 0x04,
	0x25, 0x00,
	0xff
};

unsigned char HWCRTC_pucVGANTSC400x234[] =
{
	0x00, 0x3B,
	0x01, 0x31,
	0x02, 0x34,
	0x03, 0x97,
	0x04, 0x36,
	0x05, 0x98,
	0x06, 0x09,
	0x07, 0x11,
	0x09, 0x40,
	0x10, 0xF6,
	0x11, 0x8E,
	0x12, 0xE9,
	0x15, 0xEE,
	0x16, 0x06,
	0x25, 0x00,
	0xff
};

unsigned char HWCRTC_puc100HZPAL[] =
{
	0x0, 0x67,
	0x1, 0x59,
	0x2, 0x5a,
	0x3, 0x8a,
	0x4, 0x5b,
	0x5, 0x80,
	0x6, 0x6f,
	0x7, 0xf0,
	0x9, 0x60,
	0x10, 0x4f,
	0x11, 0xc,
	0x12, 0x3f,
	0x15, 0x42,
	0x16, 0x6e,
	0x25, 0x0,
	0x39, 0x2a,
	0xff
};

unsigned char HWCRTC_pucTVNTSCFull[] =
{
	0x00, 0x66,
	0x01, 0x5a,
	0x02, 0x5b,
	0x03, 0x80,
	0x04, 0x5b,
	0x05, 0x00,
	0x06, 0x0b,
	0x07, 0x3e,
	0x09, 0x40,
	0x10, 0xe8,
	0x11, 0x05,
	0x12, 0xe0,
	0x15, 0xe0,
	0x16, 0x05,
	0x25, 0x00,
	0x40, 0xF9,
	0x41, 0x6c,
	0x42, 0x00,
	0x43, 0x15,
	0x44, 0x80,
	0xff
};

unsigned char HWCRTC_pucTVNTSCFull27MHZ[] =
{
	0x00, 0x66,
	0x01, 0x5a,
	0x02, 0x5b,
	0x03, 0x80,
	0x04, 0x5b,
	0x05, 0x00,
	0x06, 0x0b,
	0x07, 0x3e,
	0x09, 0x40,
	0x10, 0xe8,
	0x11, 0x05,
	0x12, 0xe0,
	0x15, 0xe0,
	0x16, 0x05,
	0x25, 0x00,
	0x3f, 0xf0,
	0x40, 0xF9,
	0x41, 0x6d,
	0x42, 0x28,
	0x43, 0x15,
	0x44, 0x80,
	0x80, 0x00,
	0x81, 0xa0,
	0x82, 0xb4,
	0x83, 0x1e,
	0xff
};

