/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 ST Microelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: cursor.h
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libhard\cursor
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE:		: March, 18th 2000
// AUTHOR:		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Functions to manage the hardware cursor
// ------------------------------------------------------------
// VARIABLES	: HWCURSOR_lCurrentX
//				  HWCURSOR_lCurrentY
// ------------------------------------------------------------
// MACROS		: -
// ------------------------------------------------------------
// FUNCTIONS	: HWCURSOR_Init
//				  HWCURSOR_NotifyGraphicsModeChange
//				  HWCURSOR_EnhancedChangeCursorPos
//				  HWCURSOR_Enable
//				  HWCURSOR_Disable
//				  HWCURSOR_ChangeCursorPos
//				  HWCURSOR_HideCursor
//				  HWCURSOR_ChangeCursor
//				  HWCURSOR_ChangeColor
//				  HWCURSOR_LoadAndMask
//				  HWCURSOR_LoadXorMask
//				  HWCURSOR_ShiftMask
//				  HWCURSOR_GetPositionX
//				  HWCURSOR_GetPositionY
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: July, 18th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Add the clipping of cursor on rigth side 
//				  (was wrongly displayed on the left for a high X
//				  value).
//				  Add the support of NTSC screen with an offset
//				  due to 16 lines missing on top of the screen
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

#ifndef HWCURSOR_H
#define HWCURSOR_H

#include "crtc.h"

/* ------------------------------------------------------------
// VARIABLE		: HWCURSOR_lCurrentX
// ------------------------------------------------------------
// DESCRIPTION	: Current X coordinate 
// ------------------------------------------------------------*/
extern long HWCURSOR_lCurrentX;

/* ------------------------------------------------------------
// VARIABLE		: HWCURSOR_lCurrentY
// ------------------------------------------------------------
// DESCRIPTION	: Current Y coordinate
// ------------------------------------------------------------*/
extern long HWCURSOR_lCurrentY;

/* ------------------------------------------------------------
// FUNCTION		: HWCURSOR_Init
// ------------------------------------------------------------
// DESCRIPTION	: Init the hardware cursor library
//				  This functions is necessary only when using
//				  HWCURSOR_EnhancedChangeCursorPos
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulTempMask1	: Offset in frame buffer of a space for a
//				  temporary cursor. This can be get from
//				  a HWFRAMEBUF_CursorAlloc call. This parameter
//				  is used only if cursor coordinate can be
//				  negative (need a temporary shifted cursor
//				  because the top and left clipping is not
//				  managed)
// ulTempMask2	: Offset in frame buffer of a space for a
//				  secondary temporary cursor.
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: HWCURSOR_ulShiftedMask1
//				  HWCURSOR_ulShiftedMask2
//				  HWCURSOR_ulSwitch
//				  HWCURSOR_ulShifted
//				  HWCURSOR_ulOffsetX
//				  HWCURSOR_ulOffsetY
//				  HWCURSOR_ulScreenWidth
// ------------------------------------------------------------*/
void HWCURSOR_Init(unsigned long ulTempMask1, unsigned long ulTempMask2);

/* ------------------------------------------------------------
// FUNCTIONS	: HWCURSOR_NotifyGraphicsModeChange
// ------------------------------------------------------------
// DESCRIPTION	: To be send after a graphics mode change
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCURSOR_NotifyGraphicsModeChange(void);

/* ------------------------------------------------------------
// FUNCTION		: HWCURSOR_EnhancedChangeCursorPos
// ------------------------------------------------------------
// DESCRIPTION	: Modify the location of the current cursor
//				  on screen. This functions manage clipping
//				  on top and left of the screen and the
//				  coordinate can be negative
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// lX			: New x screen coordinate of the left side
//				  of the cursor 
// lY			: New y screen coordinate of the top side
//				  of the cursor
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: HWCURSOR_ulSwitch
//				  HWCURSOR_ulShifted
// ------------------------------------------------------------*/
void HWCURSOR_EnhancedChangeCursorPos(long lX, long lY);

/* ------------------------------------------------------------
// FUNCTION		: HWCURSOR_Enable
// ------------------------------------------------------------
// DESCRIPTION	: Enable the hardware cursor display after a disable
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCURSOR_Enable(void);

/* ------------------------------------------------------------
// FUNCTION		: HWCURSOR_Disable
// ------------------------------------------------------------
// DESCRIPTION	: Disable the hardware cursor display
//				  The cursor continue to move when disable
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCURSOR_Disable(void);

/* ------------------------------------------------------------
// FUNCTION		: HWCURSOR_EnhancedChangeCursorPos
// ------------------------------------------------------------
// DESCRIPTION	: Modify the location of the current cursor
//				  on screen. This functions don't manage 
//				  clipping on top and left of the screen and 
//				  the coordinate need to be positive
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulX			: New x screen coordinate of the left side
//				  of the cursor 
// ulY			: New y screen coordinate of the top side
//				  of the cursor
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCURSOR_ChangeCursorPos(unsigned long ulX, unsigned long ulY);

/* ------------------------------------------------------------
// FUNCTION		: HWCURSOR_HideCursor
// ------------------------------------------------------------
// DESCRIPTION	: Disable the hardware cursor
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCURSOR_HideCursor(void);

/* ------------------------------------------------------------
// FUNCTION		: HWCURSOR_ChangeCursor
// ------------------------------------------------------------
// DESCRIPTION	: Change the cursor shape
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBase		: Offset of the new cursor in the frame buffer
//				  (The two mask need to be loaded with
//				  the HWCURSOR_LoadAndMask and 
//				  HWCURSOR_LoadXorMask first)
// ulHeight		: Height of the cursor
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: HWCURSOR_ulCurrentHeight
//				  HWCURSOR_ulCurrentMask
// ------------------------------------------------------------*/
void HWCURSOR_ChangeCursor(unsigned long ulBase, unsigned long ulHeight);

/* ------------------------------------------------------------
// FUNCTION		: HWCURSOR_ChangeColor
// ------------------------------------------------------------
// DESCRIPTION	: Change the cursor shape
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulColor0		: First cursor color (RGB888 value)
// ulColor1		: Second cursor color (RGB888 value)
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: HWCURSOR_ulCurrentHeight
//				  HWCURSOR_ulCurrentMask
// ------------------------------------------------------------*/
void HWCURSOR_ChangeColor(unsigned long ulColor0, unsigned long ulColor1);

/* ------------------------------------------------------------
// FUNCTION		: HWCURSOR_LoadAndMask
// ------------------------------------------------------------
// DESCRIPTION	: Load a cursor AND mask into the frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulSrcMask	: Linear address of mask to load 
// ulDstMask	: Linear address of destination
// ulPitch		: Number of bytes between two lines of the 
//				  source mask
// ulWidth		: Number of bits per lines of the mask
// ulHeight		: Number of lines of the mask
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCURSOR_LoadAndMask(unsigned long ulSrcMask, unsigned long ulDstMask, unsigned long ulPitch, unsigned long ulWidth, unsigned long ulHeight);

/* ------------------------------------------------------------
// FUNCTION		: HWCURSOR_LoadXorMask
// ------------------------------------------------------------
// DESCRIPTION	: Load a cursor XOR mask into the frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulSrcMask	: Linear address of mask to load 
// ulDstMask	: Linear address of destination
// ulPitch		: Number of bytes between two lines of the 
//				  source mask
// ulWidth		: Number of bits per lines of the mask
// ulHeight		: Number of lines of the mask
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCURSOR_LoadXorMask(unsigned long ulSrcMask, unsigned long ulDstMask, unsigned long ulPitch, unsigned long ulWidth, unsigned long ulHeight);

/* ------------------------------------------------------------
// FUNCTION		: HWCURSOR_ShiftMask
// ------------------------------------------------------------
// DESCRIPTION	: Shift mask of a cursor to manage top and left
//				  of the screen clipping
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulSrcMask	: Linear address of mask to shift 
// ulDstMask	: Linear address of destination
// ulHeight		: Number of lines of the mask to shift
// ulXShift		: Number of bits of horizontal shift
// ulYShift		: Number of lines of vertical shift
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCURSOR_ShiftMask(unsigned long ulSrcMask, unsigned long ulDstMask, unsigned long ulHeight, unsigned long ulXShift, unsigned long ulYShift);

/* ------------------------------------------------------------
// FUNCTION		: HWCURSOR_GetPositionX
// ------------------------------------------------------------
// DESCRIPTION	: Get the current cursor X position
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: X value
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define HWCURSOR_GetPositionX()	( HWCURSOR_lCurrentX )

/* ------------------------------------------------------------
// FUNCTION		: HWCURSOR_GetPositionY
// ------------------------------------------------------------
// DESCRIPTION	: Get the current cursor Y position
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Y value
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define HWCURSOR_GetPositionY()	( HWCURSOR_lCurrentY )

#endif

