/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: dump.c
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libhard\dump
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: July, 18th 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Function to save and restore current
//			  registers of different STPC block
// ------------------------------------------------------------
// VARIABLES	: -
// ------------------------------------------------------------
// MACROS		: -
// ------------------------------------------------------------
// FUNCTIONS	: HWDUMP_GetDumpSize
//				  HWDUMP_Dump
//				  HWDUMP_Restore
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.2
// DATE			: February, 12th 2001
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Added dump of TV encoder registers, CRTC
//				  screen and Video Pipeline screen
// ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: July, 18th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Add possibility to dump Video Pipeline and
//				  Video Input Port registers
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE			: March, 27th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

#include "dump.h"
#include "trace.h"
#include "host.h"
#include "crtc.h"
#include "stv.h"

/* ------------------------------------------------------------
// FUNCTIONS	: HWDUMP_GetDumpSize
// ------------------------------------------------------------
// DESCRIPTION	: Get the size o memory necessary for a dump
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulFlag		: Describe the registers to dump
//				  (HWDUMP_DUMP_ALL or a combination of
//				  HWDUMP_DUMP_... values)
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWDUMP_GetDumpSize(unsigned long ulFlag)
{
	unsigned long ulDumpSize;

	DBGTRACE_BEGINFUNCTION("HWDUMP_GetDumpSize");
	DBGTRACE_NUMERICPARAM(ulFlag);

	/* Size for the dump flag */
	ulDumpSize = sizeof(unsigned long);

	/* Dump misc register */
	if(ulFlag & HWDUMP_DUMP_MISCELLANEOUS)
	{
		DBGTRACE_BRANCH("Dump miscellaneous registers");
		ulDumpSize += HWDUMP_SIZE_MISCELLANEOUS;
	}

	/* Dump palette mask register */
	if(ulFlag & HWDUMP_DUMP_PALETTEMASK)
	{
		DBGTRACE_BRANCH("Dump palette mask register");
		ulDumpSize += HWDUMP_SIZE_PALETTEMASK;
	}

	/* Dump clocks */
	if(ulFlag & HWDUMP_DUMP_CLOCKS)
	{
		DBGTRACE_BRANCH("Dump DCLK registers");
		ulDumpSize += HWDUMP_SIZE_CLOCKS;
	}

	/* Dump color palette */
	if(ulFlag & HWDUMP_DUMP_PALETTE)
	{
		DBGTRACE_BRANCH("Dump color palette registers");
		ulDumpSize += HWDUMP_SIZE_PALETTE;
	}

	/* Dump DUMP registers */
	if(ulFlag & HWDUMP_DUMP_CRTC)
	{
		DBGTRACE_BRANCH("Dump CRTC registers");
		ulDumpSize += HWDUMP_SIZE_CRTC;
	}

	/* Dump attribut controller registers */
	if(ulFlag & HWDUMP_DUMP_ATTRIBUT)
	{
		DBGTRACE_BRANCH("Dump attribut controller registers");
		ulDumpSize += HWDUMP_SIZE_ATTRIBUT;
	}

	/* Dump graphic controller registers */
	if(ulFlag & HWDUMP_DUMP_GRAPHICS)
	{
		DBGTRACE_BRANCH("Dump graphics controller registers");
		ulDumpSize += HWDUMP_SIZE_GRAPHICS;
	}

	/* Dump sequencer controller registers */
	if(ulFlag & HWDUMP_DUMP_SEQUENCER)
	{
		DBGTRACE_BRANCH("Dump sequencer registers");
		ulDumpSize += HWDUMP_SIZE_SEQUENCER;
	}

	/* Dump video pipeline registers */
	if(ulFlag & HWDUMP_DUMP_VPIPE)
	{
		DBGTRACE_BRANCH("Dump video pipeline registers");
		ulDumpSize += HWDUMP_SIZE_VPIPE;
	}

	/* Dump VIP registers */
	if(ulFlag & HWDUMP_DUMP_VIP)
	{
		DBGTRACE_BRANCH("Dump VIP registers");
		ulDumpSize += HWDUMP_SIZE_VIP;
	}

	/* Dump TV output registers */
	if(ulFlag & HWDUMP_DUMP_TVOUTPUT)
	{
		DBGTRACE_BRANCH("Dump TV output registers");
		ulDumpSize += HWDUMP_SIZE_TVOUTPUT;
	}

	/* Dump CRTC screen */
	if(ulFlag & HWDUMP_DUMP_CRTCSCREEN)
	{
		DBGTRACE_BRANCH("Dump graphics screen registers");
		ulDumpSize += HWCRTC_GetScreenWidth() * HWCRTC_GetScreenHeight() * (HWCRTC_GetPixelDepth() >> 8);
	}

	/* Dump Video Pipeline screen */
	if(ulFlag & HWDUMP_DUMP_VPIPESCREEN)
	{
		DBGTRACE_BRANCH("Dump Video Pipeline screen");
		ulDumpSize += (REGHOST_GET_VPIPE_SRCDIM_REG() & 0xffff) * (REGHOST_GET_VPIPE_SRCDIM_REG() >> 16) * 2; 
	}

	DBGTRACE_ENDFUNCTION("HWDUMP_GetDumpSize");
	DBGTRACE_NUMERICPARAM(ulDumpSize);

	return ulDumpSize;
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWDUMP_Dump
// ------------------------------------------------------------
// DESCRIPTION	: Dump graphic mode registers
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulFlag		: Describe the registers to dump
//				  (HWDUMP_DUMP_ALL or a combination of
//				  HWDUMP_DUMP_... values)
// pucDump		: Block of memory to store registers values
//				  The size of this block can be return by the
//				  HWDUMP_GetDumpSize function
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWDUMP_Dump(unsigned long ulFlag, unsigned char *pucDump)
{
	unsigned long ulOffset;
	unsigned long i, j;

	DBGTRACE_BEGINFUNCTION("HWDUMP_Dump");
	DBGTRACE_NUMERICPARAM(ulFlag);
	DBGTRACE_POINTERPARAM(pucDump);

	ulOffset = 0;

	/* Save the dump flag */
	*(unsigned long *)(pucDump + ulOffset) = ulFlag;
	ulOffset += sizeof(unsigned long);

	/* Dump misc register */
	if(ulFlag & HWDUMP_DUMP_MISCELLANEOUS)
	{
		DBGTRACE_BRANCH("Dump miscellaneous register");
		pucDump[ulOffset] = REGHOST_GET_MISCELLANEOUS_REG();

		ulOffset += HWDUMP_SIZE_MISCELLANEOUS;
	}

	/* Dump palette mask register */
	if(ulFlag & HWDUMP_DUMP_PALETTEMASK)
	{
		DBGTRACE_BRANCH("Dump palette mask register");
		pucDump[ulOffset] = REGHOST_GET_PALETTEMASK_REG();

		ulOffset += HWDUMP_SIZE_PALETTEMASK;
	}

	/* Dump clocks */
	if(ulFlag & HWDUMP_DUMP_CLOCKS)
	{
		DBGTRACE_BRANCH("Dump DCLK registers");

		for(i = 0; i < HWDUMP_SIZE_CLOCKS; i++)
		{
			pucDump[ulOffset + i] = REGHOST_GET_CHIPSET_REG(0x40 + i);
		}

		ulOffset += HWDUMP_SIZE_CLOCKS;
	}

	/* Dump color palette */
	if(ulFlag & HWDUMP_DUMP_PALETTE)
	{
		unsigned char ucCRTC1a;
		unsigned char ucSeq6;

		DBGTRACE_BRANCH("Dump color palette registers");

		/* Unlock extended registers */
		ucSeq6 = REGHOST_GET_SEQUENCER_REG(0x6) * 0x57;
		REGHOST_SET_SEQUENCER_REG(0x6, 0x57);

		/* Set 8 bits palette access mode */
		ucCRTC1a = REGHOST_GET_CRTC_REG(0x1a);
		REGHOST_SET_CRTC_REG(0x1a, (unsigned char)(ucCRTC1a & ~0x2));

		/* Enable access to palette registers */
		REGHOST_GET_INPUTSTATUS_REG();
		REGHOST_SET_ATTRIBUT_REG(0);

		for(i = 0; i < HWDUMP_SIZE_PALETTE / 3; i ++)
		{
			/* Write look up entry index */
			REGHOST_SET_RAMDACREADINDEX_REG(i);

			/* Read RGB trio by 3 consecutive read */
			pucDump[ulOffset + 3 * i] = REGHOST_GET_RAMDACDATA_REG();
			pucDump[ulOffset + 3 * i + 1] = REGHOST_GET_RAMDACDATA_REG();
			pucDump[ulOffset + 3 * i + 2] = REGHOST_GET_RAMDACDATA_REG();
		}

		/* Enable display again */
		REGHOST_GET_INPUTSTATUS_REG();
		REGHOST_SET_ATTRIBUT_REG(20);

		/* restore registers */
		REGHOST_SET_CRTC_REG(0x1a, ucCRTC1a);
		REGHOST_SET_SEQUENCER_REG(0x6, ucSeq6);

		ulOffset += HWDUMP_SIZE_PALETTE;
	}

	/* Dump CRTC registers */
	if(ulFlag & HWDUMP_DUMP_CRTC)
	{
		unsigned char ucSaveMisc;

		DBGTRACE_BRANCH("Dump CRTC registers");

		/* Set DUMP registers access to 3d4 */
		ucSaveMisc = REGHOST_GET_MISCELLANEOUS_REG();
		REGHOST_SET_MISCELLANEOUS_REG(ucSaveMisc | 1);

		/* Get registers */
		for(i = 0; i < HWDUMP_SIZE_CRTC; i++)
		{
			pucDump[ulOffset + i] = REGHOST_GET_CRTC_REG(i);
		}

		/* Restore miscellaneous register */
		REGHOST_SET_MISCELLANEOUS_REG(ucSaveMisc);

		ulOffset += HWDUMP_SIZE_CRTC;
	}

	/* Dump attribut controller registers */
	if(ulFlag & HWDUMP_DUMP_ATTRIBUT)
	{
		unsigned char ucSaveMisc;
		unsigned char ucSaveSeq;

		DBGTRACE_BRANCH("Dump attribut controller registers");

		/* Set DUMP registers access to 3d4 and enable extended registers */
		ucSaveMisc = REGHOST_GET_MISCELLANEOUS_REG();
		REGHOST_SET_MISCELLANEOUS_REG(ucSaveMisc | 1);
		ucSaveSeq = REGHOST_GET_SEQUENCER_REG(0x06) * 0x57;
		REGHOST_SET_SEQUENCER_REG(0x06, 0x57);

		/* Get registers */
		for(i = 0; i < HWDUMP_SIZE_ATTRIBUT; i++)
		{
			/* Reset Atr contress adress/data flip-flop */
			REGHOST_GET_INPUTSTATUS_REG();

			/* Get value */
			pucDump[ulOffset + i] = REGHOST_GET_ATTRIBUT_REG(i);
		}

		/* turn on display again */
		REGHOST_GET_INPUTSTATUS_REG();
		REGHOST_SET_ATTRIBUT_REG(0x20);

		/* Restore registers */
		REGHOST_SET_MISCELLANEOUS_REG(ucSaveMisc);
		REGHOST_SET_SEQUENCER_REG(0x06, ucSaveSeq);

		ulOffset += HWDUMP_SIZE_ATTRIBUT;
	}

	/* Dump graphic controller registers */
	if(ulFlag & HWDUMP_DUMP_GRAPHICS)
	{
		DBGTRACE_BRANCH("Dump graphics controller registers");

		for(i = 0; i < HWDUMP_SIZE_GRAPHICS; i++)
		{
			pucDump[ulOffset + i] = REGHOST_GET_GRAPHICCONTROLLER_REG(i);
		}

		ulOffset += HWDUMP_SIZE_GRAPHICS;
	}

	DBGTRACE_NUMERICPARAM(ulOffset);

	/* Dump sequencer controller registers */
	if(ulFlag & HWDUMP_DUMP_SEQUENCER)
	{
		DBGTRACE_BRANCH("Dump sequencer registers");

		for(i = 0; i < HWDUMP_SIZE_SEQUENCER; i++)
		{
			pucDump[ulOffset + i] = REGHOST_GET_SEQUENCER_REG(i);
		}

		ulOffset += HWDUMP_SIZE_SEQUENCER;
	}

	/* Dump video pipeline registers */
	if(ulFlag & HWDUMP_DUMP_VPIPE)
	{
		DBGTRACE_BRANCH("Dump video pipeline registers");

		for(i = 0; i < HWDUMP_SIZE_VPIPE; i += 4)
		{
			*(unsigned long *)(pucDump + ulOffset + i) = REGHOST_GET_VPIPE_REG(i);
		}

		ulOffset += HWDUMP_SIZE_VPIPE;
	}

	/* Dump VIP registers */
	if(ulFlag & HWDUMP_DUMP_VIP)
	{
		DBGTRACE_BRANCH("Dump VIP registers");

		*(unsigned long *)(pucDump + ulOffset + 0x00) = REGHOST_GET_VIP_HORIZONTALTIMINGS_REG();
		*(unsigned long *)(pucDump + ulOffset + 0x04) = REGHOST_GET_VIP_VERTICALTIMINGS_REG();
		*(unsigned long *)(pucDump + ulOffset + 0x08) = REGHOST_GET_VIP_TIMINGS1_REG();
		*(unsigned long *)(pucDump + ulOffset + 0x0c) = REGHOST_GET_VIP_TIMINGS2_REG();
		*(unsigned long *)(pucDump + ulOffset + 0x14) = REGHOST_GET_VIP_ODDBASE_REG();
		*(unsigned long *)(pucDump + ulOffset + 0x18) = REGHOST_GET_VIP_EVENBASE_REG();
		*(unsigned long *)(pucDump + ulOffset + 0x1c) = REGHOST_GET_VIP_PITCH_REG();
		*(unsigned long *)(pucDump + ulOffset + 0x10) = REGHOST_GET_VIP_CONFIG_REG();

		ulOffset += HWDUMP_SIZE_VIP;
	}

	/* Dump TV output registers */
	if(ulFlag & HWDUMP_DUMP_TVOUTPUT)
	{
		DBGTRACE_BRANCH("Dump TV output registers");

		HWSTV_DetectHardware();

		for(i = 0; i < HWDUMP_SIZE_TVOUTPUT; i++)
		{
			*(pucDump + ulOffset + i) = HWSTV_ReadRegister((unsigned char)i);
		}

		ulOffset += HWDUMP_SIZE_TVOUTPUT;
	}

	/* Dump CRTC screen */
	if(ulFlag & HWDUMP_DUMP_CRTCSCREEN)
	{
		unsigned long ulWidth, ulHeight, ulBase, ulPitch;

		DBGTRACE_BRANCH("Dump graphics screen");

		ulWidth = HWCRTC_GetScreenWidth() * (HWCRTC_GetPixelDepth() >> 8);
		ulHeight = HWCRTC_GetScreenHeight();
		ulPitch = HWCRTC_GetPitch();
		ulBase = HWCRTC_GetBase();

		for(j = 0; j < ulHeight; j++)
		{
			for(i = 0; i < ulWidth; i++)
			{
				*(unsigned char *)(pucDump + ulOffset + i) = REGHOST_GET_ONSCREEN8(ulBase + i);
			}

			ulBase += ulPitch;
			ulOffset += ulWidth;
		}
	}

	/* Dump Video Pipeline screen */
	if(ulFlag & HWDUMP_DUMP_VPIPESCREEN)
	{
		unsigned long ulWidth, ulHeight, ulBase, ulPitch;

		DBGTRACE_BRANCH("Dump Video Pipeline screen");

		ulWidth = REGHOST_GET_VPIPE_SRCDIM_REG() & 0xffff;
		ulHeight = REGHOST_GET_VPIPE_SRCDIM_REG() >> 16;
		ulPitch = REGHOST_GET_VPIPE_PITCH_REG();
		ulBase = REGHOST_GET_VPIPE_SRCBASE_REG();

		for(j = 0; j < ulHeight; j++)
		{
			for(i = 0; i < ulWidth; i++)
			{
				*(unsigned char *)(pucDump + ulOffset + i) = REGHOST_GET_ONSCREEN8(ulBase + i);
			}

			ulBase += ulPitch;
			ulOffset += ulWidth;
		}
	}

	DBGTRACE_ENDFUNCTION("HWDUMP_Dump");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWDUMP_Restore
// ------------------------------------------------------------
// DESCRIPTION	: Restore dumped registers
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// pucDump		: Block of memory with registers values
//				  This block is filled by the HWDUMP_Dump
//				  register.
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWDUMP_Restore(unsigned char *pucDump)
{
	unsigned long ulOffset;
	unsigned long ulFlag;
	unsigned long i, j;

	DBGTRACE_BEGINFUNCTION("HWDUMP_Restore");
	DBGTRACE_POINTERPARAM(pucDump);

	ulOffset = 0;

	/* Restore the dump flag */
	ulFlag = *(unsigned long *)(pucDump + ulOffset);
	ulOffset += sizeof(unsigned long);

	/* Restore misc register */
	if(ulFlag & HWDUMP_DUMP_MISCELLANEOUS)
	{
		DBGTRACE_BRANCH("Restore miscellaneous register");

		REGHOST_SET_MISCELLANEOUS_REG(pucDump[ulOffset]);

		ulOffset += HWDUMP_SIZE_MISCELLANEOUS;
	}

	/* Restore palette mask register */
	if(ulFlag & HWDUMP_DUMP_PALETTEMASK)
	{
		DBGTRACE_BRANCH("Restore palette mask register");

		REGHOST_SET_PALETTEMASK_REG(pucDump[ulOffset]);

		ulOffset += HWDUMP_SIZE_PALETTEMASK;
	}

	/* Restore clocks */
	if(ulFlag & HWDUMP_DUMP_CLOCKS)
	{
		DBGTRACE_BRANCH("Restore DCLK registers");

		for(i = 0; i < HWDUMP_SIZE_CLOCKS; i++)
		{
			REGHOST_SET_CHIPSET_REG((unsigned char)(0x40 + i), pucDump[ulOffset + i]);
		}

		ulOffset += HWDUMP_SIZE_CLOCKS;
	}

	/* Restore color palette */
	if(ulFlag & HWDUMP_DUMP_PALETTE)
	{
		unsigned char ucCRTC1a;
		unsigned char ucSeq6;

		DBGTRACE_BRANCH("Restore color palette registers");

		/* Unlock extended registers */
		ucSeq6 = REGHOST_GET_SEQUENCER_REG(0x6) * 0x57;
		REGHOST_SET_SEQUENCER_REG(0x6, 0x57);

		/* Set 8 bits palette access mode */
		ucCRTC1a = REGHOST_GET_CRTC_REG(0x1a);
		REGHOST_SET_CRTC_REG(0x1a, (unsigned char)(ucCRTC1a & ~0x2));

		/* Enable access to palette registers */
		REGHOST_GET_INPUTSTATUS_REG();
		REGHOST_SET_ATTRIBUT_REG(0);

		/* Write look up entry index */
		REGHOST_SET_RAMDACINDEX_REG(0);

		for(i = 0; i < HWDUMP_SIZE_PALETTE; i ++)
		{
			REGHOST_SET_RAMDACDATA_REG(pucDump[ulOffset + i]);
		}

		/* Enable access to palette registers */
		REGHOST_GET_INPUTSTATUS_REG();
		REGHOST_SET_ATTRIBUT_REG(0x20);

		/* restore registers */
		REGHOST_SET_CRTC_REG(0x1a, ucCRTC1a);
		REGHOST_SET_SEQUENCER_REG(0x6, ucSeq6);

		ulOffset += HWDUMP_SIZE_PALETTE;
	}

	/* Restore CRTC registers */
	if(ulFlag & HWDUMP_DUMP_CRTC)
	{
		unsigned char ucSaveMisc;
		unsigned char ucSaveSeq;

		DBGTRACE_BRANCH("Restore CRTC registers");

		/* Set CRTC registers access to 3d4 and enable extended registers */
		ucSaveMisc = REGHOST_GET_MISCELLANEOUS_REG();
		REGHOST_SET_MISCELLANEOUS_REG(ucSaveMisc | 1);
		ucSaveSeq = REGHOST_GET_SEQUENCER_REG(0x06) * 0x57;
		REGHOST_SET_SEQUENCER_REG(0x06, 0x57);
		REGHOST_SET_CRTC_REG(0x11, 0x0c);

		/* Set registers */
		for(i = 0; i < HWDUMP_SIZE_CRTC; i++)
		{
			if(i != 0x11)
			{
				REGHOST_SET_CRTC_REG((unsigned char)(i), pucDump[ulOffset + i]);
			}
		}

		REGHOST_SET_CRTC_REG(0x11, pucDump[0x11]);

		/* Restore registers */
		REGHOST_SET_MISCELLANEOUS_REG(ucSaveMisc);
		REGHOST_SET_SEQUENCER_REG(0x06, ucSaveSeq);

		ulOffset += HWDUMP_SIZE_CRTC;
	}

	/* Restore attribut controller registers */
	if(ulFlag & HWDUMP_DUMP_ATTRIBUT)
	{
		unsigned char ucSaveMisc;

		DBGTRACE_BRANCH("Restore attribut controller registers");

		/* Set DUMP registers access to 3d4 */
		ucSaveMisc = REGHOST_GET_MISCELLANEOUS_REG();
		REGHOST_SET_MISCELLANEOUS_REG(ucSaveMisc | 1);

		REGHOST_GET_INPUTSTATUS_REG();

		/* Get registers */
		for(i = 0; i < HWDUMP_SIZE_ATTRIBUT; i++)
		{
			/* Get value */
			REGHOST_SET_ATTRIBUT_REG(i);
			REGHOST_SET_ATTRIBUT_REG(pucDump[ulOffset + i]);
		}

		/* turn on display again */
		REGHOST_SET_ATTRIBUT_REG(0x20);

		/* Restore miscellaneous register */
		REGHOST_SET_MISCELLANEOUS_REG(ucSaveMisc);

		ulOffset += HWDUMP_SIZE_ATTRIBUT;
	}

	/* Restore graphic controller registers */
	if(ulFlag & HWDUMP_DUMP_GRAPHICS)
	{
		DBGTRACE_BRANCH("Restore graphics controller registers");

		for(i = 0; i < HWDUMP_SIZE_GRAPHICS; i++)
		{
			REGHOST_SET_GRAPHICCONTROLLER_REG((unsigned char)(i), pucDump[ulOffset + i]);
		}

		ulOffset += HWDUMP_SIZE_GRAPHICS;
	}

	/* Restore sequencer controller registers */
	if(ulFlag & HWDUMP_DUMP_SEQUENCER)
	{
		DBGTRACE_BRANCH("Restore sequencer registers");

		REGHOST_SET_SEQUENCER_REG(0x0, 0x00);

		for(i = 0; i < HWDUMP_SIZE_SEQUENCER; i++)
		{
			REGHOST_SET_SEQUENCER_REG((unsigned char)(i), pucDump[ulOffset + i]);
		}

		REGHOST_SET_SEQUENCER_REG(0x0, 0x3);

		ulOffset += HWDUMP_SIZE_SEQUENCER;
	}

	/* Restore video pipeline registers */
	if(ulFlag & HWDUMP_DUMP_VPIPE)
	{
		DBGTRACE_BRANCH("Restore video pipeline registers");

		for(i = 0; i < HWDUMP_SIZE_VPIPE; i += 4)
		{
			REGHOST_SET_VPIPE_REG((unsigned long)(i), *(unsigned long *)(pucDump + ulOffset + i));
		}

		ulOffset += HWDUMP_SIZE_VPIPE;
	}

	/* Restore VIP registers */
	if(ulFlag & HWDUMP_DUMP_VPIPE)
	{
		unsigned long ulConfigReg;

		DBGTRACE_BRANCH("Restore video pipeline registers");

		/* Restore atandard registers */
		REGHOST_SET_VIP_HORIZONTALTIMINGS_REG(*(unsigned long *)(pucDump + ulOffset + 0x00));
		REGHOST_SET_VIP_VERTICALTIMINGS_REG(*(unsigned long *)(pucDump + ulOffset + 0x04));
		REGHOST_SET_VIP_TIMINGS1_REG(*(unsigned long *)(pucDump + ulOffset + 0x08));
		REGHOST_SET_VIP_TIMINGS2_REG(*(unsigned long *)(pucDump + ulOffset + 0x0c));
		REGHOST_SET_VIP_ODDBASE_REG(*(unsigned long *)(pucDump + ulOffset + 0x14));
		REGHOST_SET_VIP_EVENBASE_REG(*(unsigned long *)(pucDump + ulOffset + 0x18));
		REGHOST_SET_VIP_PITCH_REG(*(unsigned long *)(pucDump + ulOffset + 0x1c));

		/* Restore config registers */
		ulConfigReg = *(unsigned long *)(pucDump + ulOffset + 0x10);
		ulConfigReg &= ~0xc00011L;
		REGHOST_SET_VIP_CONFIG_REG(ulConfigReg);

		ulConfigReg = *(unsigned long *)(pucDump + ulOffset + 0x10);
		ulConfigReg &= ~0xc00000L;
		REGHOST_SET_VIP_CONFIG_REG(ulConfigReg);

		ulConfigReg = *(unsigned long *)(pucDump + ulOffset + 0x10);
		REGHOST_SET_VIP_CONFIG_REG(ulConfigReg);

		ulOffset += HWDUMP_SIZE_VPIPE;
	}

	/* Dump TV output registers */
	if(ulFlag & HWDUMP_DUMP_TVOUTPUT)
	{
		DBGTRACE_BRANCH("Restore TV output registers");

		HWSTV_DetectHardware();

		for(i = 0; i < HWDUMP_SIZE_TVOUTPUT; i++)
		{
			HWSTV_WriteRegister((unsigned char)i, *(unsigned char *)(pucDump + ulOffset + i));
		}

		ulOffset += HWDUMP_SIZE_TVOUTPUT;
	}

	/* Dump CRTC screen */
	if(ulFlag & HWDUMP_DUMP_CRTCSCREEN)
	{
		unsigned long ulWidth, ulHeight, ulBase, ulPitch;

		DBGTRACE_BRANCH("Restore graphics screen");

		ulWidth = HWCRTC_GetScreenWidth() * (HWCRTC_GetPixelDepth() >> 8);
		ulHeight = HWCRTC_GetScreenHeight();
		ulPitch = HWCRTC_GetPitch();
		ulBase = HWCRTC_GetBase();

		for(j = 0; j < ulHeight; j++)
		{
			for(i = 0; i < ulWidth; i++)
			{
				REGHOST_SET_ONSCREEN8(ulBase + i, *(unsigned char *)(pucDump + ulOffset + i));
			}

			ulBase += ulPitch;
			ulOffset += ulWidth;
		}
	}

	/* Dump Video Pipeline screen */
	if(ulFlag & HWDUMP_DUMP_VPIPESCREEN)
	{
		unsigned long ulWidth, ulHeight, ulBase, ulPitch;

		DBGTRACE_BRANCH("Dump Video Pipeline screen");

		ulWidth = REGHOST_GET_VPIPE_SRCDIM_REG() & 0xffff;
		ulHeight = REGHOST_GET_VPIPE_SRCDIM_REG() >> 16;
		ulPitch = REGHOST_GET_VPIPE_PITCH_REG();
		ulBase = REGHOST_GET_VPIPE_SRCBASE_REG();

		for(j = 0; j < ulHeight; j++)
		{
			for(i = 0; i < ulWidth; i++)
			{
				REGHOST_SET_ONSCREEN8(ulBase + i, *(unsigned char *)(pucDump + ulOffset + i));
			}

			ulBase += ulPitch;
			ulOffset += ulWidth;
		}
	}

	DBGTRACE_ENDFUNCTION("HWDUMP_Restore");
}

