/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: dump.h
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libhard\dump
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE:		: July, 18th 2000
// AUTHOR:		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Functions to dump registers
// ------------------------------------------------------------
// VARIABLES	: -
// ------------------------------------------------------------
// MACROS		: HWDUMP_DUMP_... 
// ------------------------------------------------------------
// FUNCTIONS	: HWDUMP_GetDumpSize
//				  HWDUMP_DumpGraphicMode
//				  HWDUMP_RestoreGraphicMode
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: July, 18th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Add possibility to dump Video Pieline and
//				  Video Input Port registers
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE			: March, 27th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

#ifndef DUMP_H
#define DUMP_H

/* ------------------------------------------------------------
// DEFINE		: HWDUMP_DUMP_...
// ------------------------------------------------------------
// DESCRIPTION	: Registers to dump
// ------------------------------------------------------------
// HWDUMP_DUMP_ALL				: All graphics and video registers
// HWDUMP_DUMP_ALLGRAPHICS		: All graphics registers
// HWDUMP_DUMP_MISCELLANEOUS	: Dump miscellanous register
// HWDUMP_DUMP_PALETTEMASK		: Dump palette mask register
// HWDUMP_DUMP_CLOCKS			: Dump clocks registers
// HWDUMP_DUMP_PALETTE			: Dump color palette registers
// HWDUMP_DUMP_CRTC				: Dump CRTC registers
// HWDUMP_DUMP_ATTRIBUT			: Dump attribut controller registers
// HWDUMP_DUMP_GRAPHICS			: Dump graphics controller registers
// HWDUMP_DUMP_SEQUENCER		: Sump sequencer register
// ------------------------------------------------------------*/
#define HWDUMP_DUMP_ALL					0xffffffff
#define HWDUMP_DUMP_ALLGRAPHICS			0x000000ff
#define HWDUMP_DUMP_MISCELLANEOUS		0x00000001
#define HWDUMP_DUMP_PALETTEMASK			0x00000002
#define HWDUMP_DUMP_CLOCKS				0x00000004
#define HWDUMP_DUMP_PALETTE				0x00000008
#define HWDUMP_DUMP_CRTC				0x00000010
#define HWDUMP_DUMP_ATTRIBUT			0x00000020
#define HWDUMP_DUMP_GRAPHICS			0x00000040
#define HWDUMP_DUMP_SEQUENCER			0x00000080
#define HWDUMP_DUMP_VPIPE				0x00010000
#define HWDUMP_DUMP_VIP					0x00020000
#define HWDUMP_DUMP_TVOUTPUT			0x00040000
#define HWDUMP_DUMP_CRTCSCREEN			0x01000000
#define HWDUMP_DUMP_VPIPESCREEN			0x02000000

/* ------------------------------------------------------------
// DEFINE		: HWDUMP_SIZE_...
// ------------------------------------------------------------
// DESCRIPTION	: Size in bytes of registers to dump
// ------------------------------------------------------------
// HWDUMP_SIZE_ALL				: All graphics and video registers
// HWDUMP_SIZE_ALLGRAPHICS		: All graphics registers
// HWDUMP_SIZE_MISCELLANEOUS	: Dump miscellanous register
// HWDUMP_SIZE_PALETTEMASK		: Dump palette mask register
// HWDUMP_SIZE_CLOCKS			: Dump clocks registers
// HWDUMP_SIZE_PALETTE			: Dump color palette registers
// HWDUMP_SIZE_CRTC				: Dump CRTC registers
// HWDUMP_SIZE_ATTRIBUT			: Dump attribut controller registers
// HWDUMP_SIZE_GRAPHICS			: Dump graphics controller registers
// HWDUMP_SIZE_SEQUENCER		: Sump sequencer register
// ------------------------------------------------------------*/
#define HWDUMP_SIZE_MISCELLANEOUS		1
#define HWDUMP_SIZE_PALETTEMASK			1
#define HWDUMP_SIZE_CLOCKS				8
#define HWDUMP_SIZE_PALETTE				(256*3)
#define HWDUMP_SIZE_CRTC				0x45
#define HWDUMP_SIZE_ATTRIBUT			0x15
#define HWDUMP_SIZE_GRAPHICS            9
#define HWDUMP_SIZE_SEQUENCER			5
#define HWDUMP_SIZE_VPIPE				0x44
#define HWDUMP_SIZE_VIP					32
#define HWDUMP_SIZE_TVOUTPUT			9

/* ------------------------------------------------------------
// FUNCTIONS	: HWDUMP_GetDumpSize
// ------------------------------------------------------------
// DESCRIPTION	: Get the size o memory necessary for a dump
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulFlag		: Describe the registers to dump
//				  (HWDUMP_DUMP_ALL or a combination of
//				  HWDUMP_DUMP_... values)
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWDUMP_GetDumpSize(unsigned long ulFlag);

/* ------------------------------------------------------------
// FUNCTIONS	: HWDUMP_Dump
// ------------------------------------------------------------
// DESCRIPTION	: Dump registers
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulFlag		: Describe the registers to dump
//				  (HWDUMP_DUMP_ALL or a combination of
//				  HWDUMP_DUMP_... values)
// pucDump		: Block of memory to store registers values
//				  The size of this block can be return by the
//				  HWDUMP_GetDumpSize function
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWDUMP_Dump(unsigned long ulFlag, unsigned char *pucDump);

/* ------------------------------------------------------------
// FUNCTIONS	: HWDUMP_Restore
// ------------------------------------------------------------
// DESCRIPTION	: Restore registers
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// pucDump		: Block of memory with registers values
//				  This block is filled by the HWDUMP_Dump
//				  register.
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWDUMP_Restore(unsigned char *pucDump);

#endif
