/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: framebuf.h
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libhard\framebuf
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: October, 20th 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Functions to manage frame buffer memory
//				  (memory allocations...)
// ------------------------------------------------------------
// VARIABLES	: -
// ------------------------------------------------------------
// MACROS		: HWFRAMEBUF_NODE
//				  HWFRAMEBUF_SURFACE
//				  HWFRAMEBUF_ALLOC_...
// ------------------------------------------------------------
// FUNCTIONS	: HWFRAMEBUF_Init
//				  HWFRAMEBUF_GetFrameBufferSize
//				  HWFRAMEBUF_Alloc
//				  HWFRAMEBUF_Free
//				  HWFRAMEBUF_GESurfaceAlloc
//				  HWFRAMEBUF_PatternAlloc
//				  HWFRAMEBUF_HWCursorAlloc
//				  HWFRAMEBUF_ClearFrameBuffer
//				  HWFRAMEBUF_Load8bppPattern
//				  HWFRAMEBUF_Load16bppPattern
//				  HWFRAMEBUF_Load24bppPattern
//				  HWFRAMEBUF_Load8bppConstantPattern
//				  HWFRAMEBUF_Load16bppConstantPattern
//				  HWFRAMEBUF_Load24bppConstantPattern
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: October, 20th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Added functions to load patterns into frame buffer
//				  HWFRAMEBUF_Load...Pattern
// ------------------------------------------------------------
// REVISION		: Rev 1.0
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

#ifndef FRAMEBUF_H
#define FRAMEBUF_H

/* ------------------------------------------------------------
// DEFINE		: HWFRAMEBUF_ALLOC_... 
// ------------------------------------------------------------
// DESCRIPTION	: How to align an allocation inside the frame 
//				  buffer
// ------------------------------------------------------------
// HWFRAMEBUF_ALLOC_BYTEALIGN	: Align on byte (no alignment)
// HWFRAMEBUF_ALLOC_WORDALIGN	: Align on 16 bits
// HWFRAMEBUF_ALLOC_DWORDALIGN	: Align on 32 bits
// HWFRAMEBUF_ALLOC_QWORDALIGN	: Align on 64 bits
// ------------------------------------------------------------*/
#define HWFRAMEBUF_ALLOC_BYTEALIGN		1
#define HWFRAMEBUF_ALLOC_WORDALIGN		2
#define HWFRAMEBUF_ALLOC_DWORDALIGN		4
#define HWFRAMEBUF_ALLOC_QWORDALIGN		8

/* ------------------------------------------------------------
// STRUCTURE	: HWFRAMEBUF_SURFACE 
// ------------------------------------------------------------
// DESCRIPTION	: 2D rectangular surface inside the frame 
//				  buffer
// ------------------------------------------------------------
// ulBase		: Base offset in the frame buffer
// ulWidth		: Width of the surface
// ulHeight		: Height of the surface
// ulPitch		: Number of bytes between two lines of the 
//				  surface
// ulBpp		: Number of bits for 1 pixel of the surface
// ------------------------------------------------------------*/
typedef struct tagHWFRAMEBUF_SURFACE
{
	unsigned long ulBase;
	unsigned long ulWidth;
	unsigned long ulHeight;
	unsigned long ulPitch;
	unsigned long ulBpp;
}HWFRAMEBUF_SURFACE;

/* ------------------------------------------------------------
// STRUCTURE	: HWFRAMEBUF_NODE 
// ------------------------------------------------------------
// DESCRIPTION	: Node for the block allocation list
// ------------------------------------------------------------
// ulHandle		: Handle of the memory allocated to store
//				  this node informations 
// pNext		: Pointer on the next node in the list
//				  NULL if last node of the list
// ulBase		: Base address of the allocated block
// ulSize		: Size of the allocated block in bytes
// ------------------------------------------------------------*/
typedef struct tagHWFRAMEBUF_NODE
{
	unsigned long ulHandle;
	struct tagHWFRAMEBUF_NODE *pNext;
	unsigned long ulBase;
	unsigned long ulSize;
}HWFRAMEBUF_NODE;

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_Init
// ------------------------------------------------------------
// DESCRIPTION	: Initialize the library
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBeginFree	: Offset of the frame buffer block used for
//				  memory allocation by this library
//				  (0 if all frame buffer is used)
// ulSizeFree	: Size of the frame buffer block used for
//				  memory allocation by this library
//				  (0xffffffff to use a block from ulBeginFree
//				  to the end of the frame buffer)
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: HWFRAMEBUF_psFirstNode
//				  HWFRAMEBUF_ulFBSize
// ------------------------------------------------------------*/
void HWFRAMEBUF_Init(unsigned long ulBeginFree, unsigned long ulSizeFree);

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_GetFrameBufferSize
// ------------------------------------------------------------
// DESCRIPTION	: Return the size of the frame buffer in bytes
//				  (between 128Kb and 4Mb)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWFRAMEBUF_GetFrameBufferSize(void);

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_Alloc
// ------------------------------------------------------------
// DESCRIPTION	: Alloc a block in the frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulSize		: Size of the block to allocate
// ulAlign		: Number of byte of alignment
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Offset of the allocate block in the frame
//				  buffer
//				  Return 0xffffffff if no more free space
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWFRAMEBUF_Alloc(unsigned long ulSize, unsigned long ulAlign);

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_Free
// ------------------------------------------------------------
// DESCRIPTION	: Free a block allocated in the frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBase		: Base address of the block to free
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWFRAMEBUF_Free(unsigned long ulBase);

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_GESurfaceAlloc
// ------------------------------------------------------------
// DESCRIPTION	: Alloc a 2D surface compatible with
//				  alignment and pitch of the Graphic Engine 
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// psSurface	: Size and pixel format of the surface to
//				  allocate, the pitch can be set to 0
//				  to be automaticly selected or filled
//				  to the required minimum value (will be
//				  ajusted to reach the Graphic Engine
//				  specification)
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// psSurface	: Fill the structure with the base address
//				  and a pitch compatible with the Graphic
//				  engine
//				  Base address = 0xffffffff if no more space
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWFRAMEBUF_GESurfaceAlloc(HWFRAMEBUF_SURFACE *psSurface);

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_PatternAlloc
// ------------------------------------------------------------
// DESCRIPTION	: Alloc space for a Graphic Engine pattern
//				  in the frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Offset in the frame buffer of the allocated
//				  space
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWFRAMEBUF_PatternAlloc(void);

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_HWCursorAlloc
// ------------------------------------------------------------
// DESCRIPTION	: Alloc space for a Graphic Engine hardware
//				  cursor in the frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Offset in the frame buffer of the allocated
//				  space
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWFRAMEBUF_HWCursorAlloc(void);

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_ClearFrameBuffer
// ------------------------------------------------------------
// DESCRIPTION	: Clear the whole frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWFRAMEBUF_ClearFrameBuffer(void);

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_Load8bppPattern
// ------------------------------------------------------------
// DESCRIPTION	: Load a pattern into frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBase		: Offset in frame buffer to set the pattern
// pPattern		: Pattern pixels (same format as the screen)
//				  8 by 8 pixels
// ulPitch		: Number if bytes between two lines
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWFRAMEBUF_Load8bppPattern(unsigned long ulBase, unsigned char *pPattern, unsigned long ulPitch);

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_Load16bppPattern
// ------------------------------------------------------------
// DESCRIPTION	: Load a pattern into frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBase		: Offset in frame buffer to set the pattern
// pPattern		: Pattern pixels (same format as the screen)
//				  8 by 8 pixels
// ulPitch		: Number if bytes between two lines
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWFRAMEBUF_Load16bppPattern(unsigned long ulBase, unsigned char *pPattern, unsigned long ulPitch);

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_Load24bppPattern
// ------------------------------------------------------------
// DESCRIPTION	: Load a pattern into frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBase		: Offset in frame buffer to set the pattern
// pPattern		: Pattern pixels (same format as the screen)
//				  8 by 8 pixels
// ulPitch		: Number if bytes between two lines
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWFRAMEBUF_Load24bppPattern(unsigned long ulBase, unsigned char *pPattern, unsigned long ulPitch);

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_Load8bppConstantPattern
// ------------------------------------------------------------
// DESCRIPTION	: Load a constant pattern into frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBase		: Offset in frame buffer to set the pattern
// ulColor		: Constant color to set
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWFRAMEBUF_Load8bppConstantPattern(unsigned long ulBase, unsigned long ulColor);

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_Load16bppConstantPattern
// ------------------------------------------------------------
// DESCRIPTION	: Load a constant pattern into frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBase		: Offset in frame buffer to set the pattern
// ulColor		: Constant color to set
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWFRAMEBUF_Load16bppConstantPattern(unsigned long ulBase, unsigned long ulColor);

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_Load24bppConstantPattern
// ------------------------------------------------------------
// DESCRIPTION	: Load a constant pattern into frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBase		: Offset in frame buffer to set the pattern
// ulColor		: Constant color to set
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWFRAMEBUF_Load24bppConstantPattern(unsigned long ulBase, unsigned long ulColor);

#endif

