/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: ge.h
// LIBRARY		: STPC Graphic Development Kit
// MODULE		: libhard\ge
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.3
// DATE			: October, 18th 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Functions to manage graphic 2D acceleration
// ------------------------------------------------------------
// VARIABLES	: HWGE_pulPitchConversion
//				  HWGE_pulROPConversion
// ------------------------------------------------------------
// MACROS		: HWGE_ROP_HAS_... 
// 				  HWGE_ROPREG_... 
// 				  HWGE_ROP_... 
// 				  HWGE_FLAG_... 
// 				  HWGE_WAIT_NOT_PENDING 
// 				  HWGE_WAIT_NOT_BUSY 
// 				  HWGE_SETPATTERN
//				  HWGE_3BPP_SETPATTERN
// 				  HWGE_SET_SOURCE_SURFACE
//				  HWGE_SET_DESTINATION_SURFACE
// 				  HWGE_PREPARE_FILL
//				  HWGE_PREPARE_SCREEN2SCREEN
//				  HWGE_PREPARE_HOST2SCREEN
// 				  HWGE_?BPP_SRC_COLORKEY
//				  HWGE_?BPP_DST_COLORKEY
// 				  HWGE_SET_FILL_COLOR
// 				  HWGE_SET_TEXT_COLOR
// 				  HWGE_SET_TRANSPTEXT_COLOR
// 				  HWGE_?BPP_FILL_START
// 				  HWGE_?BPP_OFFS2S_START
// 				  HWGE_?BPP_INVOFFS2S_START
// 				  HWGE_?BPP_S2S_START
// 				  HWGE_?BPP_HLINE_START
// 				  HWGE_?BPP_VLINE_START
// 				  HWGE_?BPP_H2S_START
// 				  HWGE_?BPP_SPECIALH2S_START
// 				  HWGE_?BPP_MONOH2S_START
// 				  HWGE_?BPP_RIGHTEXTEND_START
// 				  HWGE_?BPP_BOTTOMEXTEND_START
// 				  HWGE_?BPP_EXTEND_START
// ------------------------------------------------------------
// FUNCTIONS	: HWGE_Init
//				  HWGE_GetNearestPitch
//				  HWGE_GetPitchRegValue
//				  HWGE_SendDataToDataPort
//				  HWGE_SendMonoDataToDataPort
//				  HWGE_IsPending
//				  HWGE_IsBusy
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.3
// DATE			: October, 18th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Add the suppression of pattern in the ROP,
//				  using a precomputed table built in HWGE_Init
//				  Macro HWGE_ROP_SUPPRESSPATTERN and
//				  HWGE_ROP_OTHERSUPPRESSPATTERN
// ------------------------------------------------------------
// REVISION		: Rev 1.2
// DATE			: August, 25th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Add fast packed mode for host-to-screen
//				  HWGE_?BPP_PACKED...
// ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: July, 18th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Few bugs suppressed in macros :
//				  Set to 0 SrcXY registers in 
//				  HWGE_?BPP_SPECIALH2S_START macros
//				  Add a -1 in width and height registers of
//				  HWGE_?BPP_S2S_START macros when the direction
//				  is inverted
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

#ifndef GE_H
#define GE_H

#include "host.h"

/* ------------------------------------------------------------
// STRUCTURE	: HWGE_REGISTERS 
// ------------------------------------------------------------
// DESCRIPTION	: Temporary values for shadowed registers
// ------------------------------------------------------------*/
typedef struct tagHWGE_REGISTERS
{
	unsigned long ulPattern;
	unsigned long ulSrcBase;
	unsigned long ulSrcPitch;
	unsigned long ulDstBase;
	unsigned long ulDstPitch;
	unsigned long ulBpp;
	unsigned long ulROP;
	unsigned long ulSrcKey;
	unsigned long ulDstKey;
	unsigned long ulBkColor;
	unsigned long ulFgColor;
	unsigned long ulSrcXY;
	unsigned long ulWidth;
	unsigned long ulHeight;
}HWGE_REGISTERS;
extern		HWGE_REGISTERS HWGE_sRegisters;

/* ------------------------------------------------------------
// DEFINE		: HWGE_LOAD_REGISTERS
// ------------------------------------------------------------
// DESCRIPTION	: Function to load GE registers with shadow value
// ------------------------------------------------------------*/
#define HWGE_LOAD_REGISTERS(reg) {					\
	REGHOST_SET_GE_PATTERN_REG(reg.ulPattern);		\
	REGHOST_SET_GE_SRCBASE_REG(reg.ulSrcBase);		\
	REGHOST_SET_GE_SRCPITCH_REG(reg.ulSrcPitch);	\
	REGHOST_SET_GE_DSTBASE_REG(reg.ulDstBase);		\
	REGHOST_SET_GE_DSTPITCH_REG(reg.ulDstPitch);	\
	REGHOST_SET_GE_BPP_REG(reg.ulBpp);				\
	REGHOST_SET_GE_ROP_REG(reg.ulROP);				\
	REGHOST_SET_GE_SRCKEY_REG(reg.ulSrcKey);		\
	REGHOST_SET_GE_DSTKEY_REG(reg.ulDstKey);		\
	REGHOST_SET_GE_BKCOLOR_REG(reg.ulBkColor);		\
	REGHOST_SET_GE_FGCOLOR_REG(reg.ulFgColor);		\
	REGHOST_SET_GE_SRCXY_REG(reg.ulSrcXY);			\
	REGHOST_SET_GE_WIDTH_REG(reg.ulWidth);			\
	REGHOST_SET_GE_HEIGHT_REG(reg.ulHeight);		\
}

/* ------------------------------------------------------------
// DEFINE		: HWGE_SET_...
//				  HWGE_GET_... 
// ------------------------------------------------------------
// DESCRIPTION	: Function to access GE registers
// ------------------------------------------------------------*/
#define		HWGE_GET_STATUS_REG					REGHOST_GET_GE_STATUS_REG
#define		HWGE_GET_EXTRA_REG					REGHOST_GET_GE_EXTRA_REG
#define		HWGE_SET_CURSORXY_REG(Value)		REGHOST_SET_GE_CURSORXY_REG(Value)
#define		HWGE_SET_EXTRA_REG(Value)			REGHOST_SET_GE_EXTRA_REG(Value)
#define		HWGE_SET_DATAPORT_REG(Value)		REGHOST_SET_GE_DATAPORT_REG(Value)

/* ------------------------------------------------------------
// DEFINE		: HWGE_SET_...
//				  HWGE_GET_... 
// ------------------------------------------------------------
// DESCRIPTION	: Function to access GE registers in direct access mode
// ------------------------------------------------------------*/
#ifndef		HWGE_SHADOW_REGISTERS

#define		HWGE_SET_PATTERN_REG(Value)			REGHOST_SET_GE_PATTERN_REG(Value)
#define		HWGE_SET_SRCBASE_REG(Value)			REGHOST_SET_GE_SRCBASE_REG(Value)
#define		HWGE_SET_SRCPITCH_REG(Value)		REGHOST_SET_GE_SRCPITCH_REG(Value)
#define		HWGE_SET_DSTBASE_REG(Value)			REGHOST_SET_GE_DSTBASE_REG(Value)
#define		HWGE_SET_DSTPITCH_REG(Value)		REGHOST_SET_GE_DSTPITCH_REG(Value)
#define		HWGE_SET_BPP_REG(Value)				REGHOST_SET_GE_BPP_REG(Value)
#define		HWGE_SET_ROP_REG(Value)				REGHOST_SET_GE_ROP_REG(Value)
#define		HWGE_SET_SRCKEY_REG(Value)			REGHOST_SET_GE_SRCKEY_REG(Value)
#define		HWGE_SET_DSTKEY_REG(Value)			REGHOST_SET_GE_DSTKEY_REG(Value)
#define		HWGE_SET_BKCOLOR_REG(Value)			REGHOST_SET_GE_BKCOLOR_REG(Value)
#define		HWGE_SET_FGCOLOR_REG(Value)			REGHOST_SET_GE_FGCOLOR_REG(Value)
#define		HWGE_SET_SRCXY_REG(Value)			REGHOST_SET_GE_SRCXY_REG(Value)
#define		HWGE_SET_WIDTH_REG(Value)			REGHOST_SET_GE_WIDTH_REG(Value)
#define		HWGE_SET_HEIGHT_REG(Value)			REGHOST_SET_GE_HEIGHT_REG(Value)

#define		HWGE_SET_DSTXY_REG(Value)			REGHOST_SET_GE_DSTXY_REG(Value)
#define		HWGE_SET_DUMMYDSTXY_REG(Value)		REGHOST_SET_GE_DUMMYDSTXY_REG(Value)
#define		HWGE_SET_1WIDTHDSTXY_REG(Value)		REGHOST_SET_GE_1WIDTHDSTXY_REG(Value)
#define		HWGE_SET_2WIDTHDSTXY_REG(Value)		REGHOST_SET_GE_2WIDTHDSTXY_REG(Value)
#define		HWGE_SET_3WIDTHDSTXY_REG(Value)		REGHOST_SET_GE_3WIDTHDSTXY_REG(Value)
#define		HWGE_SET_4WIDTHDSTXY_REG(Value)		REGHOST_SET_GE_4WIDTHDSTXY_REG(Value)
#define		HWGE_SET_1HEIGHTDSTXY_REG(Value)	REGHOST_SET_GE_1HEIGHTDSTXY_REG(Value)

#endif

/* ------------------------------------------------------------
// DEFINE		: HWGE_SET_...
//				  HWGE_GET_... 
// ------------------------------------------------------------
// DESCRIPTION	: Function to access GE registers in shadow mode
// ------------------------------------------------------------*/
#ifdef		HWGE_SHADOW_REGISTERS

#define		HWGE_SET_PATTERN_REG(ulValue)		{ HWGE_sRegisters.ulPattern  = ulValue; };
#define		HWGE_SET_SRCBASE_REG(ulValue)		{ HWGE_sRegisters.ulSrcBase  = ulValue; };
#define		HWGE_SET_SRCPITCH_REG(ulValue)		{ HWGE_sRegisters.ulSrcPitch = ulValue; };
#define		HWGE_SET_DSTBASE_REG(ulValue)		{ HWGE_sRegisters.ulDstBase  = ulValue; };
#define		HWGE_SET_DSTPITCH_REG(ulValue)		{ HWGE_sRegisters.ulDstPitch = ulValue; };
#define		HWGE_SET_BPP_REG(ulValue)			{ HWGE_sRegisters.ulBpp      = ulValue; };
#define		HWGE_SET_ROP_REG(ulValue)			{ HWGE_sRegisters.ulROP      = ulValue; };
#define		HWGE_SET_SRCKEY_REG(ulValue)		{ HWGE_sRegisters.ulSrcKey   = ulValue; };
#define		HWGE_SET_DSTKEY_REG(ulValue)		{ HWGE_sRegisters.ulDstKey   = ulValue; };
#define		HWGE_SET_BKCOLOR_REG(ulValue)		{ HWGE_sRegisters.ulBkColor  = ulValue; };
#define		HWGE_SET_FGCOLOR_REG(ulValue)		{ HWGE_sRegisters.ulFgColor  = ulValue; };
#define		HWGE_SET_SRCXY_REG(ulValue)			{ HWGE_sRegisters.ulSrcXY    = ulValue; };
#define		HWGE_SET_WIDTH_REG(ulValue)			{ HWGE_sRegisters.ulWidth    = ulValue; };
#define		HWGE_SET_HEIGHT_REG(ulValue)		{ HWGE_sRegisters.ulHeight   = ulValue; };

#define		HWGE_SET_DSTXY_REG(Value)			{	\
	OSAL_ENTER_CRITICAL_SECTION;					\
	HWGE_LOAD_REGISTERS(HWGE_sRegisters);			\
	REGHOST_SET_GE_DSTXY_REG(Value);				\
	OSAL_LEAVE_CRITICAL_SECTION;					\
}
#define		HWGE_SET_DUMMYDSTXY_REG(Value)		{	\
	OSAL_ENTER_CRITICAL_SECTION;					\
	HWGE_LOAD_REGISTERS(HWGE_sRegisters);			\
	REGHOST_SET_GE_DUMMYDSTXY_REG(Value);			\
	OSAL_LEAVE_CRITICAL_SECTION;					\
}
#define		HWGE_SET_1WIDTHDSTXY_REG(Value)		{	\
	OSAL_ENTER_CRITICAL_SECTION;					\
	HWGE_LOAD_REGISTERS(HWGE_sRegisters);			\
	REGHOST_SET_GE_1WIDTHDSTXY_REG(Value);			\
	OSAL_LEAVE_CRITICAL_SECTION;					\
}
#define		HWGE_SET_2WIDTHDSTXY_REG(Value)		{	\
	OSAL_ENTER_CRITICAL_SECTION;					\
	HWGE_LOAD_REGISTERS(HWGE_sRegisters);			\
	REGHOST_SET_GE_2WIDTHDSTXY_REG(Value);			\
	OSAL_LEAVE_CRITICAL_SECTION;					\
}
#define		HWGE_SET_3WIDTHDSTXY_REG(Value)		{	\
	OSAL_ENTER_CRITICAL_SECTION;					\
	HWGE_LOAD_REGISTERS(HWGE_sRegisters);			\
	REGHOST_SET_GE_3WIDTHDSTXY_REG(Value);			\
	OSAL_LEAVE_CRITICAL_SECTION;					\
}
#define		HWGE_SET_4WIDTHDSTXY_REG(Value)		{	\
	OSAL_ENTER_CRITICAL_SECTION;					\
	HWGE_LOAD_REGISTERS(HWGE_sRegisters);			\
	REGHOST_SET_GE_4WIDTHDSTXY_REG(Value);			\
	OSAL_LEAVE_CRITICAL_SECTION;					\
}
#define		HWGE_SET_1HEIGHTDSTXY_REG(Value)	{	\
	OSAL_ENTER_CRITICAL_SECTION;					\
	HWGE_LOAD_REGISTERS(HWGE_sRegisters);			\
	REGHOST_SET_GE_1HEIGHTDSTXY_REG(Value);			\
	OSAL_LEAVE_CRITICAL_SECTION;					\
}

#endif

/* ------------------------------------------------------------
// DEFINE		: HWGE_ROP_HAS_... 
// ------------------------------------------------------------
// DESCRIPTION	: Function to analyse the sources used by
//				  a raster operation
// ------------------------------------------------------------
//  HWGE_ROP_HAS_SRC	: True if rop use a source image
//  HWGE_ROP_HAS_PAT	: True if rop use a pattern
//  HWGE_ROP_HAS_DST	: True if rop use the destination 
// ------------------------------------------------------------*/
#define HWGE_ROP_HAS_SRC(rop) (0 != (0x33 & ( (int)rop ^ ((int)rop >> 2))))
#define HWGE_ROP_HAS_PAT(rop) (0 != (0x0F & ( (int)rop ^ ((int)rop >> 4))))
#define HWGE_ROP_HAS_DST(rop) (0 != (0x55 & ( (int)rop ^ ((int)rop >> 1))))

/* ------------------------------------------------------------
// DEFINE		: HWGE_ROPREG_... 
// ------------------------------------------------------------
// DESCRIPTION	: Flags in the Graphic Engine ROP register
//				  used to describe the different sources
//				  of the raster operation
// ------------------------------------------------------------
// HWGE_ROPREG_SRCSCRN	: Operation use a source located in
//						  the frame buffer
// HWGE_ROPREG_SRCHOST	: Operation use a source from the 
//						  Data Port FIFO
// HWGE_ROPREG_PAT		: Operation use a pattern
// HWGE_ROPREG_DST		: Operation use the destination pixel
// ------------------------------------------------------------*/
#define HWGE_ROPREG_SRCSCRN		0x40000000
#define HWGE_ROPREG_SRCHOST		0x80000000
#define HWGE_ROPREG_PAT			0x20000000
#define HWGE_ROPREG_DST			0x10000000

/* ------------------------------------------------------------
// DEFINE		: HWGE_ROP_SUPPRESSPATTERN
// ------------------------------------------------------------
// DESCRIPTION	: Suppress the pattern from the ROP
// ------------------------------------------------------------*/
#define HWGE_ROP_SUPPRESSPATTERN(rop)		(HWGE_pucROPSuppressPattern[rop])

/* ------------------------------------------------------------
// DEFINE		: HWGE_ROP_... 
// ------------------------------------------------------------
// DESCRIPTION	: Raster operations (define the way to mix 
//				  the source image, the pattern and the 
//				  destination pixels)
// ------------------------------------------------------------
// HWGE_ROP_SRCCOPY		: Source direct copy
// HWGE_ROP_SRCPAINT	: Source OR Destination
// HWGE_ROP_SRCAND		: Source AND Destination
// HWGE_ROP_SRCINVERT	: Source XOR Destination
// HWGE_ROP_SRCERASE	: Source AND (NOT Destination)
// HWGE_ROP_NOTSRCCOPY	: Copy of NOT Source
// HWGE_ROP_NOTSRCERASE	: NOT Source AND NOT Destination
// HWGE_ROP_MERGECOPY	: Source AND Pattern
// HWGE_ROP_MERGEPAINT	: NOT Source OR Destination
// HWGE_ROP_PATCOPY		: Pattern copy
// HWGE_ROP_PATINVERT	: Pattern XOR Destination
// HWGE_ROP_DSTINVERT	: NOT Destination
// HWGE_ROP_BLACKNESS	: Set to 0 value
// HWGE_ROP_WHITENESS	: Set to 1 value
// HWGE_ROP_OTHER(rop)	: Any type of rop define by the rop
//						  value
// HWGE_ROP_OTHERSUPPRESSPATTERN(rop)
//						: Any type of rop define by the rop
//						  value, suppress the pattern from the ROP
// ------------------------------------------------------------*/
#define HWGE_ROP_SRCCOPY		(0x00cc)
#define HWGE_ROP_SRCPAINT		(0x00ee | HWGE_ROPREG_DST)
#define HWGE_ROP_SRCAND			(0x0088 | HWGE_ROPREG_DST)
#define HWGE_ROP_SRCINVERT		(0x0066 | HWGE_ROPREG_DST)
#define HWGE_ROP_SRCERASE		(0x0044 | HWGE_ROPREG_DST)
#define HWGE_ROP_NOTSRCCOPY		(0x0033)
#define HWGE_ROP_NOTSRCERASE	(0x0011 | HWGE_ROPREG_DST)
#define HWGE_ROP_MERGECOPY		(0x00c0 | HWGE_ROPREG_PAT)
#define HWGE_ROP_MERGEPAINT		(0x00bb | HWGE_ROPREG_DST)
#define HWGE_ROP_PATCOPY		(0x00f0 | HWGE_ROPREG_PAT)
#define HWGE_ROP_PATINVERT		(0x005a | HWGE_ROPREG_PAT)
#define HWGE_ROP_DSTINVERT		(0x0055 | HWGE_ROPREG_DST)
#define HWGE_ROP_BLACKNESS		(0x0000)
#define HWGE_ROP_WHITENESS		(0x00ff)
#define HWGE_ROP_OTHER(rop)		(HWGE_pulROPConversion[rop])
#define HWGE_ROP_OTHERSUPPRESSPATTERN(rop) (HWGE_pulROPConversion[HWGE_ROP_SUPPRESSPATTERN(rop)])

/* ------------------------------------------------------------
// DEFINE		: HWGE_FLAG_... 
// ------------------------------------------------------------
// DESCRIPTION	: Flags to describe the operation to do during
//				  the 2D transfer
// ------------------------------------------------------------
// HWGE_FLAG_EMPTY			: No specific operation
// HWGE_FLAG_MONO			: Do bitmap expansion
//							  HWGE_SET_TEXT_COLOR to set the
//							  colors
// HWGE_FLAG_MONOTRANSP		: Do transparent bitmap expansion
//							  HWGE_SET_TRANSPTEXT_COLOR to set
//							  the colors
// HWGE_FLAG_PACKED			  To be set when using 
//							  HWGE_?BPP_PACKEDMONOH2S_START
// HWGE_FLAG_PATTRANSP		: Do pattern transparency
//							  HWGE_SETPATTERN or 
//							  HWGE_3BPP_SETPATTERN to set the
//							  pattern
// HWGE_FLAG_SRCMATCH		: Do source match transparency
//							  HWGE_?BPP_SRC_COLORKEY to set 
//							  the transparency color
// HWGE_FLAG_SRCNOTMATCH	: Do source not match transparency
//							  HWGE_?BPP_SRC_COLORKEY to set 
//							  the transparency color
// HWGE_FLAG_DSTMATCH		: Do destination match transparency
//							  HWGE_?BPP_DST_COLORKEY to set 
//							  the transparency color
// HWGE_FLAG_DSTNOTMATCH	: Do destination not match 
//							  transparency
//							  HWGE_?BPP_DST_COLORKEY to set 
//							  the transparency color
// ------------------------------------------------------------*/
#define HWGE_FLAG_EMPTY			0x00000000
#define HWGE_FLAG_MONO			0x00000400
#define HWGE_FLAG_PACKED		0x00000800
#define HWGE_FLAG_MONOTRANSP	0x00000500
#define HWGE_FLAG_PATTRANSP		0x20000200
#define HWGE_FLAG_SRCMATCH		0x00003000
#define HWGE_FLAG_SRCNOTMATCH	0x00002000
#define HWGE_FLAG_DSTMATCH		0x1000c000
#define HWGE_FLAG_DSTNOTMATCH	0x10008000

/* ------------------------------------------------------------
// VARIABLE		: HWGE_VirtualDataPort 
// ------------------------------------------------------------
// DESCRIPTION	: Linear address of data port, used by the
//				  assembly language functions
// ------------------------------------------------------------*/
extern unsigned long HWGE_VirtualDataPort;

/* ------------------------------------------------------------
// VARIABLE		: HWGE_VirtualGERegister
// ------------------------------------------------------------
// DESCRIPTION	: Linear address of the Graphic Engine register,
//				  used by the assembly language functions
// ------------------------------------------------------------*/
extern unsigned long HWGE_VirtualGERegister;

/* ------------------------------------------------------------
// VARIABLE		: HWGE_pulPitchConversion 
// ------------------------------------------------------------
// DESCRIPTION	: Table of precomputed pitch register value,
//				  used to get the value to be set
//				  in Graphic Engine pitch register
//				  Offset in the table is the pitch value 
//				  devided by 32 
// ------------------------------------------------------------*/
extern unsigned long HWGE_pulPitchConversion[];

/* ------------------------------------------------------------
// VARIABLE		: HWGE_pulROPConversion 
// ------------------------------------------------------------
// DESCRIPTION	: Table of precomputed ROP register value,
//				  used to get the value to be set
//				  in Graphic Engine ROP register
//				  Offset in the table is the 8 bits ROP value 
// ------------------------------------------------------------*/
extern unsigned long HWGE_pulROPConversion[256];

/* ------------------------------------------------------------
// VARIABLE		: HWGE_pucROPSuppressPattern 
// ------------------------------------------------------------
// DESCRIPTION	: Table of precomputed ROP conversion to suppress
//				  the pattern source in a ROP
//				  To be used when the pattern is unified color
//				  for example
// ------------------------------------------------------------*/
extern unsigned char HWGE_pucROPSuppressPattern[256];

/* ------------------------------------------------------------
// FUNCTION		: HWGE_Init
// ------------------------------------------------------------
// DESCRIPTION	: Initialize graphic engine library
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: HWGE_pulPitchConversion
//				  HWGE_VirtualDataPort
//				  HWGE_VirtualGERegister
// ------------------------------------------------------------*/
void HWGE_Init(void);

/* ------------------------------------------------------------
// FUNCTION		: HWGE_GetNearestPitch
// ------------------------------------------------------------
// DESCRIPTION	: Get the nearest pitch supported by the
//				  Graphic Engine (pitch = number of bytes
//				  between two lines of an image)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulPitch		: Pitch value to process
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Nearest supported pitch
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWGE_GetNearestPitch(unsigned long ulPitch);

/* ------------------------------------------------------------
// FUNCTION		: HWGE_GetPitchRegValue
// ------------------------------------------------------------
// DESCRIPTION	: Get the nearest contend of Graphic Engine
//				  pitch register to set for a specific pitch
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulPitch		: Pitch value to process
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Value to set in the Graphic Engine pitch 
//				  register, 0 if the pitch is not supported
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWGE_GetPitchRegValue(unsigned long ulPitch);

/* ------------------------------------------------------------
// FUNCTION		: HWGE_SendDataToDataPort
// ------------------------------------------------------------
// DESCRIPTION	: Send the required data to the data port
//				  for a system memory to frame buffer copy
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulDstXY		: Y coordinate of the destination rectangle
//				  in the upper 16 bits and X coordinate in the
//				  lower 16 bis (coordinate in the destination
//				  surface)
// pData		: Linear pointer on source data in system memory
// ulPitch		: Number of bytes between two lines of the
//				  source datas
// ulGERegister : Linear offset on GE memory mapped registers
// ulDataPort   : Linear offset on Data Port memory range
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWGE_SendDataToDataPort(unsigned long ulDstXY, unsigned long pData, unsigned long ulPitch, unsigned long ulGERegister, unsigned long ulDataPort);

/* ------------------------------------------------------------
// FUNCTION		: HWGE_SendMonoDataToDataPort
// ------------------------------------------------------------
// DESCRIPTION	: Send the required data to the data port
//				  for a system memory to frame buffer copy with
//				  pixel format conversion from 1 bit per pixels
//				  to the destination format
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulDstXY		: Y coordinate of the destination rectangle
//				  in the upper 16 bits and X coordinate in the
//				  lower 16 bis (coordinate in the destination
//				  surface)
// pData		: Linear pointer on source data in system memory
// ulPitch		: Number of bytes between two lines of the
//				  source datas
// ulOffsetInBits: Offset in bits for first pixel, starting
//				  from pData
// ulPitchInBits: Number of bits between two lines of the
//				  source image
// ulWidthInBits: Width in number of bits
// ulGERegister : Linear offset on GE memory mapped registers
// ulDataPort   : Linear offset on Data Port memory range
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWGE_SendMonoDataToDataPort(unsigned long ulDstXY, unsigned long pData, unsigned long ulOffsetInBits, unsigned long ulPitchInBits, unsigned long ulWidthInBits, unsigned long ulGERegister, unsigned long ulDataPort);

/* ------------------------------------------------------------
// FUNCTION		: HWGE_IsPending
// ------------------------------------------------------------
// DESCRIPTION	: Check if Graphic Engine is pending busy
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: <> 0 if pending busy
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWGE_IsPending(void);

/* ------------------------------------------------------------
// FUNCTION		: HWGE_IsBusy
// ------------------------------------------------------------
// DESCRIPTION	: Check if Graphic Engine is busy
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: <> 0 if busy
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWGE_IsBusy(void);

/* ------------------------------------------------------------
// DEFINE		: HWGE_WAIT_NOT_PENDING 
// ------------------------------------------------------------
// DESCRIPTION	: Wait for free access to double buffered
//				  registers (x, y, width and height)
//				  In place of calling this functions,
//				  the software can access the concerned
//				  registers, the CPU will be hold, waiting
//				  for the access
// ------------------------------------------------------------*/
#define HWGE_WAIT_NOT_PENDING { \
	if(HWGE_GET_STATUS_REG & 0x40000000) \
	{ \
		while(HWGE_IsPending()); } \
	}

/* ------------------------------------------------------------
// DEFINE		: HWGE_WAIT_NOT_BUSY 
// ------------------------------------------------------------
// DESCRIPTION	: Wait for the end of Graphic Engine operation
//				  This functions have to be called before
//				  access to a none buffered register
//				  (only x, y, width and height register
//				  are double buffered) or the current operation
//				  will be affected by the registers 
//				  modifications
// ------------------------------------------------------------*/
#define HWGE_WAIT_NOT_BUSY    { \
	if(HWGE_GET_STATUS_REG & 0x80000000) \
	{ \
		while(HWGE_IsBusy()); } \
	}

/* ------------------------------------------------------------
// DEFINE		: HWGE_IS_BUSY 
// ------------------------------------------------------------
// DESCRIPTION	: Return 1 if GE is busy, 0 if not
// ------------------------------------------------------------*/
#define HWGE_IS_BUSY	( HWGE_GET_STATUS_REG & 0x80000000L )

/* ------------------------------------------------------------
// DEFINE		: HWGE_SETPATTERN
//				  HWGE_3BPP_SETPATTERN
// ------------------------------------------------------------
// DESCRIPTION	: Select a pattern for a pattern transparency,
//				  a pattern fill or a raster operation with
//				  pattern.
// ------------------------------------------------------------
// PatternOffset	: Offset of the pattern in the frame buffer
// DstX				: X coordinate of the destination rectangle
//					  used to center the pattern in 24 bits
//					  per pixel only
// ------------------------------------------------------------*/

/* Set the pattern in 8 bits or 16 bits per pixels format */
#define HWGE_SETPATTERN(PatternOffset) \
	{ HWGE_SET_PATTERN_REG(PatternOffset); }

/* Set the pattern in 24 bits per pixels format */
#define HWGE_3BPP_SETPATTERN(PatternOffset, DstX) \
	{ HWGE_SET_PATTERN_REG(PatternOffset + (((DstX >> 3) % 3) << 3)); }

/* ------------------------------------------------------------
// DEFINE		: HWGE_SET_SOURCE_SURFACE
//				  HWGE_SET_DESTINATION_SURFACE
// ------------------------------------------------------------
// DESCRIPTION	: Select a frame buffer surface for the source
//				  pixels and the destination pixels
//				  The destination pixel must always be valid but
//				  the source surface is not required for a 
//				  rectangle fill or a pattern fill (no source)
// ------------------------------------------------------------
// Base			: Offset of the surface in the frame buffer
// Pitch		: Number of bytes between two lines of the
//				  surface
// (The Graphic Engine doesn't manage clipping and the width and
//  height of the surface is not required)
// ------------------------------------------------------------*/

/* Set the source surface */
#define HWGE_SET_SOURCE_SURFACE(Base, Pitch) { \
	HWGE_SET_SRCBASE_REG(Base);		   \
	HWGE_SET_SRCPITCH_REG(HWGE_pulPitchConversion[Pitch >> 5]);	   \
	}

/* Set the destination surface */
#define HWGE_SET_DESTINATION_SURFACE(Base, Pitch) { \
	HWGE_SET_DSTBASE_REG(Base);		   \
	HWGE_SET_DSTPITCH_REG(HWGE_pulPitchConversion[Pitch >> 5]);   \
	}

/* ------------------------------------------------------------
// DEFINE		: HWGE_PREPARE_FILL
//				  HWGE_PREPARE_SCREEN2SCREEN
//				  HWGE_PREPARE_HOST2SCREEN
// ------------------------------------------------------------
// DESCRIPTION	: Select the type of 2D acceleration
// ------------------------------------------------------------
// HWGE_PREPARE_FILL			: 
//				  Fill a rectangle surface
// HWGE_PREPARE_SCREEN2SCREEN	:
//				  Copy from frame buffer to frame buffer
// HWGE_PREPARE_HOST2SCREEN		: 
//				  Copy from the Data Port to the frame buffer
// ------------------------------------------------------------
// Flag			: One raster operation to be used
//				  One of the HWGE_ROP_... value
//				  or HWGE_ROP_OTHER macro for other values
//				  and one or more additionnal operations
//				  One of the HWGE_FLAG_... values
//				  The different flags are combined with an
//				  OR operation (| symbol)
// ------------------------------------------------------------*/

/* No source : source is the background color register */
#define HWGE_PREPARE_FILL(Flag) { HWGE_SET_ROP_REG(Flag); }

/* Source is in the frame buffer */
#define HWGE_PREPARE_SCREEN2SCREEN(Flag) { HWGE_SET_ROP_REG(Flag | HWGE_ROPREG_SRCSCRN); }

/* Source is in the data port fifo */
#define HWGE_PREPARE_HOST2SCREEN(Flag) { HWGE_SET_ROP_REG(Flag | HWGE_ROPREG_SRCHOST); }


/* ------------------------------------------------------------
// DEFINE		: HWGE_?BPP_SRC_COLORKEY
//				  HWGE_?BPP_DST_COLORKEY
// ------------------------------------------------------------
// DESCRIPTION	: Select the color key in case of source
//				  or destination transparency.
// ------------------------------------------------------------
// Color		: Color key in the pixel format of the 
//				  operation
// ------------------------------------------------------------*/

/* 8 bits per pixel source color key */
#define HWGE_1BPP_SRC_COLORKEY(Color) \
	{ HWGE_SET_SRCKEY_REG((unsigned long)(Color) + ((unsigned long)(Color) << 8) + ((unsigned long)(Color) << 16) + ((unsigned long)(Color) << 24)); }

/* 16 bits per pixel source color key */
#define HWGE_2BPP_SRC_COLORKEY(Color) \
	{ HWGE_SET_SRCKEY_REG((unsigned long)(Color) + ((unsigned long)(Color) << 16)); }

/* 24 bits per pixel source color key */
#define HWGE_3BPP_SRC_COLORKEY(Color) \
	{ HWGE_SET_SRCKEY_REG((unsigned long)(Color)); }

/* 8 bits per pixel destination color key */
#define HWGE_1BPP_DST_COLORKEY(Color) \
	{ HWGE_SET_DSTKEY_REG((unsigned long)(Color) + ((unsigned long)(Color) << 8) + ((unsigned long)(Color) << 16) + ((unsigned long)(Color) << 24)); }

/* 16 bits per pixel destination color key */
#define HWGE_2BPP_DST_COLORKEY(Color) \
	{ HWGE_SET_DSTKEY_REG((unsigned long)(Color) + ((unsigned long)(Color) << 16)); }

/* 24 bits per pixel destination color key */
#define HWGE_3BPP_DST_COLORKEY_DSTMATCH(Color) \
	{ HWGE_SET_DSTKEY_REG((unsigned long)(Color)); }

/* ------------------------------------------------------------
// DEFINE		: HWGE_SET_FILL_COLOR
// ------------------------------------------------------------
// DESCRIPTION	: Select the drawing color for a rectangle or
//				  line fill
// ------------------------------------------------------------
// Color		: Color in the pixel format of the operation
// ------------------------------------------------------------*/
#define HWGE_SET_FILL_COLOR(Color) \
	{ HWGE_SET_BKCOLOR_REG(Color); }

/* ------------------------------------------------------------
// DEFINE		: HWGE_SET_TEXT_COLOR
// ------------------------------------------------------------
// DESCRIPTION	: Select the drawings colors for a bitmap
//				  expansion operation like font drawing
// ------------------------------------------------------------
// BkColor		: Bakeground color in the format of the operation
// FgColor		: Foreground color in the format of the operation
// ------------------------------------------------------------*/
#define HWGE_SET_TEXT_COLOR(BkColor, FgColor) { \
	HWGE_SET_BKCOLOR_REG(BkColor); \
	HWGE_SET_FGCOLOR_REG(FgColor); \
	}

/* ------------------------------------------------------------
// DEFINE		: HWGE_SET_TRANSPTEXT_COLOR
// ------------------------------------------------------------
// DESCRIPTION	: Select the drawing color for a transparent 
//				  bitmap expansion operation like transparent 
//				  fonts drawing
// ------------------------------------------------------------
// FgColor		: Color in the pixel format of the operation
// ------------------------------------------------------------*/
#define HWGE_SET_TRANSPTEXT_COLOR(FgColor) { \
	HWGE_SET_FGCOLOR_REG(FgColor); \
	}

/* ------------------------------------------------------------
// DEFINE		: HWGE_?BPP_FILL_START
// ------------------------------------------------------------
// DESCRIPTION	: Start a rectangle fill operation in 
//				  8, 16 or 24 bits per pixel format
// ------------------------------------------------------------
// DstX			: X coordinate of the rectangle in the 
//				  destination surface
// DstY			: Y coordinate of the rectangle in the 
//				  destination surface
// Width		: Width of the rectangle
// Height		: Height of the rectangle
// ------------------------------------------------------------*/

/* Start fill rectangle operation in 8 bits per pixel mode */
#define HWGE_1BPP_FILL_START(DstX, DstY, Width, Height) {  \
	HWGE_SET_BPP_REG(0x0); 					\
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG(Width - 1);					\
	HWGE_SET_DSTXY_REG(((unsigned long)(DstY) << 16) + (unsigned long)(DstX)); \
	}

/* Start fill rectangle operation in 16 bits per pixel mode */
#define HWGE_2BPP_FILL_START(DstX, DstY, Width, Height) { \
	HWGE_SET_BPP_REG(0x1); 					\
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG((unsigned long)(Width) + (unsigned long)(Width) - 1);					\
	HWGE_SET_DSTXY_REG(((unsigned long)(DstY) << 16) + (unsigned long)(DstX) + (unsigned long)(DstX)); \
	}

/* Start fill rectangle operation in 24 bits per pixel mode */
#define HWGE_3BPP_FILL_START(DstX, DstY, Width, Height) { \
	HWGE_SET_BPP_REG(0x2); 					\
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG((unsigned long)(Width) + (unsigned long)(Width) + (unsigned long)(Width) - 1);					\
	HWGE_SET_DSTXY_REG((((unsigned long)(DstY)) << 16) + (unsigned long)(DstX) + (unsigned long)(DstX) + (unsigned long)(DstX)); \
	}

/* Start fill rectangle operation in 32 bits per pixel mode */
#define HWGE_4BPP_FILL_START(DstX, DstY, Width, Height) { \
	HWGE_SET_BPP_REG(0x3); 					\
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG(((unsigned long)(Width) << 2) - 1);					\
	HWGE_SET_DSTXY_REG(((unsigned long)(DstY) << 16) + ((unsigned long)(DstX) << 2)); \
	}

/* ------------------------------------------------------------
// DEFINE		: HWGE_?BPP_OFFS2S_START
// ------------------------------------------------------------
// DESCRIPTION	: Start a frame buffer to frame buffer copy in 
//				  8, 16 or 24 bits per pixel format
// ------------------------------------------------------------
// SrcX			: X coordinate of the rectangle in the 
//				  source surface
// SrcY			: Y coordinate of the rectangle in the 
//				  source surface
// DstX			: X coordinate of the rectangle in the 
//				  destination surface
// DstY			: Y coordinate of the rectangle in the 
//				  destination surface
// Width		: Width of the rectangle
// Height		: Height of the rectangle
// ------------------------------------------------------------*/

/* Start the copy operation in 8 bits per pixel mode */
#define HWGE_1BPP_OFFS2S_START(SrcX, SrcY, DstX, DstY, Width, Height) {  \
	HWGE_SET_BPP_REG(0x0); 					\
	HWGE_SET_SRCXY_REG(((unsigned long)(SrcY) << 16) + (unsigned long)(SrcX)); \
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG(Width - 1);					\
	HWGE_SET_DSTXY_REG(((unsigned long)DstY << 16) + (unsigned long)(DstX)); \
	}

/* Start the copy operation in 16 bits per pixel mode */
#define HWGE_2BPP_OFFS2S_START(SrcX, SrcY, DstX, DstY, Width, Height) { \
	HWGE_SET_BPP_REG(0x1); 					\
	HWGE_SET_SRCXY_REG(((unsigned long)(SrcY) << 16) + (unsigned long)(SrcX) + (unsigned long)(SrcX)); \
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG((unsigned long)(Width) + (unsigned long)(Width) - 1);					\
	HWGE_SET_DSTXY_REG(((unsigned long)(DstY) << 16) + (unsigned long)(DstX) + (unsigned long)(DstX)); \
	}

/* Start the copy operation in 24 bits per pixel mode */
#define HWGE_3BPP_OFFS2S_START(SrcX, SrcY, DstX, DstY, Width, Height) { \
	HWGE_SET_BPP_REG(0x2); 					\
	HWGE_SET_SRCXY_REG(((unsigned long)(SrcY) << 16) + (unsigned long)(SrcX) + (unsigned long)(SrcX) + (unsigned long)(SrcX)); \
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG((unsigned long)(Width) + (unsigned long)(Width) + (unsigned long)(Width) - 1);					\
	HWGE_SET_DSTXY_REG((((unsigned long)(DstY)) << 16) + (unsigned long)(DstX) + (unsigned long)(DstX) + (unsigned long)(DstX)); \
	}

/* Start the copy operation in 32 bits per pixel mode */
#define HWGE_4BPP_OFFS2S_START(SrcX, SrcY, DstX, DstY, Width, Height) { \
	HWGE_SET_BPP_REG(0x3); 					\
	HWGE_SET_SRCXY_REG(((unsigned long)(SrcY) << 16) + ((unsigned long)(SrcX) << 2)); \
	HWGE_SET_HEIGHT_REG(nHeight - 1);				\
	HWGE_SET_WIDTH_REG(((unsigned long)(Width) << 2) - 1);					\
	HWGE_SET_DSTXY_REG(((unsigned long)(DstY) << 16) + ((unsigned long)(DstX) << 2)); \
	}

/* ------------------------------------------------------------
// DEFINE		: HWGE_?BPP_INVOFFS2S_START
// ------------------------------------------------------------
// DESCRIPTION	: Start a frame buffer to frame buffer copy in 
//				  8, 16 or 24 bits per pixel format, with 
//				  a vertical flip
// ------------------------------------------------------------
// SrcX			: X coordinate of the rectangle in the 
//				  source surface
// SrcY			: Y coordinate of the rectangle in the 
//				  source surface
// DstX			: X coordinate of the rectangle in the 
//				  destination surface
// DstY			: Y coordinate of the rectangle in the 
//				  destination surface
// Width		: Width of the rectangle
// Height		: Height of the rectangle
// ------------------------------------------------------------*/

/* Start the copy operation in 8 bits per pixel mode */
#define HWGE_1BPP_INVOFFS2S_START(SrcX, SrcY, DstX, DstY, Width, Height) {  \
	HWGE_SET_BPP_REG(0x8); 					\
	HWGE_SET_SRCXY_REG(((unsigned long)(SrcY + Height - 1) << 16) + (unsigned long)(SrcX)); \
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG(Width - 1);					\
	HWGE_SET_DSTXY_REG(((unsigned long)DstY << 16) + (unsigned long)(DstX)); \
	}

/* Start the copy operation in 16 bits per pixel mode */
#define HWGE_2BPP_INVOFFS2S_START(SrcX, SrcY, DstX, DstY, Width, Height) { \
	HWGE_SET_SRCXY_REG(((unsigned long)(SrcY + Height - 1) << 16) + (unsigned long)(SrcX) + (unsigned long)(SrcX)); \
	HWGE_SET_BPP_REG(0x81); 					\
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG((unsigned long)(Width) + (unsigned long)(Width) - 1);					\
	HWGE_SET_DSTXY_REG(((unsigned long)(DstY) << 16) + (unsigned long)(DstX) + (unsigned long)(DstX)); \
	}

/* Start the copy operation in 24 bits per pixel mode */
#define HWGE_3BPP_INVOFFS2S_START(SrcX, SrcY, DstX, DstY, Width, Height) { \
	HWGE_SET_SRCXY_REG(((unsigned long)(SrcY + Height - 1) << 16) + (unsigned long)(SrcX) + (unsigned long)(SrcX) + (unsigned long)(SrcX)); \
	HWGE_SET_BPP_REG(0x82); 					\
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG((unsigned long)(Width) + (unsigned long)(Width) + (unsigned long)(Width) - 1);					\
	HWGE_SET_DSTXY_REG((((unsigned long)(DstY)) << 16) + (unsigned long)(DstX) + (unsigned long)(DstX) + (unsigned long)(DstX)); \
	}

/* Start the copy operation in 32 bits per pixel mode */
#define HWGE_4BPP_INVOFFS2S_START(SrcX, SrcY, DstX, DstY, Width, Height) { \
	HWGE_SET_SRCXY_REG(((unsigned long)(SrcY + Height - 1) << 16) + ((unsigned long)(SrcX) << 2)); \
	HWGE_SET_BPP_REG(0x83); 					\
	HWGE_SET_HEIGHT_REG(nHeight - 1);				\
	HWGE_SET_WIDTH_REG(((unsigned long)(Width) << 2) - 1);					\
	HWGE_SET_DSTXY_REG(((unsigned long)(DstY) << 16) + ((unsigned long)(DstX) << 2)); \
	}

/* ------------------------------------------------------------
// DEFINE		: HWGE_?BPP_S2S_START
// ------------------------------------------------------------
// DESCRIPTION	: Start a frame buffer to frame buffer copy in 
//				  8, 16 or 24 bits per pixel format, with 
//				  overlapping management. On contrary to
//				  the HWGE_?BPP_OFFS2S_START function, the
//				  destination recangle can overlap the source
//				  rectangle if source and destination are
//				  in the same surface
// ------------------------------------------------------------
// SrcX			: X coordinate of the rectangle in the 
//				  source surface
// SrcY			: Y coordinate of the rectangle in the 
//				  source surface
// DstX			: X coordinate of the rectangle in the 
//				  destination surface
// DstY			: Y coordinate of the rectangle in the 
//				  destination surface
// Width		: Width of the rectangle
// Height		: Height of the rectangle
// ------------------------------------------------------------*/

/* Start the copy operation in 8 bits per pixel mode */
#define HWGE_1BPP_S2S_START(SrcX, SrcY, DstX, DstY, Width, Height) { \
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG(Width - 1); \
	if(DstX > SrcX) \
	{ \
		if(DstY > SrcY) \
		{ \
			HWGE_SET_BPP_REG(0x000000e0); 						\
			HWGE_SET_SRCXY_REG(((unsigned long)(SrcY + Height - 1) << 16) + (unsigned long)(SrcX + Width) - 1); \
			HWGE_SET_DSTXY_REG(((unsigned long)(DstY + Height - 1) << 16) + (unsigned long)(DstX + Width) - 1); \
		} \
		else \
		{ \
			HWGE_SET_BPP_REG(0x00000020); 						\
			HWGE_SET_SRCXY_REG(((unsigned long)(SrcY) << 16) + (unsigned long)(SrcX + Width) - 1); \
			HWGE_SET_DSTXY_REG(((unsigned long)(DstY) << 16) + (unsigned long)(DstX + Width) - 1); \
		} \
	}												\
	else \
	{ \
		if(DstY > SrcY) \
		{ \
			HWGE_SET_BPP_REG(0x000000C0); 						\
			HWGE_SET_SRCXY_REG(((unsigned long)(SrcY + Height - 1) << 16) + (unsigned long)(SrcX)); \
			HWGE_SET_DSTXY_REG(((unsigned long)(DstY + Height - 1) << 16) + (unsigned long)(DstX)); \
		} \
		else \
		{ \
			HWGE_SET_BPP_REG(0x00000000); 						\
			HWGE_SET_SRCXY_REG(((unsigned long)(SrcY) << 16) + (unsigned long)(SrcX)); \
			HWGE_SET_DSTXY_REG(((unsigned long)(DstY) << 16) + (unsigned long)(DstX)); \
		} \
	} \
	}

/* Start the copy operation in 16 bits per pixel mode */
#define HWGE_2BPP_S2S_START(SrcX, SrcY, DstX, DstY, Width, Height) { \
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG((unsigned long)(Width) + (unsigned long)(Width) - 1);					\
	if(DstX > SrcX) \
	{ \
		if(DstY > SrcY) \
		{ \
			HWGE_SET_BPP_REG(0x000000e1); 						\
			HWGE_SET_SRCXY_REG(((unsigned long)(SrcY + Height - 1) << 16) + ((unsigned long)(SrcX + Width) << 1) - 1); \
			HWGE_SET_DSTXY_REG(((unsigned long)(DstY + Height - 1) << 16) + ((unsigned long)(DstX + Width) << 1) - 1); \
		} \
		else \
		{ \
			HWGE_SET_BPP_REG(0x00000021); 						\
			HWGE_SET_SRCXY_REG(((unsigned long)(SrcY) << 16) + ((unsigned long)(SrcX + Width) << 1) - 1); \
			HWGE_SET_DSTXY_REG(((unsigned long)(DstY) << 16) + ((unsigned long)(DstX + Width) << 1) - 1); \
		} \
	}												\
	else \
	{ \
		if(DstY > SrcY) \
		{ \
			HWGE_SET_BPP_REG(0x000000C1); 						\
			HWGE_SET_SRCXY_REG(((unsigned long)(SrcY + Height - 1) << 16) + (unsigned long)(SrcX) + (unsigned long)(SrcX)); \
			HWGE_SET_DSTXY_REG(((unsigned long)(DstY + Height - 1) << 16) + (unsigned long)(DstX) + (unsigned long)(DstX)); \
		} \
		else \
		{ \
			HWGE_SET_BPP_REG(0x00000001); 						\
			HWGE_SET_SRCXY_REG(((unsigned long)(SrcY) << 16) + (unsigned long)(SrcX) + (unsigned long)(SrcX)); \
			HWGE_SET_DSTXY_REG(((unsigned long)(DstY) << 16) + (unsigned long)(DstX) + (unsigned long)(DstX)); \
		} \
	} \
	}

/* Start the copy operation in 24 bits per pixel mode */
#define HWGE_3BPP_S2S_START(SrcX, SrcY, DstX, DstY, Width, Height) { \
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG((unsigned long)(Width) + (unsigned long)(Width) + (unsigned long)(Width) - 1);					\
	if(DstX > SrcX) \
	{ \
		unsigned long TempSrcX; \
		unsigned long TempDstX; \
		\
		TempDstX = DstX + Width;								\
		TempSrcX = SrcX + Width;								\
		\
		if(DstY > SrcY) \
		{ \
			HWGE_SET_BPP_REG(0x000000e2); 						\
			HWGE_SET_SRCXY_REG(((unsigned long)(SrcY + Height - 1) << 16) + (unsigned long)(TempSrcX) + (unsigned long)(TempSrcX) + (unsigned long)(TempSrcX) - 1); \
			HWGE_SET_DSTXY_REG(((unsigned long)(DstY + Height - 1) << 16) + (unsigned long)(TempDstX) + (unsigned long)(TempDstX) + (unsigned long)(TempDstX) - 1); \
		} \
		else \
		{ \
			HWGE_SET_BPP_REG(0x00000022); 						\
			HWGE_SET_SRCXY_REG(((unsigned long)(SrcY) << 16) + (unsigned long)(TempSrcX) + (unsigned long)(TempSrcX) + (unsigned long)(TempSrcX) - 1); \
			HWGE_SET_DSTXY_REG(((unsigned long)(DstY) << 16) + (unsigned long)(TempDstX) + (unsigned long)(TempDstX) + (unsigned long)(TempDstX) - 1); \
		} \
	}												\
	else \
	{ \
		if(DstY > SrcY) \
		{ \
			HWGE_SET_BPP_REG(0x000000C2); 						\
			HWGE_SET_SRCXY_REG(((unsigned long)(SrcY + Height - 1) << 16) + (unsigned long)(SrcX) + (unsigned long)(SrcX) + (unsigned long)(SrcX)); \
			HWGE_SET_DSTXY_REG(((unsigned long)(DstY + Height - 1) << 16) + (unsigned long)(DstX) + (unsigned long)(DstX) + (unsigned long)(DstX)); \
		} \
		else \
		{ \
			HWGE_SET_BPP_REG(0x00000002); 						\
			HWGE_SET_SRCXY_REG(((unsigned long)(SrcY) << 16) + (unsigned long)(SrcX) + (unsigned long)(SrcX) + (unsigned long)(SrcX)); \
			HWGE_SET_DSTXY_REG(((unsigned long)(DstY) << 16) + (unsigned long)(DstX) + (unsigned long)(DstX) + (unsigned long)(DstX)); \
		} \
	} \
	}

/* Start the copy operation in 32 bits per pixel mode */
#define HWGE_4BPP_S2S_START(SrcX, SrcY, DstX, DstY, Width, Height) { \
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG(((unsigned long)(Width) << 2) - 1);					\
	if(DstX > SrcX) \
	{ \
		if(DstY > SrcY) \
		{ \
			HWGE_SET_BPP_REG(0x000000e3); 						\
			HWGE_SET_SRCXY_REG(((unsigned long)(SrcY + Height - 1) << 16) + ((unsigned long)(SrcX + Width) << 2) - 1); \
			HWGE_SET_DSTXY_REG(((unsigned long)(DstY + Height - 1) << 16) + ((unsigned long)(DstX + Width) << 2) - 1); \
		} \
		else \
		{ \
			HWGE_SET_BPP_REG(0x00000023); 						\
			HWGE_SET_SRCXY_REG(((unsigned long)(SrcY) << 16) + ((unsigned long)(SrcX + Width) << 2) - 1); \
			HWGE_SET_DSTXY_REG(((unsigned long)(DstY) << 16) + ((unsigned long)(DstX + Width) << 2) - 1); \
		} \
	}												\
	else \
	{ \
		if(DstY > SrcY) \
		{ \
			HWGE_SET_BPP_REG(0x000000C3); 						\
			HWGE_SET_SRCXY_REG(((unsigned long)(SrcY + Height - 1) << 16) + ((unsigned long)(SrcX) << 2)); \
			HWGE_SET_DSTXY_REG(((unsigned long)(DstY + Height - 1) << 16) + ((unsigned long)(DstX) << 2)); \
		} \
		else \
		{ \
			HWGE_SET_BPP_REG(0x00000003); 						\
			HWGE_SET_SRCXY_REG(((unsigned long)(SrcY) << 16) + ((unsigned long)(SrcX) << 2)); \
			HWGE_SET_DSTXY_REG(((unsigned long)(DstY) << 16) + ((unsigned long)(DstX) << 2)); \
		} \
	} \
	}

/* ------------------------------------------------------------
// DEFINE		: HWGE_?BPP_HLINE_START
// ------------------------------------------------------------
// DESCRIPTION	: Start an horizontal line drawing
// ------------------------------------------------------------
// DstX			: X coordinate in destination surface of the 
//				  left side of the line
// DstY			: Y coordinate of the line in destination 
//				  surface 
// Width		: Number of pixels of the line
// ------------------------------------------------------------*/

/* Horizontal line drawing in 8 bits per pixel format */
#define HWGE_1BPP_HLINE_START(DstX, DstY, Width) {  \
	HWGE_SET_WIDTH_REG((unsigned long)(Width) - 1);	\
	HWGE_SET_1HEIGHTDSTXY_REG(((unsigned long)(DstY) << 16) + (unsigned long)(DstX)); \
	}

/* Horizontal line drawing in 16 bits per pixel format */
#define HWGE_2BPP_HLINE_START(DstX, DstY, Width) { \
	HWGE_SET_WIDTH_REG((unsigned long)(Width) + (unsigned long)(Width) - 1);					\
	HWGE_SET_1HEIGHTDSTXY_REG(((unsigned long)(DstY) << 16) + (unsigned long)(DstX) + (unsigned long)(DstX)); \
	}

/* Horizontal line drawing in 24 bits per pixel format */
#define HWGE_3BPP_HLINE_START(DstX, DstY, Width) { \
	HWGE_SET_WIDTH_REG((unsigned long)(Width) + (unsigned long)(Width) + (unsigned long)(Width) - 1);					\
	HWGE_SET_1HEIGHTDSTXY_REG((((unsigned long)(DstY)) << 16) + (unsigned long)(DstX) + (unsigned long)(DstX) + (unsigned long)(DstX)); \
	}

/* Horizontal line drawing in 32 bits per pixel format */
#define HWGE_4BPP_HLINE_START(DstX, DstY, Width) { \
	HWGE_SET_WIDTH_REG(((unsigned long)(Width) << 2) - 1);					\
	HWGE_SET_1HEIGHTDSTXY_REG(((unsigned long)(DstY) << 16) + ((unsigned long)(DstX) << 2)); \
	}

/* ------------------------------------------------------------
// DEFINE		: HWGE_?BPP_VLINE_START
// ------------------------------------------------------------
// DESCRIPTION	: Start a vertical line drawing
// ------------------------------------------------------------
// DstX			: X coordinate of the line in destination 
//				  surface
// DstY			: Y coordinate of the top side of the line in 
//				  the destination surface 
// Width		: Number of pixels of the line
// ------------------------------------------------------------*/

/* Vertical line drawing in 8 bits per pixel format */
#define HWGE_1BPP_VLINE_START(DstX, DstY, Height) {  \
	HWGE_SET_HEIGHT_REG((Height) - 1);				\
	HWGE_SET_1WIDTHDSTXY_REG(((unsigned long)(DstY) << 16) + (unsigned long)(DstX)); \
	}

/* Vertical line drawing in 16 bits per pixel format */
#define HWGE_2BPP_VLINE_START(DstX, DstY, Height) { \
	HWGE_SET_HEIGHT_REG((Height) - 1);				\
	HWGE_SET_2WIDTHDSTXY_REG(((unsigned long)(DstY) << 16) + (unsigned long)(DstX) + (unsigned long)(DstX)); \
	}

/* Vertical line drawing in 24 bits per pixel format */
#define HWGE_3BPP_VLINE_START(DstX, DstY, Height) { \
	HWGE_SET_HEIGHT_REG((Height) - 1);				\
	HWGE_SET_3WIDTHDSTXY_REG((((unsigned long)(DstY)) << 16) + (unsigned long)(DstX) + (unsigned long)(DstX) + (unsigned long)(DstX)); \
	}

/* Vertical line drawing in 32 bits per pixel format */
#define HWGE_4BPP_VLINE_START(DstX, DstY, Height) { \
	HWGE_SET_HEIGHT_REG((Height) - 1);				\
	HWGE_SET_4WIDTHDSTXY_REG(((unsigned long)(DstY) << 16) + ((unsigned long)(DstX) << 2)); \
	}

/* ------------------------------------------------------------
// DEFINE		: HWGE_?BPP_H2S_START
// ------------------------------------------------------------
// DESCRIPTION	: Start a system memory to frame buffer copy in 
//				  8, 16 or 24 bits per pixel format.
// ------------------------------------------------------------
// Ptr			: Linear pointer in system memory of the
//				  source surface
// SrcX			: X coordinate of the rectangle in the 
//				  source surface
// SrcY			: Y coordinate of the rectangle in the
//				  source surface
// Pitch		: Number of bytes between 2 lines of the source
//				  surface
// DstX			: X coordinate of the rectangle in the 
//				  destination surface
// DstY			: Y coordinate of the rectangle in the 
//				  destination surface
// Width		: Width of the rectangle
// Height		: Height of the rectangle
// ------------------------------------------------------------*/

/* Start host to screen copy in 8 bits per pixels format */
#define HWGE_1BPP_H2S_START(Ptr, SrcX, SrcY, Pitch, DstX, DstY, Width, Height) {  \
	HWGE_SET_BPP_REG(0x0); 					\
	HWGE_SET_HEIGHT_REG((Height) - 1);				\
	HWGE_SET_WIDTH_REG((Width) - 1);					\
	HWGE_SendDataToDataPort(((unsigned long)(DstY) << 16) + (unsigned long)(DstX), (Ptr) + (SrcY) * (Pitch) + (SrcX), Pitch, HWGE_VirtualGERegister, HWGE_VirtualDataPort); \
	}

/* Start host to screen copy in 16 bits per pixels format */
#define HWGE_2BPP_H2S_START(Ptr, SrcX, SrcY, Pitch, DstX, DstY, Width, Height) { \
	HWGE_SET_BPP_REG(0x1); 					\
	HWGE_SET_HEIGHT_REG((Height) - 1);				\
	HWGE_SET_WIDTH_REG((unsigned long)(Width) + (unsigned long)(Width) - 1);					\
	HWGE_SendDataToDataPort(((unsigned long)(DstY) << 16) + (unsigned long)(DstX) + (unsigned long)(DstX), (Ptr) + (SrcY) * (Pitch) + ((unsigned long)(SrcX) << 1), Pitch, HWGE_VirtualGERegister, HWGE_VirtualDataPort); \
	}

/* Start host to screen copy in 24 bits per pixels format */
#define HWGE_3BPP_H2S_START(Ptr, SrcX, SrcY, Pitch, DstX, DstY, Width, Height) { \
	HWGE_SET_BPP_REG(0x2); 					\
	HWGE_SET_HEIGHT_REG((Height) - 1);				\
	HWGE_SET_WIDTH_REG((unsigned long)(Width) + (unsigned long)(Width) + (unsigned long)(Width) - 1);					\
	HWGE_SendDataToDataPort((((unsigned long)(DstY)) << 16) + (unsigned long)(DstX) + (unsigned long)(DstX) + (unsigned long)(DstX), (Ptr) + (SrcY) * (Pitch) + ((unsigned long)(SrcX) << 1) + (unsigned long)(SrcX), Pitch, HWGE_VirtualGERegister, HWGE_VirtualDataPort); \
	}

/* Start host to screen copy in 32 bits per pixels format */
#define HWGE_4BPP_H2S_START(Ptr, SrcX, SrcY, Pitch, DstX, DstY, Width, Height) { \
	HWGE_SET_BPP_REG(0x3); 					\
	HWGE_SET_HEIGHT_REG((Height) - 1);				\
	HWGE_SET_WIDTH_REG(((unsigned long)(Width) << 2) - 1);					\
	HWGE_SendDataToDataPort(((unsigned long)(DstY) << 16) + ((unsigned long)(DstX) << 2), (Ptr) + (SrcY) * (Pitch) + ((unsigned long)(SrcX) << 2), Pitch, HWGE_VirtualGERegister, HWGE_VirtualDataPort); \
	}

/* ------------------------------------------------------------
// DEFINE		: HWGE_?BPP_PACKEDH2S_START
// ------------------------------------------------------------
// DESCRIPTION	: Start a system memory to frame buffer copy in 
//				  8, 16 or 24 bits per pixel format.
//				  Format of input data are packed,
//				  this means bit of next line directly follow
//				  the last bit of previous line
// ------------------------------------------------------------
// Ptr			: Linear pointer in system memory of the
//				  source packed data
// DstX			: X coordinate of the rectangle in the
//				  destination surface
// DstY			: Y coordinate of the rectangle in the
//				  destination surface
// Width		: Width of the rectangle
// Height		: Height of the rectangle
// ------------------------------------------------------------*/

/* Start host to screen copy in 8 bits per pixels format */
#define HWGE_1BPP_PACKEDH2S_START(Ptr, DstX, DstY, Width, Height) {  \
	HWGE_SET_BPP_REG(0x0); 					\
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG(Width - 1);					\
	HWGE_SET_SRCXY_REG((Ptr) & 0x3); \
	HWGE_SET_DSTXY_REG(((unsigned long)DstY << 16) + (unsigned long)(DstX)); \
	HWGE_SendPackedData((Ptr) & ~0x3, HWGE_VirtualDataPort, (Width * Height + ((Ptr) & 0x3) + 3) >> 2); \
	}

/* Start host to screen copy in 16 bits per pixels format */
#define HWGE_2BPP_PACKEDH2S_START(Ptr, DstX, DstY, Width, Height) {  \
	HWGE_SET_BPP_REG(0x1); 					\
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG((unsigned long)(Width) + (unsigned long)(Width) - 1);					\
	HWGE_SET_SRCXY_REG((Ptr) & 0x3); \
	HWGE_SET_DSTXY_REG(((unsigned long)(DstY) << 16) + (unsigned long)(DstX) + (unsigned long)(DstX)); \
	HWGE_SendPackedData((Ptr) & ~0x3, HWGE_VirtualDataPort, (Width * Height + ((Ptr) & 0x3) + 3) >> 2); \
	}

/* Start host to screen copy in 24 bits per pixels format */
#define HWGE_3BPP_PACKEDH2S_START(Ptr, DstX, DstY, Width, Height) {  \
	HWGE_SET_BPP_REG(0x2); 					\
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG((unsigned long)(Width) + (unsigned long)(Width) + (unsigned long)(Width) - 1);					\
	HWGE_SET_SRCXY_REG((Ptr) & 0x3); \
	HWGE_SET_DSTXY_REG((((unsigned long)(DstY)) << 16) + (unsigned long)(DstX) + (unsigned long)(DstX) + (unsigned long)(DstX)); \
	HWGE_SendPackedData((Ptr) & ~0x3, HWGE_VirtualDataPort, (Width * Height + ((Ptr) & 0x3) + 3) >> 2); \
	}

/* Start host to screen copy in 32 bits per pixels format */
#define HWGE_4BPP_PACKEDH2S_START(Ptr, DstX, DstY, Width, Height) {  \
	HWGE_SET_BPP_REG(0x3); 					\
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG(((unsigned long)(Width) << 2) - 1);					\
	HWGE_SET_SRCXY_REG((Ptr) & 0x3); \
	HWGE_SET_DSTXY_REG(((unsigned long)(DstY) << 16) + ((unsigned long)(DstX) << 2)); \
	HWGE_SendPackedData((Ptr) & ~0x3, HWGE_VirtualDataPort, (Width * Height + ((Ptr) & 0x3) + 3) >> 2); \
	}

/* ------------------------------------------------------------
// DEFINE		: HWGE_?BPP_SPECIALH2S_START
// ------------------------------------------------------------
// DESCRIPTION	: Start a data port to frame buffer copy in
//				  8, 16 or 24 bits per pixel format. The
//				  operation is just initialized and the CPU
//				  must send the right number of data to the
//				  data port after call this macro.
//				  Warning : the system will hang if the right
//				  number of data is not send to the data port
// ------------------------------------------------------------
// Offset		: Offset in the first 64bits
//				  In bytes or in bits if bitmap expansion
// DstX			: X coordinate of the rectangle in the
//				  destination surface
// DstY			: Y coordinate of the rectangle in the
//				  destination surface
// Width		: Width of the rectangle
// Height		: Height of the rectangle
// ------------------------------------------------------------*/

/* Initialize the host to screen copy in 8 bits per pixels format */
#define HWGE_1BPP_SPECIALH2S_START(Offset, DstX, DstY, Width, Height) {  \
	HWGE_SET_BPP_REG(0x0); 					\
	HWGE_SET_HEIGHT_REG(Height - 1);			\
	HWGE_SET_WIDTH_REG(Width - 1);			\
	HWGE_SET_SRCXY_REG(Offset);					\
	HWGE_SET_DSTXY_REG(((unsigned long)DstY << 16) + (unsigned long)(DstX)); \
	}

/* Initialize the host to screen copy in 16 bits per pixels format */
#define HWGE_2BPP_SPECIALH2S_START(Offset, DstX, DstY, Width, Height) { \
	HWGE_SET_BPP_REG(0x1); 					\
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG((unsigned long)(Width) + (unsigned long)(Width) - 1);					\
	HWGE_SET_SRCXY_REG(Offset);					\
	HWGE_SET_DSTXY_REG(((unsigned long)(DstY) << 16) + (unsigned long)(DstX) + (unsigned long)(DstX)); \
	}

/* Initialize the host to screen copy in 24 bits per pixels format */
#define HWGE_3BPP_SPECIALH2S_START(Offset, DstX, DstY, Width, Height) { \
	HWGE_SET_BPP_REG(0x2); 					\
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG((unsigned long)(Width) + (unsigned long)(Width) + (unsigned long)(Width) - 1);					\
	HWGE_SET_SRCXY_REG(Offset);					\
	HWGE_SET_DSTXY_REG((((unsigned long)(DstY)) << 16) + (unsigned long)(DstX) + (unsigned long)(DstX) + (unsigned long)(DstX)); \
	}

/* Initialize the host to screen copy in 32 bits per pixels format */
#define HWGE_4BPP_SPECIALH2S_START(Offset, DstX, DstY, Width, Height) { \
	HWGE_SET_BPP_REG(0x3); 					\
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG(((unsigned long)(Width) << 2) - 1);					\
	HWGE_SET_SRCXY_REG(Offset);					\
	HWGE_SET_DSTXY_REG(((unsigned long)(DstY) << 16) + ((unsigned long)(DstX) << 2)); \
	}

/* ------------------------------------------------------------
// DEFINE		: HWGE_?BPP_MONOH2S_START
// ------------------------------------------------------------
// DESCRIPTION	: Start a system memory to frame buffer copy 
//				  with pixel format conversion from 1 bit
//				  per pixels to 8, 16 or 24 bits per pixels
//				  format.
//				  Warning : the source X and pitch are in bits
//				  and not in bytes.
// ------------------------------------------------------------
// Ptr			: Linear pointer in system memory of the
//				  source surface
// SrcX			: X coordinate of the rectangle in the
//				  source surface, in bit
// SrcY			: Y coordinate of the rectangle in the
//				  source surface
// Pitch		: Number of bits between 2 lines of the source
//				  surface
// DstX			: X coordinate of the rectangle in the
//				  destination surface
// DstY			: Y coordinate of the rectangle in the
//				  destination surface
// Width		: Width of the rectangle
// Height		: Height of the rectangle
// ------------------------------------------------------------*/

/* Start host to screen copy in 8 bits per pixels format */
#define HWGE_1BPP_MONOH2S_START(Ptr, SrcX, SrcY, Pitch, DstX, DstY, Width, Height) {  \
	HWGE_SET_BPP_REG(0x0); 					\
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG(Width - 1);					\
	HWGE_SendMonoDataToDataPort(((unsigned long)(DstY) << 16) + (unsigned long)(DstX), (unsigned long)(Ptr), (unsigned long)(SrcY) * (Pitch) + (unsigned long)(SrcX), Pitch, Width, HWGE_VirtualGERegister, HWGE_VirtualDataPort); \
	}

/* Start host to screen copy in 16 bits per pixels format */
#define HWGE_2BPP_MONOH2S_START(Ptr, SrcX, SrcY, Pitch, DstX, DstY, Width, Height) {  \
	HWGE_SET_BPP_REG(0x1); 					\
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG((unsigned long)(Width) + (unsigned long)(Width) - 1);					\
	HWGE_SendMonoDataToDataPort(((unsigned long)(DstY) << 16) + (unsigned long)(DstX) + (unsigned long)(DstX), (unsigned long)(Ptr), (SrcY) * (Pitch) + (unsigned long)(SrcX), Pitch, Width, HWGE_VirtualGERegister, HWGE_VirtualDataPort); \
	}

/* Start host to screen copy in 24 bits per pixels format */
#define HWGE_3BPP_MONOH2S_START(Ptr, SrcX, SrcY, Pitch, DstX, DstY, Width, Height) {  \
	HWGE_SET_BPP_REG(0x2); 					\
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG((unsigned long)(Width) + (unsigned long)(Width) + (unsigned long)(Width) - 1);					\
	HWGE_SendMonoDataToDataPort((((unsigned long)(DstY)) << 16) + (unsigned long)(DstX) + (unsigned long)(DstX) + (unsigned long)(DstX), (unsigned long)(Ptr), (unsigned long)(SrcY) * (Pitch) + (unsigned long)(SrcX), Pitch, Width, HWGE_VirtualGERegister, HWGE_VirtualDataPort); \
	}

/* Start host to screen copy in 32 bits per pixels format */
#define HWGE_4BPP_MONOH2S_START(Ptr, SrcX, SrcY, Pitch, DstX, DstY, Width, Height) {  \
	HWGE_SET_BPP_REG(0x3); 					\
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG(((unsigned long)(Width) << 2) - 1);					\
	HWGE_SendMonoDataToDataPort(((unsigned long)(DstY) << 16) + ((unsigned long)(DstX) << 2), (unsigned long)(Ptr), (unsigned long)(SrcY) * (Pitch) + (unsigned long)(SrcX), Pitch, Width, HWGE_VirtualGERegister, HWGE_VirtualDataPort); \
	}

/* ------------------------------------------------------------
// DEFINE		: HWGE_?BPP_PACKEDMONOH2S_START
// ------------------------------------------------------------
// DESCRIPTION	: Start a system memory to frame buffer copy 
//				  with pixel format conversion from 1 bit
//				  per pixels to 8, 16 or 24 bits per pixels
//				  format. Format of input data are packed,
//				  this means bit of next line directly follow
//				  the last bit of previous line
// ------------------------------------------------------------
// Ptr			: Linear pointer in system memory of the
//				  source packed data
// DstX			: X coordinate of the rectangle in the
//				  destination surface
// DstY			: Y coordinate of the rectangle in the
//				  destination surface
// Width		: Width of the rectangle
// Height		: Height of the rectangle
// ------------------------------------------------------------*/

/* Start host to screen copy in 8 bits per pixels format */
#define HWGE_1BPP_PACKEDMONOH2S_START(Ptr, DstX, DstY, Width, Height) {  \
	HWGE_SET_BPP_REG(0x0); 					\
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG(Width - 1);					\
	HWGE_SET_SRCXY_REG(((Ptr) & 0x3) << 3); \
	HWGE_SET_DSTXY_REG(((unsigned long)DstY << 16) + (unsigned long)(DstX)); \
	HWGE_SendPackedData((Ptr) & ~0x3, HWGE_VirtualDataPort, (Width * Height + (((Ptr) & 0x3) << 3) + 31) >> 5); \
	}

/* Start host to screen copy in 16 bits per pixels format */
#define HWGE_2BPP_PACKEDMONOH2S_START(Ptr, DstX, DstY, Width, Height) {  \
	HWGE_SET_BPP_REG(0x1); 					\
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG((unsigned long)(Width) + (unsigned long)(Width) - 1);					\
	HWGE_SET_SRCXY_REG(((Ptr) & 0x3) << 3); \
	HWGE_SET_DSTXY_REG(((unsigned long)(DstY) << 16) + (unsigned long)(DstX) + (unsigned long)(DstX)); \
	HWGE_SendPackedData((Ptr) & ~0x3, HWGE_VirtualDataPort, (Width * Height + (((Ptr) & 0x3) << 3) + 31) >> 5); \
	}

/* Start host to screen copy in 24 bits per pixels format */
#define HWGE_3BPP_PACKEDMONOH2S_START(Ptr, DstX, DstY, Width, Height) {  \
	HWGE_SET_BPP_REG(0x2); 					\
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG((unsigned long)(Width) + (unsigned long)(Width) + (unsigned long)(Width) - 1);					\
	HWGE_SET_SRCXY_REG(((Ptr) & 0x3) << 3); \
	HWGE_SET_DSTXY_REG((((unsigned long)(DstY)) << 16) + (unsigned long)(DstX) + (unsigned long)(DstX) + (unsigned long)(DstX)); \
	HWGE_SendPackedData((Ptr) & ~0x3, HWGE_VirtualDataPort, (Width * Height + (((Ptr) & 0x3) << 3) + 31) >> 5); \
	}

/* Start host to screen copy in 32 bits per pixels format */
#define HWGE_4BPP_PACKEDMONOH2S_START(Ptr, DstX, DstY, Width, Height) {  \
	HWGE_SET_BPP_REG(0x3); 					\
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG(((unsigned long)(Width) << 2) - 1);					\
	HWGE_SET_SRCXY_REG(((Ptr) & 0x3) << 3); \
	HWGE_SET_DSTXY_REG(((unsigned long)(DstY) << 16) + ((unsigned long)(DstX) << 2)); \
	HWGE_SendPackedData((Ptr) & ~0x3, HWGE_VirtualDataPort, (Width * Height + (((Ptr) & 0x3) << 3) + 31) >> 5); \
	}

/* ------------------------------------------------------------
// DEFINE		: HWGE_?BPP_RIGHTEXTEND_START
// ------------------------------------------------------------
// DESCRIPTION	: Extend a rectangle of the screen by repeated
//				  it on the right. Source surface and
//				  destination surface must be the same before
//				  use this functions
// ------------------------------------------------------------
// SrcX			: X coordinate of the rectangle to extend
// SrcY			: Y coordinate of the rectangle to extend
// Width		: Width of the rectangle to extend
// Height		: Height of the rectangle to extend
// Size			: Number of pixels of the extension, including
//				  the source rectangle itself (this mean Size
//				  must be higher than Width parameter)
// ------------------------------------------------------------*/

/* Extend an 8 bits per pixels rectangle */
#define HWGE_1BPP_RIGHTEXTEND_START(SrcX, SrcY, Width, Height, Size) {  \
	HWGE_SET_ROP_REG(0xcc | HWGE_ROPREG_SRCSCRN);  \
	HWGE_SET_BPP_REG(0x0); 					\
	HWGE_SET_SRCXY_REG(((unsigned long)(SrcY) << 16) + (unsigned long)(SrcX)); \
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG(Size - Width - 1);					\
	HWGE_SET_DSTXY_REG(((unsigned long)SrcY << 16) + (unsigned long)(SrcX + Width)); \
	}

/* Extend an 16 bits per pixels rectangle */
#define HWGE_2BPP_RIGHTEXTEND_START(SrcX, SrcY, Width, Height, Size) { \
	HWGE_SET_ROP_REG(0xcc | HWGE_ROPREG_SRCSCRN);  \
	HWGE_SET_BPP_REG(0x1); 					\
	HWGE_SET_SRCXY_REG(((unsigned long)(SrcY) << 16) + (unsigned long)(SrcX) + (unsigned long)(SrcX)); \
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG(((unsigned long)(Size - Width) << 1) - 1);					\
	HWGE_SET_DSTXY_REG(((unsigned long)(SrcY) << 16) + ((unsigned long)(SrcX + Width) << 1)); \
	}

/* Extend an 24 bits per pixels rectangle */
#define HWGE_3BPP_RIGHTEXTEND_START(SrcX, SrcY, Width, Height, Size) { \
	HWGE_SET_ROP_REG(0xcc | HWGE_ROPREG_SRCSCRN);  \
	HWGE_SET_BPP_REG(0x2); 					\
	HWGE_SET_SRCXY_REG(((unsigned long)(SrcY) << 16) + (unsigned long)(SrcX) + (unsigned long)(SrcX) + (unsigned long)(SrcX)); \
	HWGE_SET_HEIGHT_REG(Height - 1);				\
	HWGE_SET_WIDTH_REG(((unsigned long)(Size - Width) << 1) + (unsigned long)(Size - Width) - 1);					\
	HWGE_SET_DSTXY_REG((((unsigned long)(SrcY)) << 16) + ((unsigned long)(SrcX + Width) << 1) + (unsigned long)(SrcX + Width)); \
	}

/* Extend an 32 bits per pixels rectangle */
#define HWGE_4BPP_RIGHTEXTEND_START(SrcX, SrcY, Width, Height, Size) { \
	HWGE_SET_ROP_REG(0xcc | HWGE_ROPREG_SRCSCRN);  \
	HWGE_SET_BPP_REG(0x3); 					\
	HWGE_SET_SRCXY_REG(((unsigned long)(SrcY) << 16) + ((unsigned long)(SrcX) << 2)); \
	HWGE_SET_HEIGHT_REG(nHeight - 1);				\
	HWGE_SET_WIDTH_REG(((unsigned long)(Size - Width) << 2) - 1);					\
	HWGE_SET_DSTXY_REG(((unsigned long)(SrcY) << 16) + ((unsigned long)(SrcX + Width) << 2)); \
	}

/* ------------------------------------------------------------
// DEFINE		: HWGE_?BPP_BOTTOMEXTEND_START
// ------------------------------------------------------------
// DESCRIPTION	: Extend a rectangle of the screen by repeated
//				  it on the bottom. Source surface and
//				  destination surface must be the same before
//				  use this functions. 
// ------------------------------------------------------------
// SrcX			: X coordinate of the rectangle to extend
// SrcY			: Y coordinate of the rectangle to extend
// Width		: Width of the rectangle to extend
// Height		: Height of the rectangle to extend
// Size			: Number of pixels of the extension, including
//				  the source rectangle itself (this mean Size
//				  must be higher than Height parameter)
// ------------------------------------------------------------*/

/* Extend an 8 bits per pixels rectangle */
#define HWGE_1BPP_BOTTOMEXTEND_START(SrcX, SrcY, Width, Height, Size) {  \
	HWGE_SET_ROP_REG(0xcc | HWGE_ROPREG_SRCSCRN);  \
	HWGE_SET_BPP_REG(0x0); 					\
	HWGE_SET_SRCXY_REG(((unsigned long)(SrcY) << 16) + (unsigned long)(SrcX)); \
	HWGE_SET_HEIGHT_REG(Size - Height - 1);				\
	HWGE_SET_WIDTH_REG(Width - 1);					\
	HWGE_SET_DSTXY_REG(((unsigned long)(SrcY + Height) << 16) + (unsigned long)(SrcX)); \
	}

/* Extend an 16 bits per pixels rectangle */
#define HWGE_2BPP_BOTTOMEXTEND_START(SrcX, SrcY, Width, Height, Size) { \
	HWGE_SET_ROP_REG(0xcc | HWGE_ROPREG_SRCSCRN);  \
	HWGE_SET_BPP_REG(0x1); 					\
	HWGE_SET_SRCXY_REG(((unsigned long)(SrcY) << 16) + (unsigned long)(SrcX) + (unsigned long)(SrcX)); \
	HWGE_SET_HEIGHT_REG(Size - Height - 1);				\
	HWGE_SET_WIDTH_REG(((unsigned long)(Width) << 1) - 1);					\
	HWGE_SET_DSTXY_REG(((unsigned long)(SrcY + Height) << 16) + ((unsigned long)(SrcX) << 1)); \
	}

/* Extend an 24 bits per pixels rectangle */
#define HWGE_3BPP_BOTTOMEXTEND_START(SrcX, SrcY, Width, Height, Size) { \
	HWGE_SET_ROP_REG(0xcc | HWGE_ROPREG_SRCSCRN);  \
	HWGE_SET_BPP_REG(0x2); 					\
	HWGE_SET_SRCXY_REG(((unsigned long)(SrcY) << 16) + (unsigned long)(SrcX) + (unsigned long)(SrcX) + (unsigned long)(SrcX)); \
	HWGE_SET_HEIGHT_REG(Size - Height - 1);				\
	HWGE_SET_WIDTH_REG(((unsigned long)(Width) << 1) + (unsigned long)(Width) - 1);					\
	HWGE_SET_DSTXY_REG((((unsigned long)(SrcY + Height)) << 16) + ((unsigned long)(SrcX) << 1) + (unsigned long)(SrcX)); \
	}

/* Extend an 32 bits per pixels rectangle */
#define HWGE_4BPP_BOTTOMEXTEND_START(SrcX, SrcY, Width, Height, Size) { \
	HWGE_SET_ROP_REG(0xcc | HWGE_ROPREG_SRCSCRN);  \
	HWGE_SET_BPP_REG(0x3); 					\
	HWGE_SET_SRCXY_REG(((unsigned long)(SrcY) << 16) + ((unsigned long)(SrcX) << 2)); \
	HWGE_SET_HEIGHT_REG(Size - Height - 1);				\
	HWGE_SET_WIDTH_REG(((unsigned long)(Width) << 2) - 1);					\
	HWGE_SET_DSTXY_REG(((unsigned long)(SrcY + Height) << 16) + ((unsigned long)(SrcX) << 2)); \
	}

/* ------------------------------------------------------------
// DEFINE		: HWGE_?BPP_EXTEND_START
// ------------------------------------------------------------
// DESCRIPTION	: Extend a rectangle of the screen by repeated
//				  it on right and bottom. Source surface and
//				  destination surface must be the same before
//				  use this functions. 
// ------------------------------------------------------------
// SrcX			: X coordinate of the rectangle to extend
// SrcY			: Y coordinate of the rectangle to extend
// Width		: Width of the rectangle to extend
// Height		: Height of the rectangle to extend
// SizeX		: Number of pixels of the horizontal extension, 
//				  including the source rectangle itself (this 
//				  mean SizeX must be higher than Width parameter)
// SizeY		: Number of pixels of the vertical extension, 
//				  including the source rectangle itself (this 
//				  mean SizeY must be higher than Height parameter)
// ------------------------------------------------------------*/

/* Extend an 8 bits per pixels rectangle */
#define HWGE_1BPP_EXTEND_START(SrcX, SrcY, Width, Height, SizeX, SizeY) {  \
	HWGE_1BPP_RIGHTEXTEND_START(SrcX, SrcY, Width, Height, SizeX); \
	HWGE_1BPP_BOTTOMEXTEND_START(SrcX, SrcY, SizeX, Height, SizeY); \
	}

/* Extend an 16 bits per pixels rectangle */
#define HWGE_2BPP_EXTEND_START(SrcX, SrcY, Width, Height, SizeX, SizeY) {  \
	HWGE_2BPP_RIGHTEXTEND_START(SrcX, SrcY, Width, Height, SizeX); \
	HWGE_2BPP_BOTTOMEXTEND_START(SrcX, SrcY, SizeX, Height, SizeY); \
	}

/* Extend an 24 bits per pixels rectangle */
#define HWGE_3BPP_EXTEND_START(SrcX, SrcY, Width, Height, SizeX, SizeY) {  \
	HWGE_3BPP_RIGHTEXTEND_START(SrcX, SrcY, Width, Height, SizeX); \
	HWGE_3BPP_BOTTOMEXTEND_START(SrcX, SrcY, SizeX, Height, SizeY); \
	}

/* Extend an 32 bits per pixels rectangle */
#define HWGE_4BPP_EXTEND_START(SrcX, SrcY, Width, Height, SizeX, SizeY) {  \
	HWGE_4BPP_RIGHTEXTEND_START(SrcX, SrcY, Width, Height, SizeX); \
	HWGE_4BPP_BOTTOMEXTEND_START(SrcX, SrcY, SizeX, Height, SizeY); \
	}

#endif
