/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: genedata.c
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libhard\generic
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: August, 25th 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Data associated with generic.c module
// ------------------------------------------------------------
// VARIABLES	: HWGENERIC_psKeywordList
// ------------------------------------------------------------
// MACROS		: -
// ------------------------------------------------------------
// FUNCTIONS	: -
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: August, 25th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Added STPC Atlas and HCMOS7 PLLs in the keyword list
// ------------------------------------------------------------
// REVISION		: Rev 1.0
// DATE			: July, 18th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Initial source code
// ------------------------------------------------------------*/

#include "crtc.h"
#include "generic.h"
#include "saa.h"

/* ------------------------------------------------------------
// VARIABLE		: HWGENERIC_psKeywordList
// ------------------------------------------------------------
// DESCRIPTION	: List of keyword generics to all the hardware
//				  library
// ------------------------------------------------------------*/
HWGENERIC_KEYWORDENTRY HWGENERIC_psKeywordList[] =
{
	/* Type of screen */
	{ "MONITOR",           HWCRTC_OUTPUT_MONITOR,           HWGENERIC_HAS_VGA, HWGENERIC_KEYWORDTYPE_SCREEN, "SVGA Monitor 640x480, 800x600 or 1024x768" },
	{ "100HZPAL",          HWCRTC_OUTPUT_100HZPAL,          HWGENERIC_HAS_VGA, HWGENERIC_KEYWORDTYPE_SCREEN, "TV 100Hz PAL 720x576 connected on VGA" },
	{ "VGANTSC",     HWCRTC_OUTPUT_VGANTSC,     HWGENERIC_HAS_VGA, HWGENERIC_KEYWORDTYPE_SCREEN, "NTSC TFT 480x234 or 400x234 connected on VGA" },
	{ "TVPAL",             HWCRTC_OUTPUT_TVPAL,             HWGENERIC_HAS_VGA, HWGENERIC_KEYWORDTYPE_SCREEN, "TV output PAL 640x480" },
	{ "TVNTSC",            HWCRTC_OUTPUT_TVNTSC,            HWGENERIC_HAS_TV,  HWGENERIC_KEYWORDTYPE_SCREEN, "TV output NTSC 640x430" },
	{ "TVNTSC269",         HWCRTC_OUTPUT_TVNTSC269,            HWGENERIC_HAS_TV,  HWGENERIC_KEYWORDTYPE_SCREEN, "TV output NTSC 640x430 with 26.937MHz clock" },
	{ "TVNTSCINTERNAL",         HWCRTC_OUTPUT_TVNTSCINTERNAL,            HWGENERIC_HAS_TV,  HWGENERIC_KEYWORDTYPE_SCREEN, "TV output NTSC 640x430 with internal clock" },
	{ "TVPALSVHS",         HWCRTC_OUTPUT_TVPALSVHS,             HWGENERIC_HAS_VGA, HWGENERIC_KEYWORDTYPE_SCREEN, "TV output PAL 640x480" },
	{ "TVNTSCSVHS",        HWCRTC_OUTPUT_TVNTSCSVHS,            HWGENERIC_HAS_TV,  HWGENERIC_KEYWORDTYPE_SCREEN, "TV output NTSC 640x430" },
	{ "TVNTSC269SVHS",     HWCRTC_OUTPUT_TVNTSC269SVHS,         HWGENERIC_HAS_TV,  HWGENERIC_KEYWORDTYPE_SCREEN, "TV output NTSC 640x430 with 26.937MHz clock" },
	{ "TFTPHILIPSHLD1201", HWCRTC_OUTPUT_TFTPHILIPSHLD1201, HWGENERIC_HAS_TFT, HWGENERIC_KEYWORDTYPE_SCREEN, "TFT Philips HLD1201 800x600" },
	{ "TFTLGPHILIPSLM151X2", HWCRTC_OUTPUT_TFTLGPHILIPSLM151X2, HWGENERIC_HAS_TFT, HWGENERIC_KEYWORDTYPE_SCREEN, "TFT LG-Philips LM151x2 1024x768" },

	/* Type of platform */
	{ "GLORIA", HWGENERIC_PLATFORM_GLORIA, HWGENERIC_PLATFORM_GLORIA,     HWGENERIC_KEYWORDTYPE_PLATFORM, "Gloria board" },
	{ "EVAL110", HWGENERIC_PLATFORM_EVAL110, HWGENERIC_PLATFORM_EVAL110,    HWGENERIC_KEYWORDTYPE_PLATFORM, "EVAL110 board" },
	{ "EVAL120", HWGENERIC_PLATFORM_EVAL120, HWGENERIC_PLATFORM_EVAL120,    HWGENERIC_KEYWORDTYPE_PLATFORM, "EVAL120 board" },
	{ "EVAL200PC", HWGENERIC_PLATFORM_EVAL200PC, HWGENERIC_PLATFORM_EVAL200PC,  HWGENERIC_KEYWORDTYPE_PLATFORM, "EVAL200PC board" },
	{ "EVAL200EMB", HWGENERIC_PLATFORM_EVAL200EMB, HWGENERIC_PLATFORM_EVAL200EMB, HWGENERIC_KEYWORDTYPE_PLATFORM, "EVAL200EMB board" },
	{ "EVAL130", HWGENERIC_PLATFORM_EVAL130, HWGENERIC_PLATFORM_EVAL130,    HWGENERIC_KEYWORDTYPE_PLATFORM, "EVAL130 board" },
	{ "EVAL210", HWGENERIC_PLATFORM_EVAL210, HWGENERIC_PLATFORM_EVAL210,    HWGENERIC_KEYWORDTYPE_PLATFORM, "EVAL210 board" },

	/* Type of chip */
	{ "", 0, HWGENERIC_PLATFORM_GLORIA,  HWGENERIC_KEYWORDTYPE_CHIP, "STPC Client" },
	{ "", 0, HWGENERIC_PLATFORM_EVAL110, HWGENERIC_KEYWORDTYPE_CHIP, "STPC Consumer" },
	{ "", 0, HWGENERIC_PLATFORM_EVAL120, HWGENERIC_KEYWORDTYPE_CHIP, "STPC Consumer-S" },
	{ "", 0, HWGENERIC_PLATFORM_EVAL130, HWGENERIC_KEYWORDTYPE_CHIP, "STPC Consumer-II" },
	{ "", 0, HWGENERIC_PLATFORM_EVAL140, HWGENERIC_KEYWORDTYPE_CHIP, "STPC Consumer-IIA" },
	{ "", 0, HWGENERIC_PLATFORM_EVAL200PC | HWGENERIC_PLATFORM_EVAL200EMB, HWGENERIC_KEYWORDTYPE_CHIP, "STPC Industrial" },
	{ "", 0, HWGENERIC_PLATFORM_EVAL210, HWGENERIC_KEYWORDTYPE_CHIP, "STPC Atlas" },

	/* Type of hardware features */
	{ "", 0, HWGENERIC_HAS_EDO, HWGENERIC_KEYWORDTYPE_FEATURE, "EDO controller" },
	{ "", 0, HWGENERIC_HAS_SDRAM, HWGENERIC_KEYWORDTYPE_FEATURE, "SDRAM controller" },
	{ "", 0, HWGENERIC_HAS_VGA, HWGENERIC_KEYWORDTYPE_FEATURE, "SVGA controller" },
	{ "", 0, HWGENERIC_HAS_TFT, HWGENERIC_KEYWORDTYPE_FEATURE, "TFT interface" },
	{ "", 0, HWGENERIC_HAS_IOTV, HWGENERIC_KEYWORDTYPE_FEATURE, "TV encoder accessible via I/O" },
	{ "", 0, HWGENERIC_HAS_MEMTV, HWGENERIC_KEYWORDTYPE_FEATURE, "TV encoder accessible via memory" },
	{ "", 0, HWGENERIC_HAS_EXTTV, HWGENERIC_KEYWORDTYPE_FEATURE, "TV encoder accessible via i2c" },
	{ "", 0, HWGENERIC_HAS_GE, HWGENERIC_KEYWORDTYPE_FEATURE, "2D graphics accelerator" },
	{ "", 0, HWGENERIC_HAS_AFE, HWGENERIC_KEYWORDTYPE_FEATURE, "Alpha Font Engine" },
	{ "", 0, HWGENERIC_HAS_LDE, HWGENERIC_KEYWORDTYPE_FEATURE, "Line drawing Engine" },
	{ "", 0, HWGENERIC_HAS_HWCURSOR, HWGENERIC_KEYWORDTYPE_FEATURE, "Hardware cursor" },
	{ "", 0, HWGENERIC_HAS_VPIPE, HWGENERIC_KEYWORDTYPE_FEATURE, "Video overlay" },
	{ "", 0, HWGENERIC_HAS_VIP, HWGENERIC_KEYWORDTYPE_FEATURE, "Video Input Port" },
	{ "", 0, HWGENERIC_HAS_SMCSAA, HWGENERIC_KEYWORDTYPE_FEATURE, "SAA encoder on SMC GPIO" },
	{ "", 0, HWGENERIC_HAS_DDCSAA, HWGENERIC_KEYWORDTYPE_FEATURE, "SAA encoder on DDC" },
	{ "", 0, HWGENERIC_HAS_HCMOS6PLL, HWGENERIC_KEYWORDTYPE_FEATURE, "HCMOS6 PLLs" },
	{ "", 0, HWGENERIC_HAS_HCMOS7LEGACYPLL, HWGENERIC_KEYWORDTYPE_FEATURE, "HCMOS7 legacy PLLs" },
	{ "", 0, HWGENERIC_HAS_HCMOS7NEWPLL, HWGENERIC_KEYWORDTYPE_FEATURE, "HCMOS7 new PLLs" },
	{ "", 0, HWGENERIC_HAS_NTSC27MHZ, HWGENERIC_KEYWORDTYPE_FEATURE, "NTSC format with 27MHz oscillator" },

	/* Type of video input stream */
	{ "PAL",       HWSAA_FORMAT_PAL,        HWGENERIC_ANY, HWGENERIC_KEYWORDTYPE_VIDEOINPUT, "PAL"},
	{ "PALBGHI",   HWSAA_FORMAT_PALBGHI,    HWGENERIC_ANY, HWGENERIC_KEYWORDTYPE_VIDEOINPUT, "PAL BGHI"},
	{ "NTSC",      HWSAA_FORMAT_NTSC,       HWGENERIC_ANY, HWGENERIC_KEYWORDTYPE_VIDEOINPUT, "NTSC"},
	{ "NTSCM",     HWSAA_FORMAT_NTSCM,      HWGENERIC_ANY, HWGENERIC_KEYWORDTYPE_VIDEOINPUT, "NTSC M"},
	{ "NTSCJAPAN", HWSAA_FORMAT_NTSC_JAPAN, HWGENERIC_ANY, HWGENERIC_KEYWORDTYPE_VIDEOINPUT, "NTSC Japan"},
	{ "PAL443",    HWSAA_FORMAT_PAL_443,    HWGENERIC_ANY, HWGENERIC_KEYWORDTYPE_VIDEOINPUT, "PAL 443"},
	{ "PALM",      HWSAA_FORMAT_PALM,       HWGENERIC_ANY, HWGENERIC_KEYWORDTYPE_VIDEOINPUT, "PAL M"},
	{ "NTSCN",     HWSAA_FORMAT_NTSCN,      HWGENERIC_ANY, HWGENERIC_KEYWORDTYPE_VIDEOINPUT, "NTSC N"},
	{ "PALN",      HWSAA_FORMAT_PALN,       HWGENERIC_ANY, HWGENERIC_KEYWORDTYPE_VIDEOINPUT, "PAL N"},
	{ "SECAM",     HWSAA_FORMAT_SECAM,      HWGENERIC_ANY, HWGENERIC_KEYWORDTYPE_VIDEOINPUT, "SECAM"},

	{ NULL, 0, 0, 0, NULL}
};

