/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: generic.h
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libhard\generic
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.2
// DATE			: August, 25th 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Generic functions to manage STPC chipset
// ------------------------------------------------------------
// VARIABLES	: HWGENERIC_psKeywordList
// ------------------------------------------------------------
// MACROS		: HWGENERIC_INVALIDKEYWORD
//				  HWGENERIC_NOENTRY
//				  HWGENERIC_ANY
//				  HWGENERIC_PLATFORM_...
//				  HWGENERIC_HAS_...
//				  HWGENERIC_KEYWORDTYPE_...
//				  HWGENERIC_KEYWORDENTRY
//				  HWGENERIC_GDKVERSION
// ------------------------------------------------------------
// FUNCTIONS	: HWGENERIC_GetHardwareID
//				  HWGENERIC_GetGDKMajorVersion
//				  HWGENERIC_GetGDKMinorVersion
//				  HWGENERIC_GetNumberOfEntry
//				  HWGENERIC_GetFirstEntry
//				  HWGENERIC_GetNextEntry
//				  HWGENERIC_GetEntryFromKeyword
//				  HWGENERIC_GetValueFromKeyword
//				  HWGENERIC_GetFullNameFromKeyword
//				  HWGENERIC_GetEntry
//				  HWGENERIC_GetValue
//				  HWGENERIC_GetFullName
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.2
// DATE			: August, 25th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Add STPC Atlas and HCMOS7 PLLs
// ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: July, 18th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Add version number functions : 
//				  HWCRTC_GetGDKMinorVersion
//				  HWCRTC_GetGDKMajorVersion
//				  Add function to manage the list of keyword
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Initial source code
// ------------------------------------------------------------*/

#ifndef GENERIC_H
#define GENERIC_H

/* ------------------------------------------------------------
// DEFINE		: HWGENERIC_GDKVERSION
// ------------------------------------------------------------
// DESCRIPTION	: Current version of GDK (major in high 16 bits
//			  minor in low 16 bits)
// ------------------------------------------------------------*/
#define HWGENERIC_GDKVERSION		0x00010004

/* ------------------------------------------------------------
// DEFINE		: HWGENERIC_ANY
// ------------------------------------------------------------
// DESCRIPTION	: Used to select any type of hardware or feature
// ------------------------------------------------------------*/
#define HWGENERIC_ANY					0xffffffff

/* ------------------------------------------------------------
// DEFINE		: HWGENERIC_PLATFORM_...
// ------------------------------------------------------------
// DESCRIPTION	: Hardware platform ID
// ------------------------------------------------------------
// HWGENERIC_PLATFORM_GLORIA		: Gloria or Evmini board
// HWGENERIC_PLATFORM_EVAL110		: EVAL110 board
// HWGENERIC_PLATFORM_EVAL120		: EVAL120 board
// HWGENERIC_PLATFORM_EVAL200PC		: EVAL200 PC board
// HWGENERIC_PLATFORM_EVAL200EMB	: EVAL200 Embedded board
// ------------------------------------------------------------*/
#define HWGENERIC_PLATFORM_GLORIA		0x0001
#define HWGENERIC_PLATFORM_EVAL110		0x0002
#define HWGENERIC_PLATFORM_EVAL120		0x0004
#define HWGENERIC_PLATFORM_EVAL200PC	0x0008
#define HWGENERIC_PLATFORM_EVAL200EMB	0x0010
#define HWGENERIC_PLATFORM_EVAL130		0x0020
#define HWGENERIC_PLATFORM_EVAL210		0x0040
#define HWGENERIC_PLATFORM_EVAL140		0x0080

/* ------------------------------------------------------------
// DEFINE		: HWGENERIC_HAS_...
// ------------------------------------------------------------
// HWGENERIC_HAS_EDO	: This platform use EDO memory
// HWGENERIC_HAS_SDRAM	: This platform use SDRAM memory
// HWGENERIC_HAS_VGA	: This platform has an internal VGA controller
// HWGENERIC_HAS_TFT	: This platform has a TFT controller
// HWGENERIC_HAS_IOTV	: This platform has an internal TV encoder
//						  accessed by I/O based registers
// HWGENERIC_HAS_MEMTV	: This platform has an on board TV encoder
//						  accessed by memory based registers
// HWGENERIC_HAS_EXTTV	: This platform has an on board TV encoder
// HWGENERIC_HAS_PPSAA	: This platform has an SAA on parallel port
// HWGENERIC_HAS_SMCSAA	: This platform has an SAA on SMC super I/O
// HWGENERIC_HAS_DDCSAA	: This platform has an SAA on VGA connector
// ------------------------------------------------------------*/
#define HWGENERIC_HAS_EDO				(HWGENERIC_PLATFORM_GLORIA | HWGENERIC_PLATFORM_EVAL110 | HWGENERIC_PLATFORM_EVAL200PC | HWGENERIC_PLATFORM_EVAL200EMB)
#define HWGENERIC_HAS_SDRAM				(HWGENERIC_PLATFORM_EVAL120 | HWGENERIC_PLATFORM_EVAL130 | HWGENERIC_PLATFORM_EVAL140 | HWGENERIC_PLATFORM_EVAL210)
#define HWGENERIC_HAS_VGA				(HWGENERIC_PLATFORM_GLORIA | HWGENERIC_PLATFORM_EVAL110 | HWGENERIC_PLATFORM_EVAL120 | HWGENERIC_PLATFORM_EVAL140 | HWGENERIC_PLATFORM_EVAL130 | HWGENERIC_PLATFORM_EVAL200PC | HWGENERIC_PLATFORM_EVAL200EMB | HWGENERIC_PLATFORM_EVAL210)
#define HWGENERIC_HAS_IOTFT				(HWGENERIC_PLATFORM_EVAL200PC | HWGENERIC_PLATFORM_EVAL200EMB )
#define HWGENERIC_HAS_MEMTFT			(HWGENERIC_PLATFORM_EVAL210)
#define HWGENERIC_HAS_IOTV				(HWGENERIC_PLATFORM_EVAL110)
#define HWGENERIC_HAS_MEMTV				(HWGENERIC_PLATFORM_EVAL120 | HWGENERIC_PLATFORM_EVAL130 | HWGENERIC_PLATFORM_EVAL140)
#define HWGENERIC_HAS_EXTTV				(HWGENERIC_PLATFORM_GLORIA)
#define HWGENERIC_HAS_PPSAA				(HWGENERIC_ANY)
#define HWGENERIC_HAS_SMCSAA			(HWGENERIC_PLATFORM_EVAL110)
#define HWGENERIC_HAS_DDCSAA			(HWGENERIC_PLATFORM_EVAL120 | HWGENERIC_PLATFORM_EVAL130 | HWGENERIC_PLATFORM_EVAL140 | HWGENERIC_PLATFORM_EVAL210)
#define HWGENERIC_HAS_VIP				(HWGENERIC_PLATFORM_GLORIA | HWGENERIC_PLATFORM_EVAL110 | HWGENERIC_PLATFORM_EVAL120 | HWGENERIC_PLATFORM_EVAL130 | HWGENERIC_PLATFORM_EVAL140 | HWGENERIC_PLATFORM_EVAL130 | HWGENERIC_PLATFORM_EVAL200PC | HWGENERIC_PLATFORM_EVAL200EMB | HWGENERIC_PLATFORM_EVAL210)
#define HWGENERIC_HAS_AFE				(HWGENERIC_PLATFORM_EVAL210)
#define HWGENERIC_HAS_LDE				(HWGENERIC_PLATFORM_EVAL210)
#define HWGENERIC_HAS_GE				(HWGENERIC_PLATFORM_GLORIA | HWGENERIC_PLATFORM_EVAL110 | HWGENERIC_PLATFORM_EVAL120 | HWGENERIC_PLATFORM_EVAL130 | HWGENERIC_PLATFORM_EVAL140 | HWGENERIC_PLATFORM_EVAL130 | HWGENERIC_PLATFORM_EVAL200PC | HWGENERIC_PLATFORM_EVAL200EMB | HWGENERIC_PLATFORM_EVAL210)
#define HWGENERIC_HAS_HWCURSOR			(HWGENERIC_PLATFORM_GLORIA | HWGENERIC_PLATFORM_EVAL110 | HWGENERIC_PLATFORM_EVAL120 | HWGENERIC_PLATFORM_EVAL130 | HWGENERIC_PLATFORM_EVAL140 | HWGENERIC_PLATFORM_EVAL130 | HWGENERIC_PLATFORM_EVAL200PC | HWGENERIC_PLATFORM_EVAL200EMB | HWGENERIC_PLATFORM_EVAL210)
#define HWGENERIC_HAS_VPIPE				(HWGENERIC_PLATFORM_GLORIA | HWGENERIC_PLATFORM_EVAL110 | HWGENERIC_PLATFORM_EVAL120 | HWGENERIC_PLATFORM_EVAL130 | HWGENERIC_PLATFORM_EVAL140 | HWGENERIC_PLATFORM_EVAL130 | HWGENERIC_PLATFORM_EVAL200PC | HWGENERIC_PLATFORM_EVAL200EMB | HWGENERIC_PLATFORM_EVAL210)
#define HWGENERIC_HAS_HCMOS6PLL			(HWGENERIC_PLATFORM_GLORIA | HWGENERIC_PLATFORM_EVAL110 | HWGENERIC_PLATFORM_EVAL200PC | HWGENERIC_PLATFORM_EVAL200EMB | HWGENERIC_PLATFORM_EVAL120)
#define HWGENERIC_HAS_HCMOS7LEGACYPLL	(0)
#define HWGENERIC_HAS_HCMOS7NEWPLL		(HWGENERIC_PLATFORM_EVAL130 | HWGENERIC_PLATFORM_EVAL140 | HWGENERIC_PLATFORM_EVAL210)
#define HWGENERIC_HAS_STVINVERTEDPOLARITY	(HWGENERIC_PLATFORM_EVAL130 | HWGENERIC_PLATFORM_EVAL140)
#define HWGENERIC_HAS_STVV8				(HWGENERIC_PLATFORM_EVAL130 | HWGENERIC_PLATFORM_EVAL140)
#define HWGENERIC_HAS_NTSC27MHZ			(HWGENERIC_PLATFORM_EVAL140)

#define HWGENERIC_HAS_TV				(HWGENERIC_HAS_IOTV | HWGENERIC_HAS_MEMTV | HWGENERIC_HAS_EXTTV)
#define HWGENERIC_HAS_TFT				(HWGENERIC_HAS_IOTFT | HWGENERIC_HAS_MEMTFT)

/* ------------------------------------------------------------
// DEFINE		: HWGENERIC_INVALIDKEYWORD
// ------------------------------------------------------------
// DESCRIPTION	: This keyword is unknown
// ------------------------------------------------------------*/
#define HWGENERIC_INVALIDKEYWORD	0xffffffff

/* ------------------------------------------------------------
// DEFINE		: HWGENERIC_NOENTRY
// ------------------------------------------------------------
// DESCRIPTION	: No entry found
// ------------------------------------------------------------*/
#define HWGENERIC_NOENTRY			0xffffffff

/* ------------------------------------------------------------
// DEFINE		: HWGENERIC_KEYWORDTYPE_...
// ------------------------------------------------------------
// DESCRIPTION	: Type of keyword
// ------------------------------------------------------------*/
#define HWGENERIC_KEYWORDTYPE_PLATFORM		0x0001
#define HWGENERIC_KEYWORDTYPE_CHIP			0x0002
#define HWGENERIC_KEYWORDTYPE_FEATURE		0x0004
#define HWGENERIC_KEYWORDTYPE_SCREEN		0x0008
#define HWGENERIC_KEYWORDTYPE_VIDEOINPUT    0x0010

/* ------------------------------------------------------------
// STRUCTURE	: HWGENERIC_KEYWORDENTRY
// ------------------------------------------------------------
// DESCRIPTION	: Description of one value of an enumeration
// ------------------------------------------------------------
// ulValue		: Value associated to this entry
// pucKeyword	: Keyword associated to this entry
// pucFullName	: Description  associated to this entry
// ------------------------------------------------------------*/
typedef struct tagHWGENERIC_KEYWORDENTRY
{
	unsigned char *pucKeyword;
	unsigned long ulValue;
	unsigned long ulSupportedHardware;
	unsigned long ulType;
	unsigned char *pucFullName;
}HWGENERIC_KEYWORDENTRY;

/* ------------------------------------------------------------
// VARIABLE		: HWGENERIC_psKeywordList
// ------------------------------------------------------------
// DESCRIPTION	: List of keyword generics to all the hardware
//				  library
// ------------------------------------------------------------*/
extern HWGENERIC_KEYWORDENTRY HWGENERIC_psKeywordList[];

/* ------------------------------------------------------------
// FUNCTION		: HWGENERIC_GetGDKMajorVersion
// ------------------------------------------------------------
// DESCRIPTION	: Get version of current GDK
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Major part of the version number
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define HWGENERIC_GetGDKMajorVersion()		(HWGENERIC_GDKVERSION >> 16)

/* ------------------------------------------------------------
// FUNCTION		: HWGENERIC_GetGDKMinorVersion
// ------------------------------------------------------------
// DESCRIPTION	: Get version of current GDK
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Minor part of the version number
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define HWGENERIC_GetGDKMinorVersion()		(HWGENERIC_GDKVERSION & 0xffff)

/* ------------------------------------------------------------
// FUNCTION		: HWGENERIC_SetHardwareID
// ------------------------------------------------------------
// DESCRIPTION	: Fix the hardware detection manually
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulHardwareID		: Hardware ID to fix (HWGENERIC_PLATFORM_...)
//					  0xffffffff to let the library check
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWGENERIC_SetHardwareID(unsigned long ulHardwareID);

/* ------------------------------------------------------------
// FUNCTION		: HWGENERIC_GetHardwareID
// ------------------------------------------------------------
// DESCRIPTION	: Get an ID of the current hardware
//				  platform. This ID can be used with one of
//				  the HWGENERIC_IS_... value to identify the
//				  type of STPC (OR operation) and with one
//				  of the HWGENERIC_HAS_... value to understand
//				  if one feature is available on this platform
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Identificateur of the hardware : one of the
//				  HWGENERIC_PLATFORM_... parameter
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWGENERIC_GetHardwareID(void);

/* ------------------------------------------------------------
// FUNCTION		: HWGENERIC_GetNumberOfEntry
// ------------------------------------------------------------
// DESCRIPTION	: Get the number of values in the list
//				  corresponding to a specific type of value
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulType		: Type of value (HWGENERIC_KEYWORDTYPE_...)
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Number of entries
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWGENERIC_GetNumberOfEntry(unsigned long ulType);

/* ------------------------------------------------------------
// FUNCTION		: HWGENERIC_GetFirstEntry
// ------------------------------------------------------------
// DESCRIPTION	: Get the index of the first entry corresponding
//				  to a type of value
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulType		: Type of value (HWGENERIC_KEYWORDTYPE_...)
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Index in the table of the first entry
//				  HWGENERIC_NOENTRY if no entry
// ------------------------------------------------------------
// MODIFIED		: HWGENERIC_ulCurrentEntry
// ------------------------------------------------------------*/
unsigned long HWGENERIC_GetFirstEntry(unsigned long ulType);

/* ------------------------------------------------------------
// FUNCTION		: HWGENERIC_GetNextEntry
// ------------------------------------------------------------
// DESCRIPTION	: Get the index of the next entry corresponding
//				  to a type of value
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Index of the entry in the table
//				  HWGENERIC_INVALIDKEYWORD if no entry
// ------------------------------------------------------------
// MODIFIED		: HWGENERIC_ulCurrentEntry
// ------------------------------------------------------------*/
unsigned long HWGENERIC_GetNextEntry(void);

/* ------------------------------------------------------------
// FUNCTION		: HWGENERIC_GetEntryFromKeyword
// ------------------------------------------------------------
// DESCRIPTION	: Get the entry number in the list of a value
//				  identified by its keyword
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// pucKeyword	: Keyword which identify the value
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Entry number, HWGENERIC_INVALIDKEYWORD if not found
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWGENERIC_GetEntryFromKeyword(unsigned char *pucKeyword);

/* ------------------------------------------------------------
// FUNCTION		: HWGENERIC_GetValue
// ------------------------------------------------------------
// DESCRIPTION	: Get the value associated to an entry
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulEntry		: Entry number
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Value
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define HWGENERIC_GetValue(ulEntry)	(HWGENERIC_psKeywordList[ulEntry].ulValue)

/* ------------------------------------------------------------
// FUNCTION		: HWGENERIC_GetKeyword
// ------------------------------------------------------------
// DESCRIPTION	: Get the keyword associated to an entry
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulEntry		: Entry number
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Keyword
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define HWGENERIC_GetKeyword(ulEntry)	(HWGENERIC_psKeywordList[ulEntry].pucKeyword)

/* ------------------------------------------------------------
// FUNCTION		: HWGENERIC_GetFullName
// ------------------------------------------------------------
// DESCRIPTION	: Get the description associated to an entry
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulEntry		: Entry number
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Description
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define HWGENERIC_GetFullName(ulEntry)	(HWGENERIC_psKeywordList[ulEntry].pucFullName)

/* ------------------------------------------------------------
// FUNCTION		: HWGENERIC_GetValueFromKeyword
// ------------------------------------------------------------
// DESCRIPTION	: Get the value associated to an entry
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// pucKeyword	: Keyword
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Value
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define HWGENERIC_GetValueFromKeyword(pucKeyword)	(HWGENERIC_psKeywordList[HWGENERIC_GetEntryFromKeyword(pucKeyword)].ulValue)

/* ------------------------------------------------------------
// FUNCTION		: HWGENERIC_GetFullNameFromKeyword
// ------------------------------------------------------------
// DESCRIPTION	: Get the description associated to an entry
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// pucKeyword	: Keyword
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Description
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define HWGENERIC_GetFullNameFromKeyword(pucKeyword)	(HWGENERIC_psKeywordList[HWGENERIC_GetEntryFromKeyword(pucKeyword)].pucFullName)

#endif
