/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: host.h
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libreg\host
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.2
// DATE			: December, 6th 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Functions to manage access to the CPU host
//				  bus (I/O and memory address space)
// ------------------------------------------------------------
// VARIABLES	: REGHOST_ChipsetResources
// ------------------------------------------------------------
// MACROS		: REGHOST_LIB_... 
//				  REGHOST_TYPE_... 
//				  REGHOST_BUS_RESOURCE 
//				  REGHOST_CHIPSET_RESOURCES 
// 				  REGHOST_VIRTUAL_... 
// 				  REGHOST_GET_... 
//				  REGHOST_SET_... 
// 				  REGHOST_Get...Pointer
// ------------------------------------------------------------
// FUNCTIONS	: REGHOST_Init
//				  REGHOST_GetGBASE
//				  REGHOST_IndexedIORead8
//				  REGHOST_IndexedIOWrite8
//				  REGHOST_IndexedIORead2x8
//				  REGHOST_IndexedIOWrite2x8
//				  REGHOST_TFTInterfaceWrite16
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.2
// DATE			: December, 6th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Add access to STPC Atlas: Alpha Font Engine,
//				  Line Drawing Engine and memory mapped TFT
// ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: July, 18th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Add macros to access SMC registers
// ------------------------------------------------------------
// REVISION		: Rev 1.0
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

#ifndef REGISTER_H
#define REGISTER_H

#include "osal.h"

/* ------------------------------------------------------------
// DEFINE		: REGHOST_LIB_... 
// ------------------------------------------------------------
// DESCRIPTION	: Different modules of the Development kit
// ------------------------------------------------------------
// REGHOST_LIB_CRTC			: Graphic initialization
// REGHOST_LIB_FB			: Frame buffer management
// REGHOST_LIB_GE			: Graphic Engine
// REGHOST_LIB_HWCURSOR		: Harulare cursor
// REGHOST_LIB_TFT			: TFT interface
// REGHOST_LIB_TVENCODER	: TV encoder
// REGHOST_LIB_TVOUTPUT		: TV output
// REGHOST_LIB_VIP			: Video Input Port
// REGHOST_LIB_VPIPE		: Video Pipeline
// REGHOST_LIB_ALL			: All the module
// ------------------------------------------------------------*/
#define		REGHOST_LIB_CRTC			0x0001
#define		REGHOST_LIB_FB				0x0002
#define		REGHOST_LIB_GE				0x0004
#define		REGHOST_LIB_HWCURSOR		0x0008
#define		REGHOST_LIB_TFT				0x0010
#define		REGHOST_LIB_TVENCODER		0x0020
#define		REGHOST_LIB_TVOUTPUT		0x0040
#define		REGHOST_LIB_VIP				0x0080
#define		REGHOST_LIB_VPIPE			0x0100
#define		REGHOST_LIB_LDE				0x0200
#define		REGHOST_LIB_AFE				0x0400
#define		REGHOST_LIB_ALL				0xffff

/* ------------------------------------------------------------
// DEFINE		: REGHOST_TYPE_... 
// ------------------------------------------------------------
// DESCRIPTION	: Flag to describe an entry of the I/O and
//				  memory register table
// ------------------------------------------------------------
// REGHOST_TYPE_END_OF_LIST	: End of the resource list
// REGHOST_TYPE_IO			: The resource is an I/O
// REGHOST_TYPE_MEMORY		: The resource is a memory range
// REGHOST_TYPE_FIXED		: The resource address is not moveable
// ------------------------------------------------------------*/
#define		REGHOST_TYPE_END_OF_LIST	0xffffffff
#define		REGHOST_TYPE_IO				0x80000000
#define		REGHOST_TYPE_MEMORY			0x40000000
#define		REGHOST_TYPE_FIXED			0x20000000
#define		REGHOST_TYPE_GBASE			( 0x10000000 | REGHOST_TYPE_MEMORY )

/* ------------------------------------------------------------
// STRUCTURE	: REGHOST_BUS_RESOURCE
// ------------------------------------------------------------
// DESCRIPTION	: Structure to describe a resource of the table
// ------------------------------------------------------------
// ulType		: Flag to describe the resource (REGHOST_TYPE...)
// ulAddress	: Base address of the resource
// ulSize		: Range address of the resource
// ulVirtual	: CPU logical address to access the resource
// ------------------------------------------------------------*/
typedef struct tagREGHOST_BUS_RESOURCE
{
	unsigned long ulType;
	unsigned long ulAddress;
	unsigned long ulSize;
	unsigned long ulVirtual;
}REGHOST_BUS_RESOURCE;

/* ------------------------------------------------------------
// STRUCTURE	: REGHOST_CHIPSET_RESOURCES 
// ------------------------------------------------------------
// DESCRIPTION	: List of bus resource of the whole chipset
// ------------------------------------------------------------
// InputStatus		: Input status register
// MotherBoard		: Motherboard register
// VideoSubSystem1	: Video sub system register 1
// VideoSubSystem2	: Video sub system register 2
// Miscellaneous	: Miscellaneous graphic regiter
// Attribut			: Attribut controller register
// PaletteMask		: Palette mask register
// RAMDACIndex		: RAMDAC index register
// RAMDACData		: RAMDAC data register
// Sequencer		: Sequencer registers
// Chipset			: Chipset registers
// GraphicController: Graphic Controller registers
// CRTC				: CRTC registers
// SMCSuperIO		: SMC Super I/O registers
// ParallelPort		: Parrallel port register
// TFTInterface		: TFT interface registers
// GraphicEngine	: Graphic Engine registers
// DataPort			: Graphic Engine Data Port
// VideoInputPort	: Video Input Port registers
// VideoPipeline	: Video Pipeline registers
// OnScreenFrameBuffer	: On screen frame buffer
// OffScreenFrameBuffer : Off screen frame buffer
// ------------------------------------------------------------*/
typedef struct tagREGHOST_CHIPSET_RESOURCES
{
	REGHOST_BUS_RESOURCE InputStatus;
	REGHOST_BUS_RESOURCE MotherBoard;
	REGHOST_BUS_RESOURCE VideoSubSystem1;
	REGHOST_BUS_RESOURCE VideoSubSystem2;
	REGHOST_BUS_RESOURCE Miscellaneous;
	REGHOST_BUS_RESOURCE MiscellaneousRead;
	REGHOST_BUS_RESOURCE Attribut;
	REGHOST_BUS_RESOURCE PaletteMask;
	REGHOST_BUS_RESOURCE RAMDACIndex;
	REGHOST_BUS_RESOURCE RAMDACReadIndex;
	REGHOST_BUS_RESOURCE RAMDACData;
	REGHOST_BUS_RESOURCE Sequencer;
	REGHOST_BUS_RESOURCE Chipset;
	REGHOST_BUS_RESOURCE GraphicController;
	REGHOST_BUS_RESOURCE CRTC;
	REGHOST_BUS_RESOURCE SMCGPIO;
	REGHOST_BUS_RESOURCE SMCSuperIO;
	REGHOST_BUS_RESOURCE ParallelPort;
	REGHOST_BUS_RESOURCE TFTInterface;
	REGHOST_BUS_RESOURCE PCIConfigAddress;
	REGHOST_BUS_RESOURCE PCIConfigData;
	REGHOST_BUS_RESOURCE GraphicEngine;
	REGHOST_BUS_RESOURCE AlphaFontEngine;
	REGHOST_BUS_RESOURCE LineDrawingEngine;
	REGHOST_BUS_RESOURCE TFTInterfaceMem;
	REGHOST_BUS_RESOURCE DataPort;
	REGHOST_BUS_RESOURCE VideoInputPort;
	REGHOST_BUS_RESOURCE VideoPipeline;
	REGHOST_BUS_RESOURCE TVOutput;
	REGHOST_BUS_RESOURCE OnScreenFrameBuffer;
	REGHOST_BUS_RESOURCE OffScreenFrameBuffer;

	REGHOST_BUS_RESOURCE EndOfTable;
}REGHOST_CHIPSET_RESOURCES;

/* ------------------------------------------------------------
// DEFINE		: REGHOST_VIRTUAL_... 
// ------------------------------------------------------------
// DESCRIPTION	: CPU logical address of the different 
//				  I/O and memory range
// ------------------------------------------------------------*/
#define		REGHOST_VIRTUAL_INPUTSTATUS					(REGHOST_ChipsetResources.InputStatus.ulVirtual)
#define		REGHOST_VIRTUAL_MOTHERBOARD					(REGHOST_ChipsetResources.MotherBoard.ulVirtual)
#define		REGHOST_VIRTUAL_VIDEOSUBSYSTEM1				(REGHOST_ChipsetResources.VideoSubSystem1.ulVirtual)
#define		REGHOST_VIRTUAL_VIDEOSUBSYSTEM2				(REGHOST_ChipsetResources.VideoSubSystem2.ulVirtual)
#define		REGHOST_VIRTUAL_MISCELLANEOUS				(REGHOST_ChipsetResources.Miscellaneous.ulVirtual)
#define		REGHOST_VIRTUAL_MISCELLANEOUSREAD			(REGHOST_ChipsetResources.MiscellaneousRead.ulVirtual)
#define		REGHOST_VIRTUAL_ATTRIBUT					(REGHOST_ChipsetResources.Attribut.ulVirtual)
#define		REGHOST_VIRTUAL_PALETTEMASK					(REGHOST_ChipsetResources.PaletteMask.ulVirtual)
#define		REGHOST_VIRTUAL_RAMDACINDEX					(REGHOST_ChipsetResources.RAMDACIndex.ulVirtual)
#define		REGHOST_VIRTUAL_RAMDACREADINDEX				(REGHOST_ChipsetResources.RAMDACReadIndex.ulVirtual)
#define		REGHOST_VIRTUAL_RAMDACDATA					(REGHOST_ChipsetResources.RAMDACData.ulVirtual)
#define		REGHOST_VIRTUAL_SEQUENCER					(REGHOST_ChipsetResources.Sequencer.ulVirtual)
#define		REGHOST_VIRTUAL_CHIPSET						(REGHOST_ChipsetResources.Chipset.ulVirtual)
#define		REGHOST_VIRTUAL_GRAPHICCONTROLLER			(REGHOST_ChipsetResources.GraphicController.ulVirtual)
#define		REGHOST_VIRTUAL_CRTC						(REGHOST_ChipsetResources.CRTC.ulVirtual)
#define		REGHOST_VIRTUAL_SMCGPIO						(REGHOST_ChipsetResources.SMCGPIO.ulVirtual)
#define		REGHOST_VIRTUAL_SMCSUPERIO					(REGHOST_ChipsetResources.SMCSuperIO.ulVirtual)
#define		REGHOST_VIRTUAL_PARALLELPORT				(REGHOST_ChipsetResources.ParallelPort.ulVirtual)
#define		REGHOST_VIRTUAL_TFTINTERFACE				(REGHOST_ChipsetResources.TFTInterface.ulVirtual)
#define		REGHOST_VIRTUAL_PCICONFIGADDRESS			(REGHOST_ChipsetResources.PCIConfigAddress.ulVirtual)
#define		REGHOST_VIRTUAL_PCICONFIGDATA				(REGHOST_ChipsetResources.PCIConfigData.ulVirtual)
#define		REGHOST_VIRTUAL_TFTINTERFACE				(REGHOST_ChipsetResources.TFTInterface.ulVirtual)
#define		REGHOST_VIRTUAL_GE							(REGHOST_ChipsetResources.GraphicEngine.ulVirtual)
#define		REGHOST_VIRTUAL_AFE							(REGHOST_ChipsetResources.AlphaFontEngine.ulVirtual)
#define		REGHOST_VIRTUAL_LDE							(REGHOST_ChipsetResources.LineDrawingEngine.ulVirtual)
#define		REGHOST_VIRTUAL_TFTMEM						(REGHOST_ChipsetResources.TFTInterfaceMem.ulVirtual)
#define		REGHOST_VIRTUAL_DATAPORT					(REGHOST_ChipsetResources.DataPort.ulVirtual)
#define		REGHOST_VIRTUAL_VIP							(REGHOST_ChipsetResources.VideoInputPort.ulVirtual)
#define		REGHOST_VIRTUAL_VPIPE						(REGHOST_ChipsetResources.VideoPipeline.ulVirtual)
#define		REGHOST_VIRTUAL_TVOUTPUT					(REGHOST_ChipsetResources.TVOutput.ulVirtual)
#define		REGHOST_VIRTUAL_ONSCREEN					(REGHOST_ChipsetResources.OnScreenFrameBuffer.ulVirtual)
#define		REGHOST_VIRTUAL_OFFSCREEN					(REGHOST_ChipsetResources.OffScreenFrameBuffer.ulVirtual)

/* ------------------------------------------------------------
// VARIABLE		: REGHOST_ChipsetResources 
// ------------------------------------------------------------
// DESCRIPTION	: List of all the I/O and memory mapped registers
//				  with the correspondant physical and logical
//				  address
// ------------------------------------------------------------*/
extern REGHOST_CHIPSET_RESOURCES	REGHOST_ChipsetResources;

/* ------------------------------------------------------------
// DEFINE		: REGHOST_GET_...
//				  REGHOST_SET_...
// ------------------------------------------------------------
// DESCRIPTION	: Read or write a register
// ------------------------------------------------------------*/
#define		REGHOST_GET_GRAPHICCONTROLLER_REG(reg)      ( REGHOST_IndexedIORead8(REGHOST_VIRTUAL_GRAPHICCONTROLLER, (unsigned char)(reg)) )
#define		REGHOST_GET_INPUTSTATUS_REG()				( OSAL_IORead8(REGHOST_VIRTUAL_INPUTSTATUS, 0) )
#define		REGHOST_GET_CRTC_REG(reg)					( REGHOST_IndexedIORead8(REGHOST_VIRTUAL_CRTC, (unsigned char)(reg)) )
#define		REGHOST_GET_SEQUENCER_REG(reg)				( REGHOST_IndexedIORead8(REGHOST_VIRTUAL_SEQUENCER, (unsigned char)(reg)) )
#define		REGHOST_GET_CHIPSET_REG(reg)				( REGHOST_IndexedIORead8(REGHOST_VIRTUAL_CHIPSET, (unsigned char)(reg)) )
#define		REGHOST_GET_16BITSCHIPSET_REG(reg)			( REGHOST_IndexedIORead2x8(REGHOST_VIRTUAL_CHIPSET, (unsigned char)(reg)) )
#define		REGHOST_GET_SMCSUPERIO_REG(reg)				( REGHOST_IndexedIORead8(REGHOST_VIRTUAL_SMCSUPERIO, (unsigned char)(reg)) )
#define		REGHOST_GET_PARALLELPORT_REG()				( OSAL_IORead8(REGHOST_VIRTUAL_PARALLELPORT, 0) )
#define		REGHOST_GET_MISCELLANEOUS_REG()				( OSAL_IORead8(REGHOST_VIRTUAL_MISCELLANEOUSREAD, 0) )
#define		REGHOST_GET_PALETTEMASK_REG()				( OSAL_IORead8(REGHOST_VIRTUAL_PALETTEMASK, 0) )
#define		REGHOST_GET_RAMDACDATA_REG()				( OSAL_IORead8(REGHOST_VIRTUAL_RAMDACDATA, 0) )
#define		REGHOST_GET_ATTRIBUT_REG(reg)				( REGHOST_IndexedIORead8(REGHOST_VIRTUAL_ATTRIBUT, (unsigned char)(reg)) )
#define		REGHOST_GET_TVOUTPUT_REG(reg)				( OSAL_MemoryRead8(REGHOST_VIRTUAL_TVOUTPUT, reg) )
#define     REGHOST_GET_TFTMEM_REG(reg)                             ( OSAL_MemoryRead16(REGHOST_VIRTUAL_TFTMEM, reg) )

#define  	REGHOST_SET_MOTHERBOARD_REG(val)			{ OSAL_IOWrite8(REGHOST_VIRTUAL_MOTHERBOARD, 0, (unsigned char)(val)); }
#define		REGHOST_SET_VIDEOSUBSYSTEM1_REG(val)		{ OSAL_IOWrite8(REGHOST_VIRTUAL_VIDEOSUBSYSTEM1, 0, (unsigned char)(val)); }
#define		REGHOST_SET_VIDEOSUBSYSTEM2_REG(val)		{ OSAL_IOWrite8(REGHOST_VIRTUAL_VIDEOSUBSYSTEM2, 0, (unsigned char)(val)); }
#define		REGHOST_SET_MISCELLANEOUS_REG(val)			{ OSAL_IOWrite8(REGHOST_VIRTUAL_MISCELLANEOUS, 0, (unsigned char)(val)); }
#define		REGHOST_SET_ATTRIBUT_REG(val)				{ OSAL_IOWrite8(REGHOST_VIRTUAL_ATTRIBUT, 0, (unsigned char)(val)); }
#define		REGHOST_SET_PALETTEMASK_REG(val)			{ OSAL_IOWrite8(REGHOST_VIRTUAL_PALETTEMASK, 0, (unsigned char)(val)); }
#define		REGHOST_SET_RAMDACINDEX_REG(val)			{ OSAL_IOWrite8(REGHOST_VIRTUAL_RAMDACINDEX, 0, (unsigned char)(val)); }
#define		REGHOST_SET_RAMDACREADINDEX_REG(val)		{ OSAL_IOWrite8(REGHOST_VIRTUAL_RAMDACREADINDEX, 0, (unsigned char)(val)); }
#define		REGHOST_SET_RAMDACDATA_REG(val)				{ OSAL_IOWrite8(REGHOST_VIRTUAL_RAMDACDATA, 0, (unsigned char)(val)); }
#define		REGHOST_SET_SEQUENCER_REG(reg, val)			{ REGHOST_IndexedIOWrite8(REGHOST_VIRTUAL_SEQUENCER, (unsigned char)(reg), (unsigned char)(val)); }
#define		REGHOST_SET_CHIPSET_REG(reg, val)	       	{ REGHOST_IndexedIOWrite8(REGHOST_VIRTUAL_CHIPSET, (unsigned char)(reg), (unsigned char)(val)); }
#define		REGHOST_SET_16BITSCHIPSET_REG(reg, val)	       	{ REGHOST_IndexedIOWrite2x8(REGHOST_VIRTUAL_CHIPSET, (unsigned char)(reg), (unsigned char)(val)); }
#define		REGHOST_SET_GRAPHICCONTROLLER_REG(reg, val)	{ REGHOST_IndexedIOWrite8(REGHOST_VIRTUAL_GRAPHICCONTROLLER, (unsigned char)(reg), (unsigned char)(val)); }
#define		REGHOST_SET_CRTC_REG(reg, val)				{ REGHOST_IndexedIOWrite8(REGHOST_VIRTUAL_CRTC, (unsigned char)(reg), (unsigned char)(val)); }
#define		REGHOST_SET_PCICONFIGADDRESS_REG(val)		{ OSAL_IOWrite32(REGHOST_VIRTUAL_PCICONFIGADDRESS, 0, val); }
#define		REGHOST_SET_PARALLELPORT_REG(val)			{ OSAL_IOWrite8(REGHOST_VIRTUAL_PARALLELPORT, 0, (unsigned char)(val)); }
#define		REGHOST_SET_TFTINTERFACE_REG(reg, val)		{ REGHOST_TFTInterfaceWrite16(REGHOST_VIRTUAL_TFTINTERFACE, (unsigned short)(reg), (unsigned short)(val)); }
#define		REGHOST_SET_TVOUTPUT_REG(reg, val)			{ OSAL_MemoryWrite8(REGHOST_VIRTUAL_TVOUTPUT, reg, (unsigned char)(val)); }
#define     REGHOST_SET_TFTMEM_REG(reg, val)                        { OSAL_MemoryWrite16(REGHOST_VIRTUAL_TFTMEM, reg, (unsigned short)(val)); }

#define		REGHOST_UNLOCK_SMCSUPERIO_REG()				{ OSAL_IOWrite8(REGHOST_VIRTUAL_SMCSUPERIO, 0, 0x55); }
#define		REGHOST_LOCK_SMCSUPERIO_REG()				{ OSAL_IOWrite8(REGHOST_VIRTUAL_SMCSUPERIO, 0, 0xaa); }
#define		REGHOST_SET_SMCGPIO_REG(reg, val)			{ REGHOST_IndexedIOWrite8(REGHOST_VIRTUAL_SMCGPIO, (unsigned char)(reg), (unsigned char)(val)); }
#define		REGHOST_GET_SMCGPIO_REG(reg)				(REGHOST_IndexedIORead8(REGHOST_VIRTUAL_SMCGPIO, (unsigned char)(reg)))
#define		REGHOST_SET_SMCSUPERIO_REG(reg, val)		{ REGHOST_IndexedIOWrite8(REGHOST_VIRTUAL_SMCSUPERIO, (unsigned char)(reg), (unsigned char)(val)); }

/* ------------------------------------------------------------
// DEFINE		: REGHOST_GET_GE_... 
//				  REGHOST_SET_GE_... 
// ------------------------------------------------------------
// DESCRIPTION	: Read or write a Graphic Engine register
// ------------------------------------------------------------*/
#define		REGHOST_GET_GE_STATUS_REG					(OSAL_MemoryRead32(REGHOST_VIRTUAL_GE, 0x908))
#define		REGHOST_GET_GE_EXTRA_REG					(OSAL_MemoryRead32(REGHOST_VIRTUAL_GE, 0x0d4))
#define		REGHOST_SET_GE_CURSORXY_REG(Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_GE, 0x11c, Value)
#define		REGHOST_SET_GE_PATTERN_REG(Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_GE, 0x058, Value)
#define		REGHOST_SET_GE_SRCBASE_REG(Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_GE, 0x098, Value)
#define		REGHOST_SET_GE_SRCPITCH_REG(Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_GE, 0x0ac, Value)
#define		REGHOST_SET_GE_DSTBASE_REG(Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_GE, 0x018, Value)
#define		REGHOST_SET_GE_DSTPITCH_REG(Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_GE, 0x028, Value)
#define		REGHOST_SET_GE_BPP_REG(Value)				OSAL_MemoryWrite32(REGHOST_VIRTUAL_GE, 0x07c, Value)
#define		REGHOST_SET_GE_ROP_REG(Value)				OSAL_MemoryWrite32(REGHOST_VIRTUAL_GE, 0x08c, Value)
#define		REGHOST_SET_GE_SRCKEY_REG(Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_GE, 0x0ec, Value)
#define		REGHOST_SET_GE_DSTKEY_REG(Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_GE, 0x0fc, Value)
#define		REGHOST_SET_GE_BKCOLOR_REG(Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_GE, 0x004, Value)
#define		REGHOST_SET_GE_FGCOLOR_REG(Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_GE, 0x034, Value)
#define		REGHOST_SET_GE_SRCXY_REG(Value)				OSAL_MemoryWrite32(REGHOST_VIRTUAL_GE, 0x0bc, Value)
#define		REGHOST_SET_GE_WIDTH_REG(Value)				OSAL_MemoryWrite32(REGHOST_VIRTUAL_GE, 0x0c8, Value)
#define		REGHOST_SET_GE_HEIGHT_REG(Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_GE, 0x048, Value)
#define		REGHOST_SET_GE_EXTRA_REG(Value)				OSAL_MemoryWrite32(REGHOST_VIRTUAL_GE, 0x0d4, Value)
#define		REGHOST_SET_GE_DATAPORT_REG(Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_GE, 0x804, Value)
#define		REGHOST_SET_GE_DSTXY_REG(Value)				OSAL_MemoryWrite32(REGHOST_VIRTUAL_GE, 0x10000, Value)
#define		REGHOST_SET_GE_DUMMYDSTXY_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_GE, 0x1c000, Value)
#define		REGHOST_SET_GE_1WIDTHDSTXY_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_GE, 0x14000, Value)
#define		REGHOST_SET_GE_2WIDTHDSTXY_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_GE, 0x14004, Value)
#define		REGHOST_SET_GE_3WIDTHDSTXY_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_GE, 0x14008, Value)
#define		REGHOST_SET_GE_4WIDTHDSTXY_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_GE, 0x1400c, Value)
#define		REGHOST_SET_GE_1HEIGHTDSTXY_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_GE, 0x18000, Value)

/* ------------------------------------------------------------
// DEFINE		: REGHOST_GET_AFE_...
//				  REGHOST_SET_AFE_...
// ------------------------------------------------------------
// DESCRIPTION	: Read or write a Alpha Font Engine register
// ------------------------------------------------------------*/

#define		REGHOST_GET_AFE_STATUS_REG				(OSAL_MemoryRead32(REGHOST_VIRTUAL_AFE, 0x1c))
#define		REGHOST_GET_AFE_MISC_REG				(OSAL_MemoryRead32(REGHOST_VIRTUAL_AFE, 0x40))
#define		REGHOST_GET_AFE_PERF_REG				(OSAL_MemoryRead32(REGHOST_VIRTUAL_AFE, 0x48))
#define		REGHOST_GET_AFE_PREV_PERF_REG			(OSAL_MemoryRead32(REGHOST_VIRTUAL_AFE, 0x4c))

#define		REGHOST_SET_AFE_BKCOLOR_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_AFE, 0x00, Value)
#define		REGHOST_SET_AFE_DATA_REG(Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_AFE, 0x04, Value)
#define		REGHOST_SET_AFE_DSTBASE_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_AFE, 0x08, Value)
#define		REGHOST_SET_AFE_DSTPITCH_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_AFE, 0x0c, Value)
#define		REGHOST_SET_AFE_DSTXY_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_AFE, 0x10, Value)
#define		REGHOST_SET_AFE_FGCOLOR_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_AFE, 0x14, Value)
#define		REGHOST_SET_AFE_FONT_REG(Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_AFE, 0x18, Value)
#define		REGHOST_SET_AFE_SRCXY_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_AFE, 0x20, Value)
#define		REGHOST_SET_AFE_SRCBASE_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_AFE, 0x24, Value)
#define		REGHOST_SET_AFE_SRCPITCH_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_AFE, 0x28, Value)
#define		REGHOST_SET_AFE_ROP_REG(Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_AFE, 0x2c, Value)
#define		REGHOST_SET_AFE_SRCLWM_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_AFE, 0x30, Value)
#define		REGHOST_SET_AFE_DSTLWM_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_AFE, 0x34, Value)
#define		REGHOST_SET_AFE_DDHLWM_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_AFE, 0x38, Value)
#define		REGHOST_SET_AFE_ADDOFFSET_REG(Value)	OSAL_MemoryWrite32(REGHOST_VIRTUAL_AFE, 0x3c, Value)
#define		REGHOST_SET_AFE_MISC_REG(Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_AFE, 0x40, Value)
#define		REGHOST_SET_AFE_SOFTRESET_REG(Value)	OSAL_MemoryWrite32(REGHOST_VIRTUAL_AFE, 0x44, Value)

#define		REGHOST_SET_AFE_CMDFNT_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_AFE, 0x50, Value)
#define		REGHOST_SET_AFE_CMDDSTXY_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_AFE, 0x54, Value)
#define		REGHOST_SET_AFE_CMDDSTBASE_REG(Value)	OSAL_MemoryWrite32(REGHOST_VIRTUAL_AFE, 0x58, Value)
#define		REGHOST_SET_AFE_CMDDEBUG_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_AFE, 0x5c, Value)

/* ------------------------------------------------------------
// DEFINE		: REGHOST_GET_LDE_...
//				  REGHOST_SET_LDE_...
// ------------------------------------------------------------
// DESCRIPTION	: Read or write a Line Drawing Engine register
// ------------------------------------------------------------*/

#define		REGHOST_GET_LDE_STATUS_REG					OSAL_MemoryRead32(REGHOST_VIRTUAL_LDE, 0x54)
#define		REGHOST_GET_LDE_LUT_REG						OSAL_MemoryRead32(REGHOST_VIRTUAL_LDE, 0x2c)
#define		REGHOST_GET_LDE_MISC_REG					OSAL_MemoryRead32(REGHOST_VIRTUAL_LDE, 0x3c)
#define		REGHOST_GET_LDE_LINEWIDTH_REG				OSAL_MemoryRead32(REGHOST_VIRTUAL_LDE, 0x30)
#define		REGHOST_GET_LDE_LINECOLOR_REG               OSAL_MemoryRead32(REGHOST_VIRTUAL_LDE, 0x34)
#define		REGHOST_GET_LDE_BACKCOLOR_REG               OSAL_MemoryRead32(REGHOST_VIRTUAL_LDE, 0x38)
#define		REGHOST_GET_LDE_FBBASE_REG                  OSAL_MemoryRead32(REGHOST_VIRTUAL_LDE, 0x40)
#define		REGHOST_GET_LDE_FBPITCHM_REG                OSAL_MemoryRead32(REGHOST_VIRTUAL_LDE, 0x44)
#define		REGHOST_GET_LDE_FBPITCH_REG                 OSAL_MemoryRead32(REGHOST_VIRTUAL_LDE, 0x48)
#define		REGHOST_GET_LDE_MAX_CLIPCOORD_REG           OSAL_MemoryRead32(REGHOST_VIRTUAL_LDE, 0x4c)
#define		REGHOST_GET_LDE_MIN_CLIPCOORD_REG           OSAL_MemoryRead32(REGHOST_VIRTUAL_LDE, 0x64)
#define		REGHOST_GET_LDE_FLUSH_COUNT_REG             OSAL_MemoryRead32(REGHOST_VIRTUAL_LDE, 0x60)
#define		REGHOST_GET_LDE_CORRECT_REG		OSAL_MemoryRead32(REGHOST_VIRTUAL_LDE, 0x28)


#define		REGHOST_SET_LDE_XSTART_REG(Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_LDE, 0x00, Value)
#define		REGHOST_SET_LDE_XEND_REG(Value)				OSAL_MemoryWrite32(REGHOST_VIRTUAL_LDE, 0x08, Value)
#define		REGHOST_SET_LDE_YSTART_REG(Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_LDE, 0x04, Value)
#define		REGHOST_SET_LDE_YEND_REG(Value)				OSAL_MemoryWrite32(REGHOST_VIRTUAL_LDE, 0x0c, Value)

#define		REGHOST_SET_LDE_DX_REG(Value)				OSAL_MemoryWrite32(REGHOST_VIRTUAL_LDE, 0x10, Value)
#define		REGHOST_SET_LDE_DY_REG(Value)				OSAL_MemoryWrite32(REGHOST_VIRTUAL_LDE, 0x14, Value)
#define		REGHOST_SET_LDE_SCALEDDX_REG(Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_LDE, 0x18, Value)
#define		REGHOST_SET_LDE_SCALEDDY_REG(Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_LDE, 0x1c, Value)
#define		REGHOST_SET_LDE_FXY_REG(Value)				OSAL_MemoryWrite32(REGHOST_VIRTUAL_LDE, 0x20, Value)
#define		REGHOST_SET_LDE_SCALEDFXY_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_LDE, 0x24, Value)
#define		REGHOST_SET_LDE_CORRECT_REG(Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_LDE, 0x28, Value)
#define		REGHOST_SET_LDE_LUT_REG(Value)				OSAL_MemoryWrite32(REGHOST_VIRTUAL_LDE, 0x2c, Value)
#define		REGHOST_SET_LDE_LINEWIDTH_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_LDE, 0x30, Value)
#define		REGHOST_SET_LDE_LINECOLOR_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_LDE, 0x34, Value)
#define		REGHOST_SET_LDE_BACKCOLOR_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_LDE, 0x38, Value)
#define		REGHOST_SET_LDE_MISC_REG(Value)				OSAL_MemoryWrite32(REGHOST_VIRTUAL_LDE, 0x3c, Value)
#define		REGHOST_SET_LDE_FBBASE_REG(Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_LDE, 0x40, Value)
#define		REGHOST_SET_LDE_FBPITCHM_REG(Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_LDE, 0x44, Value)
#define		REGHOST_SET_LDE_FBPITCH_REG(Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_LDE, 0x48, Value)
#define		REGHOST_SET_LDE_MAX_CLIPCOORD_REG(Value)	OSAL_MemoryWrite32(REGHOST_VIRTUAL_LDE, 0x4c, Value)
#define		REGHOST_SET_LDE_COMMAND_REG(Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_LDE, 0x50, Value)
#define		REGHOST_SET_LDE_SOFT_RESET_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_LDE, 0x58, Value)
#define		REGHOST_SET_LDE_PERFORMANCE_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_LDE, 0x5c, Value)
#define		REGHOST_SET_LDE_FLUSH_COUNT_REG(Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_LDE, 0x60, Value)
#define		REGHOST_SET_LDE_MIN_CLIPCOORD_REG(Value)	OSAL_MemoryWrite32(REGHOST_VIRTUAL_LDE, 0x64, Value)

/* ------------------------------------------------------------
// DEFINE		: REGHOST_GET_ONSCREEN...
//				  REGHOST_SET_ONSCREEN...
//				  REGHOST_GET_OFFSCREEN... 
//				  REGHOST_SET_OFFSCREEN... 
// ------------------------------------------------------------
// DESCRIPTION	: Read or write the frame buffer
// ------------------------------------------------------------*/
#define		REGHOST_SET_ONSCREEN8(Base, Value)			OSAL_MemoryWrite8(REGHOST_VIRTUAL_ONSCREEN, Base, Value)
#define		REGHOST_SET_OFFSCREEN8(Base, Value)			OSAL_MemoryWrite8(REGHOST_VIRTUAL_OFFSCREEN, Base, Value)
#define		REGHOST_SET_ONSCREEN16(Base, Value)			OSAL_MemoryWrite16(REGHOST_VIRTUAL_ONSCREEN, Base, Value)
#define		REGHOST_SET_OFFSCREEN16(Base, Value)		OSAL_MemoryWrite16(REGHOST_VIRTUAL_OFFSCREEN, Base, Value)
#define		REGHOST_SET_ONSCREEN32(Base, Value)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_ONSCREEN, Base, Value)
#define		REGHOST_SET_OFFSCREEN32(Base, Value)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_OFFSCREEN, Base, Value)

#define		REGHOST_GET_ONSCREEN8(Base)					(OSAL_MemoryRead8(REGHOST_VIRTUAL_ONSCREEN, Base))
#define		REGHOST_GET_OFFSCREEN8(Base)				(OSAL_MemoryRead8(REGHOST_VIRTUAL_OFFSCREEN, Base))
#define		REGHOST_GET_ONSCREEN16(Base)				(OSAL_MemoryRead16(REGHOST_VIRTUAL_ONSCREEN, Base))
#define		REGHOST_GET_OFFSCREEN16(Base)				(OSAL_MemoryRead16(REGHOST_VIRTUAL_OFFSCREEN, Base))
#define		REGHOST_GET_ONSCREEN32(Base)				(OSAL_MemoryRead32(REGHOST_VIRTUAL_ONSCREEN, Base))
#define		REGHOST_GET_OFFSCREEN32(Base)				(OSAL_MemoryRead32(REGHOST_VIRTUAL_OFFSCREEN, Base))

/* ------------------------------------------------------------
// DEFINE		: REGHOST_Get...Pointer
// ------------------------------------------------------------
// DESCRIPTION	: Get the CPU logical address of specific
//				  location in a memory range
// ------------------------------------------------------------*/
#define		REGHOST_GetOffscreenPointer(base)			(OSAL_GetLinearPointer(REGHOST_VIRTUAL_OFFSCREEN, base))
#define 	REGHOST_GetOnscreenPointer(base)			(OSAL_GetLinearPointer(REGHOST_VIRTUAL_ONSCREEN, base))
#define 	REGHOST_GetGERegisterPointer(base)			(OSAL_GetLinearPointer(REGHOST_VIRTUAL_GE, base))
#define 	REGHOST_GetAFERegisterPointer(base)			(OSAL_GetLinearPointer(REGHOST_VIRTUAL_AFE, base))
#define 	REGHOST_GetDataPortPointer(base)			(OSAL_GetLinearPointer(REGHOST_VIRTUAL_DATAPORT, base))

/* ------------------------------------------------------------
// DEFINE		: REGHOST_GET_VIP_... 
//				  REGHOST_SET_VIP_... 
// ------------------------------------------------------------
// DESCRIPTION	: Read or write a Video Input Port register
// ------------------------------------------------------------*/
#define		REGHOST_GET_VIP_REG(reg)					(OSAL_MemoryRead32(REGHOST_VIRTUAL_VIP, reg))
#define		REGHOST_SET_VIP_REG(reg, val)				OSAL_MemoryWrite32(REGHOST_VIRTUAL_VIP, reg, val)

#define 	REGHOST_GET_VIP_HORIZONTALTIMINGS_REG()		(OSAL_MemoryRead32(REGHOST_VIRTUAL_VIP, 0x30))
#define 	REGHOST_GET_VIP_VERTICALTIMINGS_REG()		(OSAL_MemoryRead32(REGHOST_VIRTUAL_VIP, 0x34))
#define 	REGHOST_GET_VIP_TIMINGS1_REG()				(OSAL_MemoryRead32(REGHOST_VIRTUAL_VIP, 0x28))
#define 	REGHOST_GET_VIP_TIMINGS2_REG()				(OSAL_MemoryRead32(REGHOST_VIRTUAL_VIP, 0x2c))
#define 	REGHOST_GET_VIP_CONFIG_REG()				(OSAL_MemoryRead32(REGHOST_VIRTUAL_VIP, 0x04))
#define 	REGHOST_GET_VIP_ODDBASE_REG()				(OSAL_MemoryRead32(REGHOST_VIRTUAL_VIP, 0x0c))
#define 	REGHOST_GET_VIP_EVENBASE_REG()				(OSAL_MemoryRead32(REGHOST_VIRTUAL_VIP, 0x10))
#define 	REGHOST_GET_VIP_PITCH_REG()					(OSAL_MemoryRead32(REGHOST_VIRTUAL_VIP, 0x14))
#define 	REGHOST_GET_VIP_STATUS_REG()				(OSAL_MemoryRead32(REGHOST_VIRTUAL_VIP, 0x08))

#define 	REGHOST_SET_VIP_HORIZONTALTIMINGS_REG(val)	OSAL_MemoryWrite32(REGHOST_VIRTUAL_VIP, 0x30, val)
#define 	REGHOST_SET_VIP_VERTICALTIMINGS_REG(val)	OSAL_MemoryWrite32(REGHOST_VIRTUAL_VIP, 0x34, val)
#define 	REGHOST_SET_VIP_TIMINGS1_REG(val)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_VIP, 0x28, val)
#define 	REGHOST_SET_VIP_TIMINGS2_REG(val)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_VIP, 0x2c, val)
#define 	REGHOST_SET_VIP_CONFIG_REG(val)				OSAL_MemoryWrite32(REGHOST_VIRTUAL_VIP, 0x04, val)
#define 	REGHOST_SET_VIP_ODDBASE_REG(val)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_VIP, 0x0c, val)
#define 	REGHOST_SET_VIP_EVENBASE_REG(val)			OSAL_MemoryWrite32(REGHOST_VIRTUAL_VIP, 0x10, val)
#define 	REGHOST_SET_VIP_PITCH_REG(val)				OSAL_MemoryWrite32(REGHOST_VIRTUAL_VIP, 0x14, val)
#define 	REGHOST_SET_VIP_STATUS_REG(val)				OSAL_MemoryWrite32(REGHOST_VIRTUAL_VIP, 0x08, val)

/* ------------------------------------------------------------
// DEFINE		: REGHOST_GET_VPIPE_... 
//				  REGHOST_SET_VPIPE_... 
// ------------------------------------------------------------
// DESCRIPTION	: Read or write a Video Pipeline register
// ------------------------------------------------------------*/
#define		REGHOST_GET_VPIPE_REG(reg)						(OSAL_MemoryRead32(REGHOST_VIRTUAL_VPIPE, reg))
#define		REGHOST_SET_VPIPE_REG(reg, val)					OSAL_MemoryWrite32(REGHOST_VIRTUAL_VPIPE, reg, val)

#define 	REGHOST_GET_VPIPE_SRCBASE_REG()					(OSAL_MemoryRead32(REGHOST_VIRTUAL_VPIPE, 0x00))
#define 	REGHOST_GET_VPIPE_PITCH_REG() 					(OSAL_MemoryRead32(REGHOST_VIRTUAL_VPIPE, 0x04))
#define 	REGHOST_GET_VPIPE_HORIZONTALSCALE_REG()			(OSAL_MemoryRead32(REGHOST_VIRTUAL_VPIPE, 0x20))
#define 	REGHOST_GET_VPIPE_VERTICALSCALE_REG() 			(OSAL_MemoryRead32(REGHOST_VIRTUAL_VPIPE, 0x28))
#define 	REGHOST_GET_VPIPE_SRCDIM_REG() 					(OSAL_MemoryRead32(REGHOST_VIRTUAL_VPIPE, 0x08))
#define 	REGHOST_GET_VPIPE_DSTDIM_REG() 					(OSAL_MemoryRead32(REGHOST_VIRTUAL_VPIPE, 0x18))
#define 	REGHOST_SET_VPIPE_CRTCBURSTLENGHT_REG(val)		OSAL_MemoryWrite32(REGHOST_VIRTUAL_VPIPE, 0x0c, val)
#define 	REGHOST_SET_VPIPE_VIDEOBURSTLENGHT_REG(val) 	OSAL_MemoryWrite32(REGHOST_VIRTUAL_VPIPE, 0x10, val)
#define 	REGHOST_SET_VPIPE_COLORSPACECONVERTER_REG(val) 	OSAL_MemoryWrite32(REGHOST_VIRTUAL_VPIPE, 0x2c, val)
#define 	REGHOST_SET_VPIPE_PITCH_REG(val) 				OSAL_MemoryWrite32(REGHOST_VIRTUAL_VPIPE, 0x04, val)
#define 	REGHOST_SET_VPIPE_MIXMODE_REG(val) 				OSAL_MemoryWrite32(REGHOST_VIRTUAL_VPIPE, 0x30, val)
#define 	REGHOST_SET_VPIPE_COLORKEY_REG(val) 			OSAL_MemoryWrite32(REGHOST_VIRTUAL_VPIPE, 0x34, val)
#define 	REGHOST_SET_VPIPE_CHROMALOW_REG(val) 			OSAL_MemoryWrite32(REGHOST_VIRTUAL_VPIPE, 0x38, val)
#define 	REGHOST_SET_VPIPE_CHROMAHIGH_REG(val) 			OSAL_MemoryWrite32(REGHOST_VIRTUAL_VPIPE, 0x3c, val)
#define 	REGHOST_SET_VPIPE_HORIZONTALSCALE_REG(val) 		OSAL_MemoryWrite32(REGHOST_VIRTUAL_VPIPE, 0x20, val)
#define 	REGHOST_SET_VPIPE_VERTICALSCALE_REG(val) 		OSAL_MemoryWrite32(REGHOST_VIRTUAL_VPIPE, 0x28, val)
#define 	REGHOST_SET_VPIPE_SRCBASE_REG(val) 				OSAL_MemoryWrite32(REGHOST_VIRTUAL_VPIPE, 0x00, val)
#define 	REGHOST_SET_VPIPE_SRCDIM_REG(val) 				OSAL_MemoryWrite32(REGHOST_VIRTUAL_VPIPE, 0x08, val)
#define 	REGHOST_SET_VPIPE_DSTXY_REG(val) 				OSAL_MemoryWrite32(REGHOST_VIRTUAL_VPIPE, 0x14, val)
#define 	REGHOST_SET_VPIPE_DSTDIM_REG(val) 				OSAL_MemoryWrite32(REGHOST_VIRTUAL_VPIPE, 0x18, val)
#define 	REGHOST_SET_VPIPE_VIDEOENABLE_REG(val) 			OSAL_MemoryWrite32(REGHOST_VIRTUAL_VPIPE, 0x40, val)

/* ------------------------------------------------------------
// FUNCTION		: REGHOST_Init
// ------------------------------------------------------------
// DESCRIPTION	: Initialize the access to I/O and memory
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulLib		: The registers of which module need to be
//				  initialized : a OR between REGHOST_LIB_...
//				  values
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGHOST_Init(unsigned long ulLib);

/* ------------------------------------------------------------
// FUNCTION		: REGHOST_Free
// ------------------------------------------------------------
// DESCRIPTION	: Free the library
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGHOST_Free(void);

/* ------------------------------------------------------------
// FUNCTION		: REGHOST_GetGBASE
// ------------------------------------------------------------
// DESCRIPTION	: Get the value of GBASE (base physical address
//				  of graphics registers)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: GBASE
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long REGHOST_GetGBASE(void);

/* ------------------------------------------------------------
// FUNCTION		: REGHOST_IndexedIORead8
// ------------------------------------------------------------
// DESCRIPTION	: Read a 8 bit indexed I/O register
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulVirtual	: CPU logical address of the I/O to access
// ucIndex		: Index of the register to access
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned char REGHOST_IndexedIORead8(unsigned long ulVirtual, unsigned char ucIndex);

/* ------------------------------------------------------------
// FUNCTION		: REGHOST_IndexedIOWrite8
// ------------------------------------------------------------
// DESCRIPTION	: Write a 8 bit indexed I/O register
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulVirtual	: CPU logical address of the I/O to access
// ucIndex		: Index of the register to access
// ucData		: Value to write in the register
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGHOST_IndexedIOWrite8(unsigned long ulVirtual, unsigned char ucIndex, unsigned char ucData);

/* ------------------------------------------------------------
// FUNCTION		: REGHOST_IndexedIORead2x8
// ------------------------------------------------------------
// DESCRIPTION	: Read a 16 bits value by 8 bits indexed I/O 
//				  (low bits first followed by high bits)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulVirtual	: CPU logical address of the I/O to access
// ucIndex		: Index of the register to access
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: 16 bits value
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned short REGHOST_IndexedIORead2x8(unsigned long ulVirtual, unsigned char ucIndex);

/* ------------------------------------------------------------
/* ------------------------------------------------------------
// FUNCTION		: REGHOST_IndexedIOWrite2x8
// ------------------------------------------------------------
// DESCRIPTION	: Write a 16 bits value by 8 bits indexed I/O
//				  (low bits followed by high bits)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulVirtual	: CPU logical address of the I/O to access
// ucIndex		: Index of the register to access
// usData		: Value to wtite in the register
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGHOST_IndexedIOWrite2x8(unsigned long ulVirtual, unsigned char ucIndex, unsigned short usData);

/* ------------------------------------------------------------
// FUNCTION		: REGHOST_TFTInterfaceWrite16
// ------------------------------------------------------------
// DESCRIPTION	: Write a value in a TFT interfrace register
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulVirtual	: CPU logical address of the TFT interface I/O
// usIndex		: Index of the register to access
// usData		: Value to wtite in the register
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGHOST_TFTInterfaceWrite16(unsigned long ulVirtual, unsigned short usIndex, unsigned short usData);

#endif
