/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: i2c.c
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libreg\i2c
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: July, 18th 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Functions to manage i2c bus
// ------------------------------------------------------------
// VARIABLES	: -
// ------------------------------------------------------------
// MACROS		: -
// ------------------------------------------------------------
// FUNCTIONS	: REGI2C_Wait
//				  REGI2C_GPIOFunction
//				  REGI2C_SMCGPIOFunction
//				  REGI2C_PPFunction
//				  REGI2C_DDCFunction
//				  REGI2C_Start
//				  REGI2C_ReStart
//				  REGI2C_Stop
//				  REGI2C_Write
//				  REGI2C_WriteRegister
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: July, 18th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Initialize SMC GPIOs as none inverted output
//				  before use the i2c on SMC GPIOs
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

#include "trace.h"
#include "host.h"
#include "i2c.h"

/* ------------------------------------------------------------
// FUNCTION		: REGI2C_Wait
// ------------------------------------------------------------
// DESCRIPTION	: 2c wait state
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
volatile long i;
void REGI2C_Wait(void)
{
	for(i = 0; i < 100; i++);
}

/* ------------------------------------------------------------
// FUNCTION		: REGI2C_GPIOFunction
// ------------------------------------------------------------
// DESCRIPTION	: Function to manage an i2c on STPC GPIO
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulFlag		: Operation to process
//				  REGI2C_FUNCTION_INIT		: Enable i2c lines 
//				  REGI2C_FUNCTION_SETDATA	: Set data line to 1
//				  REGI2C_FUNCTION_RESETDATA	: Set data line to 0
//				  REGI2C_FUNCTION_SETCLOCK	: Set clock line to 1
//				  REGI2C_FUNCTION_RESETCLOCK	: Set clock line to 0
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGI2C_GPIOFunction(unsigned long ulFlag)
{
	switch(ulFlag)
	{
		case REGI2C_FUNCTION_INIT:
			/* Init lines */
			break;

		case REGI2C_FUNCTION_SETCLOCK:
			/* Set clock */
			REGHOST_SET_CHIPSET_REG(0x7b, (unsigned char)(REGHOST_GET_CHIPSET_REG(0x7b) | 0x8));
			break;

		case REGI2C_FUNCTION_RESETCLOCK:
			/* Clear clock */
			REGHOST_SET_CHIPSET_REG(0x7b, (unsigned char)(REGHOST_GET_CHIPSET_REG(0x7b) & ~0x8));
			break;

		case REGI2C_FUNCTION_SETDATA:
			/* Set data */
			REGHOST_SET_CHIPSET_REG(0x7b, (unsigned char)(REGHOST_GET_CHIPSET_REG(0x7b) | 0x4));
			break;

		case REGI2C_FUNCTION_RESETDATA:
			/* Clear data */
			REGHOST_SET_CHIPSET_REG(0x7b, (unsigned char)(REGHOST_GET_CHIPSET_REG(0x7b) & ~0x4));
			break;
	}
}

/* ------------------------------------------------------------
// FUNCTION		: REGI2C_SMCGPIOFunction
// ------------------------------------------------------------
// DESCRIPTION	: Function to manage an i2c on SMC SuperI/O GPIO
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulFlag		: Operation to process
//				  REGI2C_FUNCTION_INIT		: Enable i2c lines 
//				  REGI2C_FUNCTION_SETDATA	: Set data line to 1
//				  REGI2C_FUNCTION_RESETDATA	: Set data line to 0
//				  REGI2C_FUNCTION_SETCLOCK	: Set clock line to 1
//				  REGI2C_FUNCTION_RESETCLOCK	: Set clock line to 0
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGI2C_SMCGPIOFunction(unsigned long ulFlag)
{
	switch(ulFlag)
	{
		case REGI2C_FUNCTION_INIT:
			/* Unlock access to SMC registers */
			REGHOST_UNLOCK_SMCSUPERIO_REG();

			/* Select module number 8 (the GPIOs) */
			REGHOST_SET_SMCSUPERIO_REG(0x07, 0x08);

			/* Set the i2c lines as none invert output */
			REGHOST_SET_SMCSUPERIO_REG(0xe6, 0);
			REGHOST_SET_SMCSUPERIO_REG(0xe7, 0);
			REGHOST_SET_SMCSUPERIO_REG(0xed, 0);

			/* Lock access to SMC registers */
			REGHOST_LOCK_SMCSUPERIO_REG();
			/* Set clock */

			/* Select the SAA as input for Video Input Port */
			REGHOST_SET_SMCGPIO_REG(0x2, (unsigned char)((REGHOST_GET_SMCGPIO_REG(0x2) | 0x10) & 0xdf));
			break;

		case REGI2C_FUNCTION_SETCLOCK:
			/* Set clock */
			REGHOST_SET_SMCGPIO_REG(0x1, (unsigned char)(REGHOST_GET_SMCGPIO_REG(0x1) | 0x80));
			break;

		case REGI2C_FUNCTION_RESETCLOCK:
			/* Clear clock */
			REGHOST_SET_SMCGPIO_REG(0x1, (unsigned char)(REGHOST_GET_SMCGPIO_REG(0x1) & ~0x80));
			break;

		case REGI2C_FUNCTION_SETDATA:
			/* Set data */
			REGHOST_SET_SMCGPIO_REG(0x1, (unsigned char)(REGHOST_GET_SMCGPIO_REG(0x1) | 0x40));
			break;

		case REGI2C_FUNCTION_RESETDATA:
			/* Clear data */
			REGHOST_SET_SMCGPIO_REG(0x1, (unsigned char)(REGHOST_GET_SMCGPIO_REG(0x1) & ~0x40));
			break;
	}
}

/* ------------------------------------------------------------
// FUNCTION		: REGI2C_PPFunction
// ------------------------------------------------------------
// DESCRIPTION	: Function to manage an i2c on Parrallel port
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulFlag		: Operation to process
//				  REGI2C_FUNCTION_INIT		: Enable i2c lines 
//				  REGI2C_FUNCTION_SETDATA	: Set data line to 1
//				  REGI2C_FUNCTION_RESETDATA	: Set data line to 0
//				  REGI2C_FUNCTION_SETCLOCK	: Set clock line to 1
//				  REGI2C_FUNCTION_RESETCLOCK	: Set clock line to 0
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGI2C_PPFunction(unsigned long ulFlag)
{
	switch(ulFlag)
	{
		case REGI2C_FUNCTION_INIT:
			/* Init lines */
			break;

		case REGI2C_FUNCTION_SETCLOCK:
			/* Set clock */
			REGHOST_SET_PARALLELPORT_REG(REGHOST_GET_PARALLELPORT_REG() | 0x2);
			break;

		case REGI2C_FUNCTION_RESETCLOCK:
			/* Clear clock */
			REGHOST_SET_PARALLELPORT_REG(REGHOST_GET_PARALLELPORT_REG() & ~0x2);
			break;

		case REGI2C_FUNCTION_SETDATA:
			/* Set data */
			REGHOST_SET_PARALLELPORT_REG(REGHOST_GET_PARALLELPORT_REG() | 0x1);
			break;

		case REGI2C_FUNCTION_RESETDATA:
			/* Clear data */
			REGHOST_SET_PARALLELPORT_REG(REGHOST_GET_PARALLELPORT_REG() & ~0x1);
			break;
	}
}

/* ------------------------------------------------------------
// FUNCTION		: REGI2C_DDCFunction
// ------------------------------------------------------------
// DESCRIPTION	: Function to manage an i2c on VGA connector
//				  DDC lines
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulFlag		: Operation to process
//				  REGI2C_FUNCTION_INIT		: Enable i2c lines 
//				  REGI2C_FUNCTION_SETDATA	: Set data line to 1
//				  REGI2C_FUNCTION_RESETDATA	: Set data line to 0
//				  REGI2C_FUNCTION_SETCLOCK	: Set clock line to 1
//				  REGI2C_FUNCTION_RESETCLOCK	: Set clock line to 0
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGI2C_DDCFunction(unsigned long ulFlag)
{
	switch(ulFlag)
	{
		case REGI2C_FUNCTION_INIT:
			/* Init lines */
			break;

		case REGI2C_FUNCTION_SETCLOCK:
			/* Set clock */
			if (REGHOST_GET_CRTC_REG(0x3f) & 0xa0)
			{
				/* Actual data is 1, set data to 1 and clock to 1 */
				REGHOST_SET_CRTC_REG(0x3f, (REGHOST_GET_CRTC_REG(0x3f) & 0x0f) | 0xc0);
			}
			else
			{
				/* Actual data is 0, set data to 0 and clock to 1 */
				REGHOST_SET_CRTC_REG(0x3f, (REGHOST_GET_CRTC_REG(0x3f) & 0x0f) | 0x40);
			}
			break;

		case REGI2C_FUNCTION_RESETCLOCK:
			/* Clear clock */
			if (REGHOST_GET_CRTC_REG(0x3f) & 0xa0)
			{
				/* Actual data is 1, set data to 1 and clock to 0 */
				REGHOST_SET_CRTC_REG(0x3f, (REGHOST_GET_CRTC_REG(0x3f) & 0x0f) | 0x80);
			}
			else
			{
				/* Actual data is 0, set data to 0 and clock to 0 */
				REGHOST_SET_CRTC_REG(0x3f, (REGHOST_GET_CRTC_REG(0x3f) & 0x0f) | 0x00);
			}
			break;

		case REGI2C_FUNCTION_SETDATA:
			/* Set data */
			if (REGHOST_GET_CRTC_REG(0x3f) & 0x10)
			{
				/* Actual clock is 1, set data to 1 and clock to 1 */
				REGHOST_SET_CRTC_REG(0x3f, (REGHOST_GET_CRTC_REG(0x3f) & 0x0f) | 0xc0);
			}
			else
			{
				/* Actual clock is 0, set data to 1 and clock to 0 */
				REGHOST_SET_CRTC_REG(0x3f, (REGHOST_GET_CRTC_REG(0x3f) & 0x0f) | 0x80);
			}
			break;

		case REGI2C_FUNCTION_RESETDATA:
			/* Clear data */
			if (REGHOST_GET_CRTC_REG(0x3f) & 0x10)
			{
				/* Actual clock is 1, set data to 0 and clock to 1 */
				REGHOST_SET_CRTC_REG(0x3f, (REGHOST_GET_CRTC_REG(0x3f) & 0x0f) | 0x40);
			}
			else
			{
				/* Actual clock is 0, set data to 0 and clock to 0 */
				REGHOST_SET_CRTC_REG(0x3f, (REGHOST_GET_CRTC_REG(0x3f) & 0x0f) | 0x00);
			}
			break;
	}
}

/* ------------------------------------------------------------
// FUNCTION		: REGI2C_Start
// ------------------------------------------------------------
// DESCRIPTION	: Start i2c protocol : an high to low transition 
//				  on data line when clock line is high
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// pfctFunction	: Function to manage the i2c data and clock lines
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGI2C_Start (REGI2C_FUNCTION pfctFunction)
{
	REGI2C_Wait();
	REGI2C_Wait();
	(*pfctFunction)(REGI2C_FUNCTION_RESETDATA);
	REGI2C_Wait();
	(*pfctFunction)(REGI2C_FUNCTION_RESETCLOCK);
}

/* ------------------------------------------------------------
// FUNCTION		: REGI2C_ReStart
// ------------------------------------------------------------
// DESCRIPTION	: Restart condition after send adresse data for 
//				  new data in the same protocol
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// pfctFunction	: Function to manage the i2c data and clock lines
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGI2C_ReStart(REGI2C_FUNCTION pfctFunction)
{
	REGI2C_Wait();
	(*pfctFunction)(REGI2C_FUNCTION_SETDATA);
	REGI2C_Wait();
	(*pfctFunction)(REGI2C_FUNCTION_SETCLOCK);
	REGI2C_Wait();
	REGI2C_Wait();
	REGI2C_Wait();
	(*pfctFunction)(REGI2C_FUNCTION_RESETDATA);
	REGI2C_Wait();
	(*pfctFunction)(REGI2C_FUNCTION_RESETCLOCK);
}

/* ------------------------------------------------------------
// FUNCTION		: REGI2C_Stop
// ------------------------------------------------------------
// DESCRIPTION	: Stop the i2c protocol
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// pfctFunction	: Function to manage the i2c data and clock lines
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGI2C_Stop(REGI2C_FUNCTION pfctFunction)
{
	REGI2C_Wait();
	(*pfctFunction)(REGI2C_FUNCTION_RESETDATA);
	REGI2C_Wait();
	(*pfctFunction)(REGI2C_FUNCTION_SETCLOCK);
	REGI2C_Wait();
	REGI2C_Wait();
	(*pfctFunction)(REGI2C_FUNCTION_SETDATA);
}

/* ------------------------------------------------------------
// FUNCTION		: REGI2C_Write
// ------------------------------------------------------------
// DESCRIPTION	: Send a data to i2c bus
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// pfctFunction	: Function to manage the i2c data and clock lines
// ucData		: Data to send
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGI2C_Write(REGI2C_FUNCTION pfctFunction, unsigned char ucData)
{
	int i;

	/* For each bit of the data */
	for (i = 0; i < 8; i ++)
	{
		/* Set the data line */
		REGI2C_Wait();
		if(ucData & 0x80)
		{
			(*pfctFunction)(REGI2C_FUNCTION_SETDATA);
		}
		else
		{
			(*pfctFunction)(REGI2C_FUNCTION_RESETDATA);
		}

		/* Toggle the clock */
		REGI2C_Wait();
		(*pfctFunction)(REGI2C_FUNCTION_SETCLOCK);
		REGI2C_Wait();
		(*pfctFunction)(REGI2C_FUNCTION_RESETCLOCK);

		/* Go to next bit */
		ucData <<= 1;
	}
}

/* ------------------------------------------------------------
// FUNCTION		: REGI2C_WriteRegister
// ------------------------------------------------------------
// DESCRIPTION	: Write a register to an i2c chip 
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// pfctFunction	: Function to manage the i2c data and clock lines
// ucID			: I2c ID of the chip
// ucReg		: Register to access
// ucData		: Data to write
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGI2C_WriteRegister(REGI2C_FUNCTION pfctFunction, unsigned char ucID, unsigned char ucReg, unsigned char ucData)
{
	DBGTRACE_BEGINFUNCTION("REGI2C_WriteRegister");
	DBGTRACE_NUMERICPARAM(pfctFunction);
	DBGTRACE_NUMERICPARAM(ucID);
	DBGTRACE_NUMERICPARAM(ucReg);
	DBGTRACE_NUMERICPARAM(ucData);

	/* Init the I2C lines */
	(*pfctFunction)(REGI2C_FUNCTION_INIT);

	/* Start bus */
	REGI2C_Start(pfctFunction);

	/* Send STV ID, reg and data */
	REGI2C_Write(pfctFunction, ucID);
	REGI2C_ReStart(pfctFunction);
	REGI2C_Write(pfctFunction, ucReg);
	REGI2C_ReStart(pfctFunction);
	REGI2C_Write(pfctFunction, ucData);
	REGI2C_ReStart(pfctFunction);

	/* Stop bus */
	REGI2C_Stop(pfctFunction);

	DBGTRACE_ENDFUNCTION("REGI2C_WriteRegister");
}
