/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: i2c.h
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libreg\i2c
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: July, 18th 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Functions to manage i2c bus
// ------------------------------------------------------------
// VARIABLES	: -
// ------------------------------------------------------------
// MACROS		: REGI2C_FUNCTION 
//				  REGI2C_LINES_...
//				  REGI2C_FUNCTION_... 
// ------------------------------------------------------------
// FUNCTIONS	: REGI2C_GPIOFunction
//				  REGI2C_SMCGPIOFunction
//				  REGI2C_PPFunction
//				  REGI2C_DDCFunction
//				  REGI2C_WriteRegister
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: July, 18th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Initialize SMC GPIOs as none inverted output
//				  before use the i2c on SMC GPIOs
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

#ifndef I2C_H
#define I2C_H

/* ------------------------------------------------------------
// TYPEDEF		: REGI2C_FUNCTION 
//				  REGI2C_LINES_...
// ------------------------------------------------------------
// DESCRIPTION	: Functions to manage i2c lines 
// ------------------------------------------------------------
// REGI2C_LINES_GPIO	: Use STPC GPIO lines for i2c
// REGI2C_LINES_SMCGPIO	: Use SMC super I/O GPIO lines for i2c
// REGI2C_LINES_PP		: Use parallel port lines for i2c
// REGI2C_LINES_DDC		: Use VGA connector DDC lines for i2c
// ------------------------------------------------------------*/
typedef void (*REGI2C_FUNCTION)(unsigned long);
#define REGI2C_LINES_GPIO		REGI2C_GPIOFunction
#define REGI2C_LINES_SMCGPIO	REGI2C_SMCGPIOFunction
#define REGI2C_LINES_PP			REGI2C_PPFunction
#define REGI2C_LINES_DDC		REGI2C_DDCFunction

/* ------------------------------------------------------------
// DEFINE		: REGI2C_FUNCTION_... 
// ------------------------------------------------------------
// DESCRIPTION	: Different functions to manage i2c lines
// ------------------------------------------------------------
// REGI2C_FUNCTION_INIT			: Init i2c lines
// REGI2C_FUNCTION_SETDATA		: Set data line to 1
// REGI2C_FUNCTION_RESETDATA	: Set data line to 0
// REGI2C_FUNCTION_SETCLOCK		: Set clock line to 1
// REGI2C_FUNCTION_RESETCLOCK	: Set clock line to 0
// ------------------------------------------------------------*/
#define REGI2C_FUNCTION_INIT			1
#define REGI2C_FUNCTION_SETDATA			2
#define REGI2C_FUNCTION_RESETDATA		3
#define REGI2C_FUNCTION_SETCLOCK		4
#define REGI2C_FUNCTION_RESETCLOCK		5

/* ------------------------------------------------------------
// FUNCTION		: REGI2C_GPIOFunction
// ------------------------------------------------------------
// DESCRIPTION	: Function to manage an i2c on STPC GPIO
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulFlag		: Operation to process
//				  REGI2C_INIT		: Enable i2c lines 
//				  REGI2C_SETDATA	: Set data line to 1
//				  REGI2C_RESETDATA	: Set data line to 0
//				  REGI2C_SETCLOCK	: Set clock line to 1
//				  REGI2C_RESETCLOCK	: Set clock line to 0
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGI2C_GPIOFunction(unsigned long ulFlag);

/* ------------------------------------------------------------
// FUNCTION		: REGI2C_SMCGPIOFunction
// ------------------------------------------------------------
// DESCRIPTION	: Function to manage an i2c on SMC SuperI/O GPIO
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulFlag		: Operation to process
//				  REGI2C_INIT		: Enable i2c lines 
//				  REGI2C_SETDATA	: Set data line to 1
//				  REGI2C_RESETDATA	: Set data line to 0
//				  REGI2C_SETCLOCK	: Set clock line to 1
//				  REGI2C_RESETCLOCK	: Set clock line to 0
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGI2C_SMCGPIOFunction(unsigned long ulFlag);

/* ------------------------------------------------------------
// FUNCTION		: REGI2C_PPFunction
// ------------------------------------------------------------
// DESCRIPTION	: Function to manage an i2c on Parrallel port
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulFlag		: Operation to process
//				  REGI2C_INIT		: Enable i2c lines 
//				  REGI2C_SETDATA	: Set data line to 1
//				  REGI2C_RESETDATA	: Set data line to 0
//				  REGI2C_SETCLOCK	: Set clock line to 1
//				  REGI2C_RESETCLOCK	: Set clock line to 0
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGI2C_PPFunction(unsigned long ulFlag);

/* ------------------------------------------------------------
// FUNCTION		: REGI2C_DDCFunction
// ------------------------------------------------------------
// DESCRIPTION	: Function to manage an i2c on VGA connector
//				  DDC lines
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulFlag		: Operation to process
//				  REGI2C_INIT		: Enable i2c lines 
//				  REGI2C_SETDATA	: Set data line to 1
//				  REGI2C_RESETDATA	: Set data line to 0
//				  REGI2C_SETCLOCK	: Set clock line to 1
//				  REGI2C_RESETCLOCK	: Set clock line to 0
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGI2C_DDCFunction(unsigned long ulFlag);

/* ------------------------------------------------------------
// FUNCTION		: REGI2C_WriteRegister
// ------------------------------------------------------------
// DESCRIPTION	: Write a register to an i2c chip 
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// pfctFunction	: Function to manage the i2c data and clock lines
// ucID			: I2c ID of the chip
// ucReg		: Register to access
// ucData		: Data to write
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGI2C_WriteRegister(REGI2C_FUNCTION pfctFunction, unsigned char ucID, unsigned char ucReg, unsigned char ucData);

#endif





