/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: osal.h
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libosal\linux
// PORTABILITY	: Linux Operating System
// ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: July, 18th 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Functions to interface the library with a Linux
//				  Operating System
// ------------------------------------------------------------
// VARIABLES	: -
// ------------------------------------------------------------
// MACROS		: -
// ------------------------------------------------------------
// FUNCTIONS	: OSAL_Init
// 				  OSAL_Free
//				  OSAL_IOMap
//				  OSAL_MemoryMap
//				  OSAL_LINEAR
//				  OSAL_GetLinearPointer
//				  OSAL_IORead...
//				  OSAL_IOWrite...
//				  OSAL_MemoryRead...
//				  OSAL_MemoryWrite...
//				  OSAL_MemoryAlloc
//				  OSAL_MemoryGetPtr
//				  OSAL_MemoryFree
//				  OSAL_IsFileValid
//				  OSAL_OpenFileToRead
//				  OSAL_OpenFileToWrite
//				  OSAL_ReadFile
//				  OSAL_WriteFile
//				  OSAL_CloseFile
//				  OSAL_DisplayString
//				  OSAL_DisplayHexa
//				  OSAL_DisplayDecimal
//				  OSAL_DisplayCR
//				  OSAL_Wait
//				  OSAL_WaitKey
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: July, 18th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: New functions for GDK1.2
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Initial source code
// ------------------------------------------------------------*/

#ifndef OSAL_H
#define OSAL_H

#include <linux/mm.h>
#include <asm/io.h>
#include <linux/slab.h>
#include <linux/delay.h>

/* ------------------------------------------------------------
// FUNCTION		: OSAL_Init
// ------------------------------------------------------------
// DESCRIPTION	: Init the OS Abstraction Layer library
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define OSAL_Init()

/* ------------------------------------------------------------
// FUNCTION		: OSAL_Free
// ------------------------------------------------------------
// DESCRIPTION	: Free resources allocated by the OSAL library
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define OSAL_Free()

/* ------------------------------------------------------------
// FUNCTION		: OSAL_IOMap
// ------------------------------------------------------------
// DESCRIPTION	: Map a range of I/O in the CPU address space
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// base			: Base physical address of the range
// size			: Size in bytes of the range to map
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define OSAL_IOMap(base, size) ( (unsigned long) base )

/* ------------------------------------------------------------
// FUNCTION		: OSAL_MemoryMap
// ------------------------------------------------------------
// DESCRIPTION	: Map arange of memory in the CPU address space
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBase		: Base physical address of the range
// ulSize		: Size in bytes of the range to map
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define OSAL_MemoryMap(ulBase, ulSize )  ( (unsigned long)ioremap(ulBase, ulSize) )

/* ------------------------------------------------------------
// FUNCTION		: OSAL_GetLinearPointer
// ------------------------------------------------------------
// DESCRIPTION	: Get a CPU linear offset in fs segment inside a
//				  mapped memory range
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// virtual		: Concerned memory range (get from OSAL_MemoryMap)
// offset		: Offset in the range of the address to get
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Linear address
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define OSAL_GetLinearPointer(virtual, offset) (virtual + offset)

/* ------------------------------------------------------------
// FUNCTION		: OSAL_LINEAR
// ------------------------------------------------------------
// DESCRIPTION	: Convert a C pointer into linear offset in the
//				  fs code segment
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// address		: C pointer to convert
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Linear offset
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define OSAL_LINEAR(address) ((unsigned long)(address))

/* ------------------------------------------------------------
// FUNCTION		: OSAL_IO...
// ------------------------------------------------------------
// DESCRIPTION	: Access to an I/O
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulVirtual	: Concerned I/O range (get from OSAL_IOMap)
// ulOffset		: Offset in the range of the I/O to access
// ??Data		: Data to write when write access
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Readed data when read access
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define OSAL_IOWrite8(ulVirtual, ulOffset, ucData)	{ outb_p(ucData, ulVirtual + ulOffset); }
#define OSAL_IOWrite16(ulVirtual, ulOffset, usData)	{ outw_p(usData, ulVirtual + ulOffset); }
#define OSAL_IOWrite32(ulVirtual, ulOffset, ulData)	{ outl_p(ulData, ulVirtual + ulOffset); }
#define OSAL_IORead8(ulVirtual, ulOffset)	( (unsigned char)inb_p(ulVirtual + ulOffset) )
#define OSAL_IORead16(ulVirtual, ulOffset)	( (unsigned short)inw_p(ulVirtual + ulOffset) )
#define OSAL_IORead32(ulVirtual, ulOffset)	( (unsigned long)inl_p(ulVirtual + ulOffset) )

/* ------------------------------------------------------------
// FUNCTION		: OSAL_Memory...
// ------------------------------------------------------------
// DESCRIPTION	: Access to a memory address
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// virtual		: Concerned memory range (get from OSAL_MemoryMap)
// offset		: Offset in the range of the memory to access
// data			: Data to write when write access
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Readed data when read access
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define OSAL_MemoryWrite8(virtual, offset, data) { *((unsigned char *)(virtual + offset)) = data; }
#define OSAL_MemoryWrite16(virtual, offset, data) { *((unsigned short *)(virtual + offset)) = data; }
#define OSAL_MemoryWrite32(virtual, offset, data) { *((unsigned long *)(virtual + offset)) = data; }
#define OSAL_MemoryRead8(virtual, offset) ( *((unsigned char *)(virtual + offset)) ) 
#define OSAL_MemoryRead16(virtual, offset) ( *((unsigned short *)(virtual + offset)) ) 
#define OSAL_MemoryRead32(virtual, offset) ( *((unsigned long *)(virtual + offset)) ) 

/* ------------------------------------------------------------
// FUNCTION		: OSAL_MemoryAlloc
// ------------------------------------------------------------
// DESCRIPTION	: Alloc a block in system meory
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// size			: Size in bytes of the block to allocate
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Handle of the allocated block
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define OSAL_MemoryAlloc(size) ( (unsigned long)kmalloc(size, GFP_KERNEL) )

/* ------------------------------------------------------------
// FUNCTION		: OSAL_MemoryGetPtr
// ------------------------------------------------------------
// DESCRIPTION	: Get a C pointer from an allocated block
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// handle		: Handle of the allocated block
//				  (from OSAL_MemoryAlloc)
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: C pointer on the block
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define OSAL_MemoryGetPtr(handle) ((void *)handle)

/* ------------------------------------------------------------
// FUNCTION		: OSAL_MemoryFree
// ------------------------------------------------------------
// DESCRIPTION	: Free an allocated block
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// handle		: Handle of the allocated block to free
//				  (from OSAL_MemoryAlloc)
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define OSAL_MemoryFree(handle) { kfree((void *)handle); }

/* ------------------------------------------------------------
// FUNCTION		: OSAL_IsFileValid
// ------------------------------------------------------------
// DESCRIPTION	: Check if a recently open file is valid
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// handle		: Handle of the file to check (this handle come
//				  from OSAL_OpenFileTo... function)
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define OSAL_IsFileValid(handle) (handle != (unsigned long)NULL)

/* ------------------------------------------------------------
// FUNCTION		: OSAL_OpenFileToRead
// ------------------------------------------------------------
// DESCRIPTION	: Open a file in read mode
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// name			: Name of the file to open
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Handle on the opened file
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define OSAL_OpenFileToRead(name) ((unsigned long)fopen(name, "rb")) 

/* ------------------------------------------------------------
// FUNCTION		: OSAL_OpenFileToRead
// ------------------------------------------------------------
// DESCRIPTION	: Open a file in write mode
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// name			: Name of the file to open
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Handle on the opened file
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define OSAL_OpenFileToWrite(name) ((unsigned long)fopen(name, "wb")) 

/* ------------------------------------------------------------
// FUNCTION		: OSAL_ReadFile
// ------------------------------------------------------------
// DESCRIPTION	: Read data from a file
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// handle		: Handle of the file to read (this handle come
//				  from OSAL_OpenFileToRead function)
// buffer		: C pointer on the buffer to fill
// size			: Number of bytes to read from the file
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define OSAL_ReadFile(handle, buffer, size) ( fread(buffer, 1, size, (FILE *)handle) )

/* ------------------------------------------------------------
// FUNCTION		: OSAL_WriteFile
// ------------------------------------------------------------
// DESCRIPTION	: Write data into a file
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// handle		: Handle of the file to write (this handle come
//				  from OSAL_OpenFileToWrite function)
// buffer		: C pointer on the buffer to write
// size			: Number of bytes to write into the file
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define OSAL_WriteFile(handle, buffer, size) { fwrite(buffer, 1, size, (FILE *)handle); }

/* ------------------------------------------------------------
// FUNCTION		: OSAL_CloseFile
// ------------------------------------------------------------
// DESCRIPTION	: Close a file
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// handle		: Handle of the file to close (this handle come
//				  from OSAL_OpenFileTo... function)
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define OSAL_CloseFile(handle) { fclose((FILE *)handle); }

/* ------------------------------------------------------------
// FUNCTION		: OSAL_DisplayString
// ------------------------------------------------------------
// DESCRIPTION	: Display a string in the standard output
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// name			: String to display
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define OSAL_DisplayString(name) 	{ printk(name); }

/* ------------------------------------------------------------
// FUNCTION		: OSAL_DisplayString
// ------------------------------------------------------------
// DESCRIPTION	: Display a numeric value in hexadecimal
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// name			: Value to display
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define OSAL_DisplayHexa(name) 		{ printk("%lx", (unsigned long)name); }

/* ------------------------------------------------------------
// FUNCTION		: OSAL_DisplayString
// ------------------------------------------------------------
// DESCRIPTION	: Display a numeric value in decimal
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// name			: Value to display
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define OSAL_DisplayDecimal(name) 	{ printk("%li", (unsigned long)name); }

/* ------------------------------------------------------------
// FUNCTION		: OSAL_DisplayCR
// ------------------------------------------------------------
// DESCRIPTION	: Display a carriage return in the standard output
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define OSAL_DisplayCR() 			{ printk("\n"); }

/* ------------------------------------------------------------
// FUNCTION		: OSAL_Wait
// ------------------------------------------------------------
// DESCRIPTION	: Wait
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulTime		: Time to wait in ms
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define OSAL_Wait(ulTime)		{ udelay(ulTime * 1000); }

/* ------------------------------------------------------------
// FUNCTION		: OSAL_WaitKey
// ------------------------------------------------------------
// DESCRIPTION	: Wait for a key
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define OSAL_WaitKey()		{ getchar(); }

/* ------------------------------------------------------------
// FUNCTION		: OSAL_GetCurrentTime
// ------------------------------------------------------------
// DESCRIPTION	: Get current time in millisecond. This function
//				  is used to compute interval of time and
//				  the time reference is not important.
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Time in milliseconds
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
#define OSAL_GetCurrentTime()		((((unsigned long)clock()) * 1000L) / CLOCKS_PER_SEC)

#endif
