/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: pci.h
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libreg\pci
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE:		: March, 1st 2000
// AUTHOR:		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Functions to manage PCI bus access
// ------------------------------------------------------------
// VARIABLES	: -
// ------------------------------------------------------------
// MACROS		: -
// ------------------------------------------------------------
// FUNCTIONS	: REGPCI_ConfigSpaceRead8
//				  REGPCI_ConfigSpaceRead16
//				  REGPCI_ConfigSpaceRead32
//				  REGPCI_ConfigSpaceWrite8
//				  REGPCI_ConfigSpaceWrite16
//				  REGPCI_ConfigSpaceWrite32
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

#ifndef PCI_H
#define PCI_H

#include "pci.h"
#include "host.h"

/* ------------------------------------------------------------
// FUNCTION		: REGPCI_ConfigSpaceRead8
// ------------------------------------------------------------
// DESCRIPTION	: Read an 8 bits PCI config space register
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBus		: PCI bus number
// ulDevice		: Device number in the bus
// ulFct		: Function number in the device
// ulReg		: Register to access
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Data from the register
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned char REGPCI_ConfigSpaceRead8(unsigned long ulBus, unsigned long ulDevice, unsigned long ulFunction, unsigned long ulReg);

/* ------------------------------------------------------------
// FUNCTION		: REGPCI_ConfigSpaceRead16
// ------------------------------------------------------------
// DESCRIPTION	: Read an 16 bits PCI config space register
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBus		: PCI bus number
// ulDevice		: Device number in the bus
// ulFct		: Function number in the device
// ulReg		: Register to access
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Data from the register
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned short REGPCI_ConfigSpaceRead16(unsigned long ulBus, unsigned long ulDevice, unsigned long ulFunction, unsigned long ulReg);

/* ------------------------------------------------------------
// FUNCTION		: REGPCI_ConfigSpaceRead32
// ------------------------------------------------------------
// DESCRIPTION	: Read an 32 bits PCI config space register
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBus		: PCI bus number
// ulDevice		: Device number in the bus
// ulFct		: Function number in the device
// ulReg		: Register to access
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Data from the register
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long REGPCI_ConfigSpaceRead32(unsigned long ulBus, unsigned long ulDevice, unsigned long ulFunction, unsigned long ulReg);

/* ------------------------------------------------------------
// FUNCTION		: REGPCI_ConfigSpaceWrite8
// ------------------------------------------------------------
// DESCRIPTION	: Write an 8 bits PCI config space register
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBus		: PCI bus number
// ulDevice		: Device number in the bus
// ulFct		: Function number in the device
// ulReg		: Register to access
// ucData		: Data to write
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGPCI_ConfigSpaceWrite8(unsigned long ulBus, unsigned long ulDevice, unsigned long ulFunction, unsigned long ulReg, unsigned char ucData);

/* ------------------------------------------------------------
// FUNCTION		: REGPCI_ConfigSpaceWrite16
// ------------------------------------------------------------
// DESCRIPTION	: Write an 16 bits PCI config space register
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBus		: PCI bus number
// ulDevice		: Device number in the bus
// ulFct		: Function number in the device
// ulReg		: Register to access
// usData		: Data to write
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGPCI_ConfigSpaceWrite16(unsigned long ulBus, unsigned long ulDevice, unsigned long ulFunction, unsigned long ulReg, unsigned short usData);

/* ------------------------------------------------------------
// FUNCTION		: REGPCI_ConfigSpaceWrite32
// ------------------------------------------------------------
// DESCRIPTION	: Write an 32 bits PCI config space register
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBus		: PCI bus number
// ulDevice		: Device number in the bus
// ulFct		: Function number in the device
// ulReg		: Register to access
// ulData		: Data to write
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGPCI_ConfigSpaceWrite32(unsigned long ulBus, unsigned long ulDevice, unsigned long ulFunction, unsigned long ulReg, unsigned long ulData);

#endif

