/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: saa.c
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libhard\saa
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE:		: March, 1st 2000
// AUTHOR:		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Functions to manage SAA7111 tv encoder
// ------------------------------------------------------------
// VARIABLES	: HWSAA_pfctRegAccess
// ------------------------------------------------------------
// MACROS		: -
// ------------------------------------------------------------
// FUNCTIONS	: HWSAA_AccessDDCRegister
//				  HWSAA_AccessPPRegister
//				  HWSAA_AccessSMCGPIORegister
//				  HWSAA_DetectHardware
//				  HWSAA_Init
//				  HWSAA_ChangeInput
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

#include "trace.h"
#include "saa.h"
#include "host.h"
#include "i2c.h"
#include "generic.h"

/* ------------------------------------------------------------
// VARIABLE		: HWSAA_pfctRegAccess 
// ------------------------------------------------------------
// DESCRIPTION	: Functions to access SAA registers on the
//				  current hardware
// ------------------------------------------------------------*/
HWSAA_REGISTERACCESS HWSAA_pfctRegAccess;

/* ------------------------------------------------------------
// FUNCTION		: HWSAA_AccessPPRegister
// ------------------------------------------------------------
// DESCRIPTION	: Access to SAA registers on parallel port i2c
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ucReg		: Register number to access
// ucData		: Data to write 
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWSAA_AccessPPRegister(unsigned char ucReg, unsigned char ucData)
{
	REGI2C_WriteRegister(REGI2C_LINES_PP, 0x48, ucReg, ucData);
}

/* ------------------------------------------------------------
// FUNCTION		: HWSAA_AccessSMCGPIORegister
// ------------------------------------------------------------
// DESCRIPTION	: Access to SAA registers on SMC Super I/O GPIO i2c
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ucReg		: Register number to access
// ucData		: Data to write 
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWSAA_AccessSMCGPIORegister(unsigned char ucReg, unsigned char ucData)
{
	REGI2C_WriteRegister(REGI2C_LINES_SMCGPIO, 0x48, ucReg, ucData);
}

/* ------------------------------------------------------------
// FUNCTION		: HWSAA_AccessDDCRegister
// ------------------------------------------------------------
// DESCRIPTION	: Access to SAA registers on VGA connector i2c
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ucReg		: Register number to access
// ucData		: Data to write 
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWSAA_AccessDDCRegister(unsigned char ucReg, unsigned char ucData)
{
	REGI2C_WriteRegister(REGI2C_LINES_DDC, 0x48, ucReg, ucData);
}

/* ------------------------------------------------------------
// FUNCTION		: HWSAA_DetectHardware
// ------------------------------------------------------------
// DESCRIPTION	: Detect the current SAA wait to access and
//				  initialize it
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWSAA_DetectHardware(void)
{
	unsigned long ulHardwareID;

	DBGTRACE_BEGINFUNCTION("HWSAA_DetectHardware");

	/* Get hardware ID */
	ulHardwareID = HWGENERIC_GetHardwareID();

	/* Initialize the right hardware */ 
	if(ulHardwareID & HWGENERIC_HAS_SMCSAA)
	{
		DBGTRACE_BRANCH("SMC GPIO SAA");

		HWSAA_pfctRegAccess = HWSAA_AccessSMCGPIORegister;
	}
	else if(ulHardwareID & HWGENERIC_HAS_DDCSAA)
	{
		DBGTRACE_BRANCH("VGA connector SAA");

		HWSAA_pfctRegAccess = HWSAA_AccessDDCRegister;
	}
	else
	{
		DBGTRACE_BRANCH("Parallel Port SAA");

		HWSAA_pfctRegAccess = HWSAA_AccessPPRegister;
	}

	DBGTRACE_ENDFUNCTION("HWSAA_DetectHardware");
}

/* ------------------------------------------------------------
// FUNCTION		: HWSAA_Init
// ------------------------------------------------------------
// DESCRIPTION	: Initialize the SAA
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulInput		: Number of the video input
// ulMode		: Type of video (HWSAA_FORMAT_...)
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWSAA_Init(unsigned long ulInput, unsigned long ulMode)
{
	DBGTRACE_BEGINFUNCTION("HWSAA_Init");
	DBGTRACE_NUMERICPARAM(ulInput);
	DBGTRACE_NUMERICPARAM(ulMode);

	/* Detect the type of SAA access */
	HWSAA_DetectHardware();

	/* Initialize the SAA */
	(*HWSAA_pfctRegAccess)(0x2, (unsigned char)(0xc0 + ulInput));
	(*HWSAA_pfctRegAccess)(0x3, 0x22);
	(*HWSAA_pfctRegAccess)(0x4, 0x00);
	(*HWSAA_pfctRegAccess)(0x5, 0x00);
	(*HWSAA_pfctRegAccess)(0x6, 0xeb);
	(*HWSAA_pfctRegAccess)(0x7, 0xe0);
	(*HWSAA_pfctRegAccess)(0x8, 0x88);
	(*HWSAA_pfctRegAccess)(0x9, 0x01);
	(*HWSAA_pfctRegAccess)(0xa, 0x80);
	(*HWSAA_pfctRegAccess)(0xb, 0x47);
	(*HWSAA_pfctRegAccess)(0xc, 0x40);
	(*HWSAA_pfctRegAccess)(0xd, 0x00);
	(*HWSAA_pfctRegAccess)(0xe, (unsigned char)(ulMode));
	(*HWSAA_pfctRegAccess)(0x10, 0xc1);
	(*HWSAA_pfctRegAccess)(0x11, 0x1c);
	(*HWSAA_pfctRegAccess)(0x12, 0x1);
	(*HWSAA_pfctRegAccess)(0x13, 0);
	(*HWSAA_pfctRegAccess)(0x14, 0);
	(*HWSAA_pfctRegAccess)(0x15, 0);
	(*HWSAA_pfctRegAccess)(0x16, 0);
	(*HWSAA_pfctRegAccess)(0x17, 0);
	(*HWSAA_pfctRegAccess)(0x2, (unsigned char)(0xc0 + ulInput));

	DBGTRACE_ENDFUNCTION("HWSAA_Init");
}

/* ------------------------------------------------------------
// FUNCTION		: HWSAA_ChangeInput
// ------------------------------------------------------------
// DESCRIPTION	: Switch the video input
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulInput		: Number of the new video input
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWSAA_ChangeInput(unsigned long ulInput)
{
	DBGTRACE_BEGINFUNCTION("HWSAA_ChangeInput");
	DBGTRACE_NUMERICPARAM(ulInput);

	(*HWSAA_pfctRegAccess)(0x2, (unsigned char)(0xc0 + ulInput));

	DBGTRACE_ENDFUNCTION("HWSAA_ChangeInput");
}

