/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: saa.h
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libhard\saa
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE:		: March, 1st 2000
// AUTHOR:		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Functions to manage SAA7111 tv encoder
// ------------------------------------------------------------
// VARIABLES	: -
// ------------------------------------------------------------
// MACROS		: HWSAA_REGISTERACCESS
//				  HWSAA_FORMAT_...
// ------------------------------------------------------------
// FUNCTIONS	: HWSAA_Init
//				  HWSAA_ChangeInput
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

#ifndef SAA_H
#define SAA_H

#include "vip.h"

/* ------------------------------------------------------------
// DEFINE		: HWSAA_FORMAT_... 
// ------------------------------------------------------------
// DESCRIPTION	: Video input format
// ------------------------------------------------------------*/
#define HWSAA_FORMAT_PAL        		(0x1  + ((unsigned long)(HWVIP_FORMAT_PAL)     << 16))
#define HWSAA_FORMAT_PALBGHI			(0x1  + ((unsigned long)(HWVIP_FORMAT_PALBGHI) << 16))
#define HWSAA_FORMAT_NTSC				(0x1  + ((unsigned long)(HWVIP_FORMAT_NTSC)    << 16))
#define HWSAA_FORMAT_NTSCM				(0x1  + ((unsigned long)(HWVIP_FORMAT_NTSCM)   << 16))
#define HWSAA_FORMAT_NTSC_JAPAN 		(0x1  + ((unsigned long)(HWVIP_FORMAT_NTSC)    << 16))
#define HWSAA_FORMAT_PAL_443			(0x11 + ((unsigned long)(HWVIP_FORMAT_PAL)     << 16))
#define HWSAA_FORMAT_PALM				(0x31 + ((unsigned long)(HWVIP_FORMAT_PAL)     << 16))
#define HWSAA_FORMAT_NTSCN				(0x31 + ((unsigned long)(HWVIP_FORMAT_NTSC)    << 16))
#define HWSAA_FORMAT_PALN				(0x41 + ((unsigned long)(HWVIP_FORMAT_PAL)     << 16))
#define HWSAA_FORMAT_SECAM				(0x51 + ((unsigned long)(HWVIP_FORMAT_SECAM)   << 16))

/* ------------------------------------------------------------
// DEFINE		: HWSAA_REGISTERACCESS 
// ------------------------------------------------------------
// DESCRIPTION	: Function to access SAA registers
// ------------------------------------------------------------*/
typedef void (*HWSAA_REGISTERACCESS)(unsigned char, unsigned char);

/* ------------------------------------------------------------
// FUNCTION		: HWSAA_Init
// ------------------------------------------------------------
// DESCRIPTION	: Initialize the SAA
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulInput		: Number of the video input
// ulMode		: Type of video (HWSAA_FORMAT_...)
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWSAA_Init(unsigned long ulInput, unsigned long ulMode);

/* ------------------------------------------------------------
// FUNCTION		: HWSAA_ChangeInput
// ------------------------------------------------------------
// DESCRIPTION	: Switch the video input
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulInput		: Number of the new video input
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWSAA_ChangeInput(unsigned long ulInput);

#endif

