/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: stv.c
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libhard\stv
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: December, 20th 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Functions to manage TV output
// ------------------------------------------------------------
// VARIABLES	: HWSTV_pfctRegAccess
//				  HWSTV_ulCurrentFormat
// ------------------------------------------------------------
// MACROS		: -
// ------------------------------------------------------------
// FUNCTIONS	: HWSTV_AccessInternalRegister
//				  HWSTV_AccessGPIORegister
//				  HWSTV_DetectHardware
//				  HWSTV_Init
//				  HWSTV_ChangeFormat
//				  HWSTV_FlickerFilter
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: December, 20th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Added direct to TV mode
//				  Added support of Consumer	II DENC 
// ------------------------------------------------------------
// REVISION		: Rev 1.0
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

#include "trace.h"
#include "host.h"
#include "stv.h"
#include "i2c.h"
#include "generic.h"

/* ------------------------------------------------------------
// VARIABLE		: HWSTV_ulCurrentFormat 
// ------------------------------------------------------------
// DESCRIPTION	: Current output format
// ------------------------------------------------------------*/
unsigned long HWSTV_ulCurrentFormat;

/* ------------------------------------------------------------
// VARIABLE		: HWSTV_pfctReadAccess 
// ------------------------------------------------------------
// DESCRIPTION	: Function to read STV registers
// ------------------------------------------------------------*/
HWSTV_READACCESS HWSTV_pfctReadAccess;

/* ------------------------------------------------------------
// VARIABLE		: HWSTV_pfctWriteAccess 
// ------------------------------------------------------------
// DESCRIPTION	: Function to write STV registers
// ------------------------------------------------------------*/
HWSTV_WRITEACCESS HWSTV_pfctWriteAccess;

/* ------------------------------------------------------------
// FUNCTION		: HWSTV_ReadDummyRegister
// ------------------------------------------------------------
// DESCRIPTION	: Return 0xff for any read access
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ucReg		: Number of the register to read
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: 0xff
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned char HWSTV_ReadDummyRegister(unsigned char ucReg)
{
	return 0xff;
}

/* ------------------------------------------------------------
// FUNCTION		: HWSTV_WriteDummyRegister
// ------------------------------------------------------------
// DESCRIPTION	: Simulate a write access to STV
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ucReg		: Number of the register to modify
// ucData		: Data to write
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWSTV_WriteDummyRegister(unsigned char ucReg, unsigned char ucData)
{
}

/* ------------------------------------------------------------
// FUNCTION		: HWSTV_WriteGPIORegister
// ------------------------------------------------------------
// DESCRIPTION	: Access STV by STPC GPIO (STPC Client)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ucReg		: Number of the register to modify
// ucData		: Data to write
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWSTV_WriteGPIORegister(unsigned char ucReg, unsigned char ucData)
{
	REGI2C_WriteRegister(REGI2C_LINES_GPIO, 0x40, ucReg, ucData);
}

/* ------------------------------------------------------------
// FUNCTION		: HWSTV_ReadIORegister
// ------------------------------------------------------------
// DESCRIPTION	: Read STV registers by I/O (STPC Consumer)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ucReg		: Number of the register to read
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Contend of the register
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned char HWSTV_ReadIORegister(unsigned char ucReg)
{
	REGHOST_SET_CHIPSET_REG(0x2b, ucReg);
	return REGHOST_GET_CHIPSET_REG(0x2c);
}

/* ------------------------------------------------------------
// FUNCTION		: HWSTV_WriteIORegister
// ------------------------------------------------------------
// DESCRIPTION	: Access internal STV by I/O (STPC Consumer)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ucReg		: Number of the register to modify
// ucData		: Data to write
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWSTV_WriteIORegister(unsigned char ucReg, unsigned char ucData)
{
	REGHOST_SET_CHIPSET_REG(0x2b, ucReg);
	REGHOST_SET_CHIPSET_REG(0x2c, ucData);
}

/* ------------------------------------------------------------
// FUNCTION		: HWSTV_ReadMemoryRegister
// ------------------------------------------------------------
// DESCRIPTION	: Read STV register through memory space (STPC Consumer-S/II)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ucReg		: Number of the register to read
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Register value
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned char HWSTV_ReadMemoryRegister(unsigned char ucReg)
{
	return REGHOST_GET_TVOUTPUT_REG(ucReg);
}

/* ------------------------------------------------------------
// FUNCTION		: HWSTV_WriteMemoryRegister
// ------------------------------------------------------------
// DESCRIPTION	: Write STV register through memory space (STPC Consumer-S/II)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ucReg		: Number of the register to modify
// ucData		: Data to write
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWSTV_WriteMemoryRegister(unsigned char ucReg, unsigned char ucData)
{
	REGHOST_SET_TVOUTPUT_REG(ucReg, ucData);
}

/* ------------------------------------------------------------
// FUNCTION		: HWSTV_DetectHardware
// ------------------------------------------------------------
// DESCRIPTION	: Detect the way to access the STV in the
//				  current hardware and initialize it
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWSTV_DetectHardware(void)
{
	unsigned long ulHardwareID;

	/* Get the ID of the current hardware */
	ulHardwareID = HWGENERIC_GetHardwareID();

	/* Detect the type of STV */
	if(ulHardwareID & HWGENERIC_HAS_IOTV)
	{
		DBGTRACE_BRANCH("Internal STV by I/O");

		HWSTV_pfctReadAccess = HWSTV_ReadIORegister;
		HWSTV_pfctWriteAccess = HWSTV_WriteIORegister;
	}
	else if(ulHardwareID & HWGENERIC_HAS_MEMTV)
	{
		DBGTRACE_BRANCH("Internal STV by memory");

		HWSTV_pfctReadAccess = HWSTV_ReadMemoryRegister;
		HWSTV_pfctWriteAccess = HWSTV_WriteMemoryRegister;
	}
	else if(ulHardwareID & HWGENERIC_HAS_EXTTV)
	{
		DBGTRACE_BRANCH("GPIO STV");

		/* Reset STV119 */
		REGHOST_SET_CHIPSET_REG(0x7b, (unsigned char)(REGHOST_GET_CHIPSET_REG(0x7b) & ~0x2));
		OSAL_Wait(1);
		REGHOST_SET_CHIPSET_REG(0x7b, (unsigned char)(REGHOST_GET_CHIPSET_REG(0x7b) | 0x2));

		/* Set input clock = DCLK */
		REGHOST_SET_CHIPSET_REG(0x7b, (unsigned char)(REGHOST_GET_CHIPSET_REG(0x7b) | 0x40));

		/* Read access to GPIO i2c is not possible */
		HWSTV_pfctReadAccess = HWSTV_ReadDummyRegister;
		HWSTV_pfctWriteAccess = HWSTV_WriteGPIORegister;
	}
	else
	{
		DBGTRACE_WARNING("No STV on this platform");

		HWSTV_pfctReadAccess = HWSTV_ReadDummyRegister;
		HWSTV_pfctWriteAccess = HWSTV_WriteDummyRegister;
	}
}

/* ------------------------------------------------------------
// FUNCTION		: HWSTV_Init
// ------------------------------------------------------------
// DESCRIPTION	: Initialize the STV
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulFormat		: Video format (HWSTV_FORMAT_...) 
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWSTV_Init(unsigned long ulFormat)
{
	DBGTRACE_BEGINFUNCTION("HWSTV_Init");
	DBGTRACE_NUMERICPARAM(ulFormat);

	HWSTV_DetectHardware();

	HWSTV_ChangeFormat(ulFormat);
	HWSTV_FlickerFilter(HWSTV_FILTER_Y121);

	DBGTRACE_ENDFUNCTION("HWSTV_Init");
}

/* ------------------------------------------------------------
// FUNCTION		: HWSTV_ChangeFormat
// ------------------------------------------------------------
// DESCRIPTION	: Change the video format
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulFormat		: Video format (HWSTV_FORMAT_...)
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWSTV_ChangeFormat(unsigned long ulFormat)
{
	DBGTRACE_BEGINFUNCTION("HWSTV_ChangeFormat");
	DBGTRACE_NUMERICPARAM(ulFormat);

	/* RGB or SVHS? */
	if((ulFormat & 0xff00) == HWSTV_FORMAT_SVHS)
	{
		REGHOST_SET_CRTC_REG(0x42, REGHOST_GET_CRTC_REG(0x42) | 0x08);
	}
	else
	{
		REGHOST_SET_CRTC_REG(0x42, REGHOST_GET_CRTC_REG(0x42) & ~0x08);
	}

	/* Now check only the format */
	ulFormat &= 0xff;

	HWSTV_ulCurrentFormat = ulFormat;

	switch(ulFormat)
	{
		case HWSTV_FORMAT_NTSC:
		case HWSTV_FORMAT_NTSC269:

			if(HWGENERIC_GetHardwareID() & HWGENERIC_HAS_STVV8)
			{
				HWSTV_WriteRegister(0x00, 0x94);
				HWSTV_WriteRegister(0x05, 0x80);
			}
			/* CCIR601 if internal STV, CCIR656 if external STV */
			else if(HWGENERIC_GetHardwareID() & (HWGENERIC_HAS_IOTV | HWGENERIC_HAS_MEMTV))
			{
				HWSTV_WriteRegister(0x00, 0x91);
				HWSTV_WriteRegister(0x05, 0x01);
			}
			else
			{
				HWSTV_WriteRegister(0x00, 0x99);
				HWSTV_WriteRegister(0x05, 0x31);
			}

			HWSTV_WriteRegister(0x01, 0x14);
			HWSTV_WriteRegister(0x03, 0x80);

			/* Modify synchro delay for STV v8 */
			if(HWGENERIC_GetHardwareID() & HWGENERIC_HAS_STVV8)
			{
				HWSTV_WriteRegister(0x04, 0xc8);
			}
			else
			{
				HWSTV_WriteRegister(0x04, 0x80);
			}

			if(ulFormat == HWSTV_FORMAT_NTSC269)
			{
				if(HWGENERIC_GetHardwareID() & HWGENERIC_HAS_STVV8)
				{
					HWSTV_WriteRegister(0x0a, 0x22);
					HWSTV_WriteRegister(0x0b, 0x04);
					HWSTV_WriteRegister(0x0c, 0xf8);
				}
				else
				{
					HWSTV_WriteRegister(0x0a, 0x22);
					HWSTV_WriteRegister(0x0b, 0x04);
					HWSTV_WriteRegister(0x0c, 0xc9);
				}
			}
			else
			{
				HWSTV_WriteRegister(0x0a, 0x21);
				HWSTV_WriteRegister(0x0b, 0xf0);
				HWSTV_WriteRegister(0x0c, 0x7c);
			}
			break;

		case HWSTV_FORMAT_PAL:
			/* CCIR601 if internal STV, CCIR656 if external STV */
			if(HWGENERIC_GetHardwareID() & (HWGENERIC_HAS_IOTV | HWGENERIC_HAS_MEMTV))
			{
				HWSTV_WriteRegister(0x00, 0x09);
			}
			else
			{
				HWSTV_WriteRegister(0x00, 0x19);
			}

			/* Output on board amplifier is inverted */
			if(HWGENERIC_GetHardwareID() & HWGENERIC_HAS_STVINVERTEDPOLARITY)
			{
				HWSTV_WriteRegister(0x05, 0x80);
			}
			else
			{
				HWSTV_WriteRegister(0x05, 0x01);
			}

			HWSTV_WriteRegister(0x01, 0xf4);
			HWSTV_WriteRegister(0x02, 0x61);
			HWSTV_WriteRegister(0x03, 0xc0);
			HWSTV_WriteRegister(0x04, 0x0);

			HWSTV_WriteRegister(0x0a, 0x2a);
			HWSTV_WriteRegister(0x0b, 0x09);
			HWSTV_WriteRegister(0x0c, 0x8b);
			break;
	}

	/* Disable SCAM for STV v8 */
	if(HWGENERIC_GetHardwareID() & HWGENERIC_HAS_STVV8)
	{
		HWSTV_WriteRegister(0x07, 0x0);
		HWSTV_WriteRegister(0x08, 0x20);
	}

	/* Reset the STV to take into account new parameters */
	HWSTV_WriteRegister(0x02, 0x28);
	HWSTV_WriteRegister(0x06, 0x90);

	DBGTRACE_ENDFUNCTION("HWSTV_ChangeFormat");
}

/* ------------------------------------------------------------
// FUNCTION		: HWSTV_FlickerFilter
// ------------------------------------------------------------
// DESCRIPTION	: Change the flicker filter
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulFilter		: Flicker filter type (HWSTV_FILTER_...) 
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWSTV_FlickerFilter(unsigned long ulFilter)
{
	unsigned char ucCR40;

	DBGTRACE_BEGINFUNCTION("HWSTV_FlickerFilter");
	DBGTRACE_NUMERICPARAM(ulFilter);

	ucCR40 = REGHOST_GET_CRTC_REG(0x40) & 0xf8;
	ucCR40 |= ulFilter;
	REGHOST_SET_CRTC_REG(0x40, ucCR40);

	DBGTRACE_ENDFUNCTION("HWSTV_FlickerFilter");
}

/* ------------------------------------------------------------
// FUNCTION		: HWSTV_DirectToTV
// ------------------------------------------------------------
// DESCRIPTION	: Set or reset VIP direct to TV mode
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulDirect		: 0 to reset, <> 0 to set 
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWSTV_DirectToTV(unsigned long ulDirect)
{
	unsigned long ulHardwareID;

	ulHardwareID = HWGENERIC_GetHardwareID();

	if(ulDirect)
	{
		/* Enable direct to TV */
		REGHOST_SET_CRTC_REG(0x40, REGHOST_GET_CRTC_REG(0x40) & 0xcf);

		/* Set VCLK as input of the STV */
		if(ulHardwareID & HWGENERIC_HAS_EXTTV)
		{
			/* Use the GPIO to access external switch */
			REGHOST_SET_CHIPSET_REG(0x7b, REGHOST_GET_CHIPSET_REG(0x7b) & 0xbf);
		}
		else if(ulHardwareID == HWGENERIC_PLATFORM_EVAL110)
		{
			/* Set DCLK as external */
			REGHOST_SET_CHIPSET_REG(0x4c, REGHOST_GET_CHIPSET_REG(0x4c) & 0xbf);
			
			/* Set VCLK on external DCLK pin using GPIO */
			REGHOST_SET_SMCSUPERIO_REG(0x01, REGHOST_GET_SMCSUPERIO_REG(0x01) & 0xef);
		}
		else
		{
			REGHOST_SET_CHIPSET_REG(0x4c, REGHOST_GET_CHIPSET_REG(0x4c) | 0x10);
		}

		/* Switch from CCIR656 to CCIR601 if necessary */
		if( (!(ulHardwareID & HWGENERIC_HAS_EXTTV)) &&
			  (HWSTV_ulCurrentFormat != HWSTV_FORMAT_PAL) )
		{
			if(HWGENERIC_GetHardwareID() & HWGENERIC_HAS_STVV8)
			{
				HWSTV_WriteRegister(0x00, 0x9c);
//				HWSTV_WriteRegister(0x00, 0x94);
				HWSTV_WriteRegister(0x04, 0x00);
			}
			else
			{
				HWSTV_WriteRegister(0x00, 0x99);
				HWSTV_WriteRegister(0x04, 0x00);
			}
		}
	}
	else
	{
		/* Disable direct to TV */
		REGHOST_SET_CRTC_REG(0x40, (REGHOST_GET_CRTC_REG(0x40) & 0x0f) | 0xe0);

		/* Set DCLK as input of the STV */
		if(ulHardwareID & HWGENERIC_HAS_EXTTV)
		{
			/* Use the GPIO to access external switch */
			REGHOST_SET_CHIPSET_REG(0x7b, REGHOST_GET_CHIPSET_REG(0x7b) | 0x40);
		}
		else if(ulHardwareID == HWGENERIC_PLATFORM_EVAL110)
		{
			/* Set DCLK as external */
			REGHOST_SET_CHIPSET_REG(0x4c, REGHOST_GET_CHIPSET_REG(0x4c) | 0x40);
			
			/* Set VCLK on external DCLK pin using GPIO */
			REGHOST_SET_SMCSUPERIO_REG(0x01, REGHOST_GET_SMCSUPERIO_REG(0x01) | 0x10);
		}
		else
		{
			REGHOST_SET_CHIPSET_REG(0x4c, REGHOST_GET_CHIPSET_REG(0x4c) & 0xef);
		}

		/* Switch from CCIR601 to CCIR656 if necessary */
		if( (!(ulHardwareID & HWGENERIC_HAS_EXTTV)) &&
			  (HWSTV_ulCurrentFormat != HWSTV_FORMAT_PAL) )
		{
			/* Enable bottom/top signal for CCIR601  */
			REGHOST_SET_CRTC_REG(0x40, REGHOST_GET_CRTC_REG(0x40) | 0x10);
			if(HWGENERIC_GetHardwareID() & HWGENERIC_HAS_STVV8)
			{
				HWSTV_WriteRegister(0x00, 0x94);
				HWSTV_WriteRegister(0x04, 0xc8);
			}
			else
			{
				HWSTV_WriteRegister(0x00, 0x91);
				HWSTV_WriteRegister(0x04, 0x80);
			}
		}
	}
}


