/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : www.st.com - STPC product
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: stv.h
// LIBRARY		: STPC Graphic Development Kit
// MODULE		: libhard\stv
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE:		: March, 1st 2000
// AUTHOR:		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Functions to manage TV output
// ------------------------------------------------------------
// VARIABLES	: HWSTV_pfctRegAccess
// ------------------------------------------------------------
// MACROS		: -
// ------------------------------------------------------------
// FUNCTIONS	: HWSTV_Init
//				  HWSTV_ChangeFormat
//				  HWSTV_FlickerFilter
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

#ifndef STV_H
#define STV_H

/* ------------------------------------------------------------
// DEFINE		: HWSTV_READACCESS 
// 			  HWSTV_WRITEACCESS 
// ------------------------------------------------------------
// DESCRIPTION	: Function to access STV registers
// ------------------------------------------------------------*/
typedef unsigned char (*HWSTV_READACCESS)(unsigned char);
typedef void (*HWSTV_WRITEACCESS)(unsigned char, unsigned char);

/* ------------------------------------------------------------
// DEFINE		: HWSTV_FORMAT_... 
// ------------------------------------------------------------
// DESCRIPTION	: TV output format (NTSC269 = NTSC with an external 
//				  clock of 26.9... MHz in place of 27MHz)
// ------------------------------------------------------------*/
#define HWSTV_FORMAT_PAL				0x0001
#define HWSTV_FORMAT_NTSC				0x0002
#define HWSTV_FORMAT_NTSC269			0x0003
#define HWSTV_FORMAT_NTSCINTERNAL		0x0004
#define HWSTV_FORMAT_SVHS				0x0100
#define HWSTV_FORMAT_PALSVHS			(HWSTV_FORMAT_PAL | HWSTV_FORMAT_SVHS)
#define HWSTV_FORMAT_NTSCSVHS			(HWSTV_FORMAT_NTSC | HWSTV_FORMAT_SVHS)
#define HWSTV_FORMAT_NTSC269SVHS		(HWSTV_FORMAT_NTSC269 | HWSTV_FORMAT_SVHS)

/* ------------------------------------------------------------
// DEFINE		: HWSTV_FILTER_... 
// ------------------------------------------------------------
// DESCRIPTION	: Flicker filter type
// ------------------------------------------------------------
// HWSTV_FILTER_DISABLE	: No flicker filter
// HWSTV_FILTER_011		: [0, 1, 1] vertical filter
// HWSTV_FILTER_121		: [1, 2, 1] vertical filter
// HWSTV_FILTER_131		: [1, 3, 1] vertical filter
// HWSTV_FILTER_Y011	: [0, 1, 1] vertical filter only on luminance
// HWSTV_FILTER_Y121	: [1, 2, 1] vertical filter only on luminance
// HWSTV_FILTER_Y131	: [1, 3, 1] vertical filter only on luminance
// ------------------------------------------------------------*/
#define HWSTV_FILTER_DISABLE		0
#define HWSTV_FILTER_011			1
#define HWSTV_FILTER_121			2
#define HWSTV_FILTER_131			3
#define HWSTV_FILTER_Y011			5
#define HWSTV_FILTER_Y121			6
#define HWSTV_FILTER_Y131			7

/* ------------------------------------------------------------
// VARIABLE		: HWSTV_pfctReadAccess 
// ------------------------------------------------------------
// DESCRIPTION	: Function to read STV registers
// ------------------------------------------------------------*/
extern HWSTV_READACCESS HWSTV_pfctReadAccess;

/* ------------------------------------------------------------
// VARIABLE		: HWSTV_pfctWriteAccess 
// ------------------------------------------------------------
// DESCRIPTION	: Function to write STV registers
// ------------------------------------------------------------*/
extern HWSTV_WRITEACCESS HWSTV_pfctWriteAccess;

/* ------------------------------------------------------------
// DEFINE		: HWSTV_ReadRegister 
// ------------------------------------------------------------
// DESCRIPTION	: Function to read an STV register
// ------------------------------------------------------------*/
#define HWSTV_ReadRegister(reg) ( (*HWSTV_pfctReadAccess)(reg) )

/* ------------------------------------------------------------
// DEFINE		: HWSTV_WriteRegister 
// ------------------------------------------------------------
// DESCRIPTION	: Function to write an STV register
// ------------------------------------------------------------*/
#define HWSTV_WriteRegister(reg,val) { (*HWSTV_pfctWriteAccess)(reg, val); }

/* ------------------------------------------------------------
// FUNCTION		: HWSTV_DetectHardware
// ------------------------------------------------------------
// DESCRIPTION	: Detect the way to access the STV in the
//				  current hardware and initialize it
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWSTV_DetectHardware(void);

/* ------------------------------------------------------------
// FUNCTION		: HWSTV_Init
// ------------------------------------------------------------
// DESCRIPTION	: Initialize the STV
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulFormat		: Video format (HWSTV_FORMAT_...) 
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWSTV_Init(unsigned long ulFormat);

/* ------------------------------------------------------------
// FUNCTION		: HWSTV_ChangeFormat
// ------------------------------------------------------------
// DESCRIPTION	: Change the video format
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulFormat		: Video format (HWSTV_FORMAT_...) 
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWSTV_ChangeFormat(unsigned long ulFormat);

/* ------------------------------------------------------------
// FUNCTION		: HWSTV_FlickerFilter
// ------------------------------------------------------------
// DESCRIPTION	: Change the flicker filter
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulFilter		: Flicker filter type (HWSTV_FILTER_...) 
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWSTV_FlickerFilter(unsigned long ulFilter);

/* ------------------------------------------------------------
// FUNCTION		: HWSTV_DirectToTV
// ------------------------------------------------------------
// DESCRIPTION	: Set or reset VIP direct to TV mode
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulDirect		: 0 to reset, <> 0 to set 
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWSTV_DirectToTV(unsigned long ulDirect);

#endif
