/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: tft.c
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libhard\tft
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE:		: March, 1st 2000
// AUTHOR:		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Functions to manage tft interface
// ------------------------------------------------------------
// VARIABLES	: -
// ------------------------------------------------------------
// MACROS		: -
// ------------------------------------------------------------
// FUNCTIONS	: HWTFT_InitializeInterface
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// MODIFICATIONS: Initial source code
// ------------------------------------------------------------*/

#include "trace.h"
#include "host.h"
#include "tft.h"
#include "generic.h"

/* ------------------------------------------------------------
// FUNCTION		: HWTFT_RegisterAccess
// ------------------------------------------------------------
// DESCRIPTION	: Set up a TFT interface register
// ------------------------------------------------------------*/
void (*HWTFT_RegisterAccess)(unsigned long, unsigned long);

/* ------------------------------------------------------------
// FUNCTION		: HWTFT_IORegister
// ------------------------------------------------------------
// DESCRIPTION	: Set up an IO mapped register
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulReg		: Register number
// ulValue		: Value to be set
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWTFT_IORegister(unsigned long ulReg, unsigned long ulValue)
{
	REGHOST_SET_TFTINTERFACE_REG(ulReg, ulValue);
}

/* ------------------------------------------------------------
// FUNCTION		: HWTFT_MemoryRegister
// ------------------------------------------------------------
// DESCRIPTION	: Set up a memory mapped register
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulReg		: Register number
// ulValue		: Value to be set
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWTFT_MemoryRegister(unsigned long ulReg, unsigned long ulValue)
{
	/* manage the differences in registers organization */
	switch(ulReg)
	{
		/* Register 10 became register 12!! */
		case 10:
			ulReg = 12;
			break;

		/* Register 12 is low case of register 10 */
		case 12:
			ulValue = (ulValue & 0xff) + (REGHOST_GET_TFTMEM_REG(20) & 0xff00);
			ulReg = 10;
			break;

		/* Register 13 is high case of register 10 */
		case 13:
			ulValue = ((ulValue & 0xff) << 8) + (REGHOST_GET_TFTMEM_REG(20) & 0xff);
			ulReg = 10;
			break;

		/* Register 14 is low case of register 11 */
		case 14:
			ulValue = (ulValue & 0xff) + (REGHOST_GET_TFTMEM_REG(22) & 0xff00);
			ulReg = 11;
			break;

		/* Register 15 is few bits of register 11 */
		case 15:
			ulValue = ((ulValue & 0x1f) << 11) + (REGHOST_GET_TFTMEM_REG(22) & 0x7ff);
			ulReg = 11;
			break;

		/* Register 11 is few bits of register 11 */
		case 11:
			ulValue = ((ulValue & 0x7) << 8) + (REGHOST_GET_TFTMEM_REG(22) & 0xf8ff);
			ulReg = 11;
			break;
	}

	/* Nothimg special for other registers */
	REGHOST_SET_TFTMEM_REG(ulReg * 2, ulValue);
}

/* ------------------------------------------------------------
// FUNCTION		: HWTFT_InitializeInterface
// ------------------------------------------------------------
// DESCRIPTION	: Initialize the TFT interface
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulTFTType	: Type of TFT screen to initialize
//				  (HWTFT_FORMAT_...)
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWTFT_Init(unsigned long ulTFTType)
{
	unsigned short *pusTFTData;
	int i;

	DBGTRACE_BEGINFUNCTION("HWTFT_InitializeInterface");
	DBGTRACE_NUMERICPARAM(ulTFTType);

	/* IO or memory registers depending on chip */
	if(HWGENERIC_GetHardwareID() & HWGENERIC_HAS_IOTFT)
	{
		HWTFT_RegisterAccess = HWTFT_IORegister;
	}
	else
	{
		HWTFT_RegisterAccess = HWTFT_MemoryRegister;
	}

	/* Find the correct settings in TFT table */
	for(i = 0; HWTFT_ScreenTable[i] != 0xffffffff; i += 2)
	{
		if(ulTFTType == HWTFT_ScreenTable[i])
		{
			pusTFTData = (unsigned short *)HWTFT_ScreenTable[i + 1];
			break;
		}
	}

	if(HWTFT_ScreenTable[i] == 0xffffffff)
	{
		DBGTRACE_WARNING("Unknown TFT screen");

		return;
	}

	/* Initialize TFT interface */
	i = 0;
	while(pusTFTData[i] != 0xff)
	{
		(*HWTFT_RegisterAccess)(pusTFTData[i], pusTFTData[i + 1]);
		i += 2;
	}

	/* set TFT power control */

	/* enable power to TFT power supply */
	(*HWTFT_RegisterAccess)(0xb,0x5);

	/* wait for 15 mili second and enable signals to TFT*/
	OSAL_Wait(15);
	(*HWTFT_RegisterAccess)(0xb, 0x4);

	/* wait for 5 milisec and enable back light */
	OSAL_Wait(5);
	(*HWTFT_RegisterAccess)(0xb,0x6);

	DBGTRACE_ENDFUNCTION("HWTFT_InitializeInterface");
}
