/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: tftdata.h
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libhard\tft
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE:		: March, 1st 2000
// AUTHOR:		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Table of TFT screen settings
// ------------------------------------------------------------
// VARIABLES	: HWTFT_ScreenTable
// ------------------------------------------------------------
// MACROS		: -
// ------------------------------------------------------------
// FUNCTIONS	: -
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: July, 21th 2001
// AUTHOR		: Fabrice Couteau
// MODIFICATIONS: Add 1024x768 TFT screen support
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

#include "tft.h"

/* ------------------------------------------------------------
// VARIABLE		: HWTFT_... 
// ------------------------------------------------------------
// DESCRIPTION	: Table of registers settings
// ------------------------------------------------------------*/
extern unsigned short HWTFT_PhilipsHLD1201[];
extern unsigned short HWTFT_LGPhilipsLM151X2[];

/* ------------------------------------------------------------
// VARIABLE		: HWTFT_ScreenTable 
// ------------------------------------------------------------
// DESCRIPTION	: Table of the TFT screen and the interface
//				  registers settings
// ------------------------------------------------------------*/
unsigned long HWTFT_ScreenTable[] =
{
	/* Philips HLD1201 */
	HWTFT_FORMAT_PHILIPSHLD1201, (unsigned long)HWTFT_PhilipsHLD1201,

	/* LG-Philips LM151X2 */
	HWTFT_FORMAT_LGPHILIPSLM151X2, (unsigned long)HWTFT_LGPhilipsLM151X2,

	0xffffffff
};

/* ------------------------------------------------------------
// VARIABLE		: HWTFT_... 
// ------------------------------------------------------------
// DESCRIPTION	: TFT Philips Model HLD 1201 settings
// ------------------------------------------------------------*/
unsigned short HWTFT_PhilipsHLD1201[] =
{
   0x00, 800,	/* Input scan line active pixel count */
   0x01, 80,	/* Input scan line back porch delay -1 */
   0x02, 32,	/* Flat panel hsync width */
   0x03, 102,	/* Flat panel horizontal back porch -2 */
   0x04, 800,	/* Flat panel active pixel count */
   0x05, 1,		/* Flat panel minimum front porch -1 */
   0x06, 1,		/* Flat panel vsync width */
   0x07, 25,	/* Flat panel vertical back porch */
   0x08, 600,	/* Flat panel active line count */
   0x09, 0x0,	/* Set Flat panel interface control */
   0x0a, 0x202,	/* Set 10 KHz 50% duty cycle PWM */
   0x0c, 0x0,	/* Set blank pixel red/green/blue values */
   0x0d, 0x0,
   0x0e, 0x0,
   0x0f, 0x1f,	/* set polarity control */
   0xff
};

/* ------------------------------------------------------------
// VARIABLE		: HWTFT_... 
// ------------------------------------------------------------
// DESCRIPTION	: TFT LG-Philips Model LM151X2 settings
// ------------------------------------------------------------*/
unsigned short HWTFT_LGPhilipsLM151X2[] =
{
   0x00, 1024,	/* Input scan line active pixel count */
   0x01, 47,	/* Input scan line back porch delay -1 */
   0x02, 45,	/* Flat panel hsync width */
   0x03, 32,	/* Flat panel horizontal back porch -2 */
   0x04, 1024,	/* Flat panel active pixel count */
   0x05, 160,	/* Flat panel minimum front porch -1 */
   0x06, 44,	/* Flat panel vsync width */
   0x07, 1,		/* Flat panel vertical back porch */
   0x08, 768,	/* Flat panel active line count */
   0x09, 0x0,	/* Set Flat panel interface control */
   0x0a, 0x802,	/* Set 10 KHz 50% duty cycle PWM */
   0x0c, 0x0,	/* Set blank pixel red/green/blue values */
   0x0d, 0x0,
   0x0e, 0x0,
   0x0f, 0x1b,	/* set polarity control */
   0xff
};
