/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: trace.h
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libdbg\trace
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE:		: March, 1st 2000
// AUTHOR:		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Functions to manage debug trace functions
// ------------------------------------------------------------
// VARIABLES	: -
// ------------------------------------------------------------
// MACROS		: DBGTRACE_BEGINFUNCTION
//				  DBGTRACE_ENDFUNCTION
//				  DBGTRACE_LOOP
//				  DBGTRACE_BRANCH
//				  DBGTRACE_WARNING
//				  DBGTRACE_INFO
//				  DBGTRACE_NUMERICPARAM
//				  DBGTRACE_POINTERPARAM
//				  DBGTRACE_STRINGPARAM
//				  DBGTRACE_MEMORYIMAGEPARAM
//				  DBGTRACE_FRAMEBUFSURFACEPARAM
//				  DBGTRACE_SCREENINFOPARAM
// ------------------------------------------------------------
// FUNCTIONS	: -
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

#ifndef TRACE_H
#define TRACE_H

#include "osal.h"

#define DBGTRACE_WARNING_ENABLE

/*
#define DBGTRACE_BEGINFUNCTION_ENABLE
#define DBGTRACE_ENDFUNCTION_ENABLE
#define DBGTRACE_LOOP_ENABLE
#define DBGTRACE_BRANCH_ENABLE
#define DBGTRACE_WARNING_ENABLE
#define DBGTRACE_INFO_ENABLE
#define DBGTRACE_NUMERICPARAM_ENABLE
#define DBGTRACE_POINTERPARAM_ENABLE
#define DBGTRACE_STRINGPARAM_ENABLE
#define DBGTRACE_MEMORYIMAGEPARAM_ENABLE
#define DBGTRACE_FRAMEBUFSURFACEPARAM_ENABLE
#define DBGTRACE_SCREENINFOPARAM_ENABLE
*/
/* ------------------------------------------------------------
// DEFINE		: DBGTRACE_BEGINFUNCTION 
// ------------------------------------------------------------
// DESCRIPTION	: Trace the beginning of a function
// ------------------------------------------------------------
// name			: Name of the function
// ------------------------------------------------------------*/
#ifndef DBGTRACE_BEGINFUNCTION_ENABLE
#define DBGTRACE_BEGINFUNCTION(name)
#else
#define DBGTRACE_BEGINFUNCTION(name) { \
	OSAL_DisplayString("BEGIN : "); \
	OSAL_DisplayString(name); \
	OSAL_DisplayCR(); \
	}
#endif

/* ------------------------------------------------------------
// DEFINE		: DBGTRACE_ENDFUNCTION 
// ------------------------------------------------------------
// DESCRIPTION	: Trace the end of a function
// ------------------------------------------------------------
// name			: Name of the function
// ------------------------------------------------------------*/
#ifndef DBGTRACE_ENDFUNCTION_ENABLE
#define DBGTRACE_ENDFUNCTION(name)
#else
#define DBGTRACE_ENDFUNCTION(name) { \
	OSAL_DisplayString("END : "); \
	OSAL_DisplayString(name); \
	OSAL_DisplayCR(); \
	}
#endif

/* ------------------------------------------------------------
// DEFINE		: DBGTRACE_LOOP 
// ------------------------------------------------------------
// DESCRIPTION	: Trace a for or while loop
// ------------------------------------------------------------
// name			: Description of the loop
// ------------------------------------------------------------*/
#ifndef DBGTRACE_LOOP_ENABLE
#define DBGTRACE_LOOP(name)
#else
#define DBGTRACE_LOOP(name) { \
	OSAL_DisplayString("LOOP : "); \
	OSAL_DisplayString(name); \
	OSAL_DisplayCR(); \
	}
#endif

/* ------------------------------------------------------------
// DEFINE		: DBGTRACE_BRANCH 
// ------------------------------------------------------------
// DESCRIPTION	: Trace a branch (if, switch/case...)
// ------------------------------------------------------------
// name			: Description of the branch
// ------------------------------------------------------------*/
#ifndef DBGTRACE_BRANCH_ENABLE
#define DBGTRACE_BRANCH(name)
#else
#define DBGTRACE_BRANCH(name) { \
	OSAL_DisplayString("BRANCH : "); \
	OSAL_DisplayString(name); \
	OSAL_DisplayCR(); \
	}
#endif

/* ------------------------------------------------------------
// DEFINE		: DBGTRACE_WARNING 
// ------------------------------------------------------------
// DESCRIPTION	: Detect a problem (invalid parameter...)
// ------------------------------------------------------------
// name			: Description of the problem
// ------------------------------------------------------------*/
#ifndef DBGTRACE_WARNING_ENABLE
#define DBGTRACE_WARNING(name)
#else
#define DBGTRACE_WARNING(name) { \
	OSAL_DisplayString("WARNING : "); \
	OSAL_DisplayString(name); \
	OSAL_DisplayCR(); \
	}
#endif

/* ------------------------------------------------------------
// DEFINE		: DBGTRACE_INFO 
// ------------------------------------------------------------
// DESCRIPTION	: Display an information
// ------------------------------------------------------------
// name			: Information to display
// ------------------------------------------------------------*/
#ifndef DBGTRACE_INFO_ENABLE
#define DBGTRACE_INFO(name)
#else
#define DBGTRACE_INFO(name) { \
	OSAL_DisplayString("INFO : "); \
	OSAL_DisplayString(name); \
	OSAL_DisplayCR(); \
	}
#endif

/* ------------------------------------------------------------
// DEFINE		: DBGTRACE_NUMERICPARAM 
// ------------------------------------------------------------
// DESCRIPTION	: Display a numeric parameter
// ------------------------------------------------------------
// name			: Name of the variable
// ------------------------------------------------------------*/
#ifndef DBGTRACE_NUMERICPARAM_ENABLE
#define DBGTRACE_NUMERICPARAM(name)
#else
#define DBGTRACE_NUMERICPARAM(name) { \
	OSAL_DisplayString("  "#name" = 0x"); \
	OSAL_DisplayHexa((unsigned long)name); \
	OSAL_DisplayString(" ("); \
	OSAL_DisplayDecimal((unsigned long)name); \
	OSAL_DisplayString(")"); \
	OSAL_DisplayCR(); \
	}
#endif

/* ------------------------------------------------------------
// DEFINE		: DBGTRACE_POINTERPARAM 
// ------------------------------------------------------------
// DESCRIPTION	: Display a pointer value
// ------------------------------------------------------------
// name			: Name o the pointer variable
// ------------------------------------------------------------*/
#ifndef DBGTRACE_POINTERPARAM_ENABLE
#define DBGTRACE_POINTERPARAM(name)
#else
#define DBGTRACE_POINTERPARAM(name) { \
	OSAL_DisplayString("  "#name" = 0x"); \
	OSAL_DisplayHexa((unsigned long)name); \
	OSAL_DisplayCR(); \
	}
#endif

/* ------------------------------------------------------------
// DEFINE		: DBGTRACE_STRINGPARAM 
// ------------------------------------------------------------
// DESCRIPTION	: Display a string parameter
// ------------------------------------------------------------
// name			: Pointer on the string varaiable
// ------------------------------------------------------------*/
#ifndef DBGTRACE_STRINGPARAM_ENABLE
#define DBGTRACE_STRINGPARAM(name)
#else
#define DBGTRACE_STRINGPARAM(name) { \
	OSAL_DisplayString("  "#name" = "); \
	OSAL_DisplayString(name); \
	OSAL_DisplayCR(); \
	}
#endif

/* ------------------------------------------------------------
// DEFINE		: DBGTRACE_MEMORYIMAGEPARAM 
// ------------------------------------------------------------
// DESCRIPTION	: Display a SWBNP_MEMORY_IMAGE structure
// ------------------------------------------------------------
// name			: Pointer on the structure to display
// ------------------------------------------------------------*/
#ifndef DBGTRACE_MEMORYIMAGEPARAM_ENABLE
#define DBGTRACE_MEMORYIMAGEPARAM(name)
#else
#define DBGTRACE_MEMORYIMAGEPARAM(name) { \
	OSAL_DisplayString("  "#name" = { 0x"); \
	OSAL_DisplayHexa(name->ulMemHandle); \
	OSAL_DisplayString(", 0x"); \
	OSAL_DisplayHexa((unsigned long)name->pPtr); \
	OSAL_DisplayString(", "); \
	OSAL_DisplayDecimal(name->ulWidth); \
	OSAL_DisplayString(", "); \
	OSAL_DisplayDecimal(name->ulHeight); \
	OSAL_DisplayString(", "); \
	OSAL_DisplayDecimal(name->ulPitch); \
	OSAL_DisplayString(", "); \
	OSAL_DisplayDecimal(name->ulBpp); \
	OSAL_DisplayString(" }"); \
	OSAL_DisplayCR(); \
	}
#endif

/* ------------------------------------------------------------
// DEFINE		: DBGTRACE_FRAMEBUFSURFACEPARAM 
// ------------------------------------------------------------
// DESCRIPTION	: Display a HWFRAMEBUF_SURFACE structure
// ------------------------------------------------------------
// name			: Pointer on the structure to display
// ------------------------------------------------------------*/
#ifndef DBGTRACE_FRAMEBUFSURFACEPARAM_ENABLE
#define DBGTRACE_FRAMEBUFSURFACEPARAM(name)
#else
#define DBGTRACE_FRAMEBUFSURFACEPARAM(name) { \
	OSAL_DisplayString("  "#name" = { 0x"); \
	OSAL_DisplayHexa(name->ulBase); \
	OSAL_DisplayString(", "); \
	OSAL_DisplayDecimal(name->ulWidth); \
	OSAL_DisplayString(", "); \
	OSAL_DisplayDecimal(name->ulHeight); \
	OSAL_DisplayString(", "); \
	OSAL_DisplayDecimal(name->ulPitch); \
	OSAL_DisplayString(", "); \
	OSAL_DisplayDecimal(name->ulBpp); \
	OSAL_DisplayString(" }"); \
	OSAL_DisplayCR(); \
	}
#endif

/* ------------------------------------------------------------
// DEFINE		: DBGTRACE_SCREENINFOPARAM 
// ------------------------------------------------------------
// DESCRIPTION	: Display a HWCRTC_SCREENINFO structure
// ------------------------------------------------------------
// name			: Pointer on the structure to display
// ------------------------------------------------------------*/
#ifndef DBGTRACE_SCREENINFOPARAM_ENABLE
#define DBGTRACE_SCREENINFOPARAM(name)
#else
#define DBGTRACE_SCREENINFOPARAM(name) { \
	OSAL_DisplayString("  "#name" = { "); \
	OSAL_DisplayDecimal(name->ulOffsetX); \
	OSAL_DisplayString(", "); \
	OSAL_DisplayDecimal(name->ulOffsetY); \
	OSAL_DisplayString(", "); \
	OSAL_DisplayDecimal(name->ulScreenWidth); \
	OSAL_DisplayString(", "); \
	OSAL_DisplayDecimal(name->ulScreenHeight); \
	OSAL_DisplayString(" }"); \
	OSAL_DisplayCR(); \
	}
#endif

#endif
