;/* ------------------------------------------------------------
;// Copyright 2000 ST Microelectronics
;// ------------------------------------------------------------
;// This software is the property of ST Microelectronics which
;// specifically grants the user the right to modify, use and
;// distribute this software provided this notice is not removed
;// or altered. All other rights are reserved by ST.
;// ------------------------------------------------------------
;// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
;// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
;// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
;// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
;// SOFTWARE.
;// ------------------------------------------------------------
;// So that all may benefit from your experience, please report
;// any problems or suggestions about this software back to ST.
;// Please include your name, company, telephone number, email,
;// ST product requiring support and question or problem
;// encountered.
;// ------------------------------------------------------------
;// ST Microelectronics
;// Technoparc du pays de gex
;// 165 rue Edouard Branly bp112
;// 06637 St Genis Pouilly cedex
;// France
;// ------------------------------------------------------------
;// To contact us : http://www.st.com/stpc
;// ------------------------------------------------------------*/

;/* ------------------------------------------------------------
;// Copyright (C) 2000 STMicroelectronics
;// ------------------------------------------------------------
;// NVG/SICL/System Solution
;// ------------------------------------------------------------
;// FILE NAME	: cursasm.c
;// LIBRARY		: STPC Graphics Development Kit
;// MODULE		: libhard\cursor
;// PORTABILITY	: NASM under Linux OS
;// ------------------------------------------------------------
;// REVISION	: Rev 1.00
;// DATE:		: March, 1st 2000
;// AUTHOR:		: Thierry Vuillaume
;// ------------------------------------------------------------
;// DESCRIPTION	: Functions to manage the hardware cursor,
;//				  assembly language part
;// ------------------------------------------------------------
;// VARIABLES	: -
;// ------------------------------------------------------------
;// MACROS		: -
;// ------------------------------------------------------------
;// FUNCTIONS	: HWCURSOR_LoadAndMask
;//				  HWCURSOR_LoadXorMask
;//				  HWCURSOR_ShiftMask
;// ------------------------------------------------------------*/

;/* ------------------------------------------------------------
;// REVISION	: Rev 1.00
;// DATE		: March, 1st 2000
;// AUTHOR		: Thierry Vuillaume
;// ------------------------------------------------------------
;// MODIFICATIONS: Initial source code
;// ------------------------------------------------------------*/

GLOBAL HWCURSOR_LoadAndMask
GLOBAL HWCURSOR_LoadXorMask
GLOBAL HWCURSOR_ShiftMask

; void HWCURSOR_LoadAndMask(unsigned long pSrcMask, unsigned long pDstMask, unsigned long dwPitch, unsigned long dwWidth, unsigned long dwHeight)
HWCURSOR_LoadAndMask:
	push	ebp
	mov	ebp, esp

	;;; Load registers

	pushad
	push	ds

	mov	esi, [ebp + 8]		; ESI = Source mask
	mov	edi, [ebp + 12]		; EDI = Destination mask
	mov	ebx, [ebp + 16]		; EBX = Pitch
	mov	ecx, [ebp + 20]		; ECX = Width
	mov	edx, [ebp + 24]		; EDX = Height

	;;; Load registers end

	; 1 dword or 2 dword per line?
	cmp	ecx, 32
	ja	load_and_mask_sup32

	xor	ebp, ebp
	cmp	ecx, 32
	je	load_and_mask_ret

	mov	ebp, 0ffffffffh
	shr	ebp, cl
	bswap	ebp

load_and_mask_ret:
	mov	eax, [esi]		; And mask 
	or	eax, ebp
	mov	[edi], eax
	mov	dword [edi + 4], 0ffffffffh

	add	edi, 8			; next line
	add	esi, ebx

	dec	edx
	jnz	load_and_mask_ret	

	jmp	load_and_mask_end

load_and_mask_sup32:
	xor	ebp, ebp
	cmp	ecx, 64
	je	load_and_mask_sup32_ret

	mov	ebp, 0ffffffffh
	sub	ecx, 32
	shr	ebp, cl
	bswap	ebp

load_and_mask_sup32_ret:
	mov	eax, [esi]		; And mask 
	mov	[edi], eax
	mov	eax, [esi + 4]		
	or	eax, ebp
	mov	[edi + 4], eax

	add	edi, 8
	add	esi, ebx

	dec	edx
	jnz	load_and_mask_sup32_ret	

load_and_mask_end:
	pop	ds
	popad
	pop	ebp
	ret

; void HWCURSOR_LoadXorMask(unsigned long pSrcMask, unsigned long pDstMask, unsigned long dwPitch, unsigned long dwWidth, unsigned long dwHeight)
HWCURSOR_LoadXorMask:
	push	ebp
	mov	ebp, esp

	;;; Load registers

	pushad
	push	ds

	mov	esi, [ebp + 8]		; ESI = Source mask
	mov	edi, [ebp + 12]		; EDI = Destination mask
	mov	ebx, [ebp + 16]		; EBX = Pitch
	mov	ecx, [ebp + 20]		; ECX = Width
	mov	edx, [ebp + 24]		; EDX = Height

	;;; Load registers end

	; 1 dword or 2 dword per line?
	cmp	ecx, 32
	ja	load_or_mask_sup32

	mov	ebp, 0ffffffffh
	cmp	ecx, 32
	je	load_or_mask_ret

	shr	ebp, cl
	bswap	ebp
	not	ebp

load_or_mask_ret:
	mov	eax, [esi]		; Xor mask 
	and	eax, ebp
	mov	[edi + 512], eax
	mov	dword [edi + 516], 0

	add	edi, 8
	add	esi, ebx

	dec	edx
	jnz	load_or_mask_ret	

	jmp	load_or_mask_end

load_or_mask_sup32:
	mov	ebp, 0ffffffffh
	cmp	ecx, 64
	je	load_or_mask_sup32_ret

	sub	ecx, 32
	shr	ebp, cl
	bswap	ebp
	not	ebp

load_or_mask_sup32_ret:
	mov	eax, [esi]		; Xor mask 
	mov	[edi + 512], eax
	mov	eax, [esi + 4]		
	and	eax, ebp
	mov	[edi + 516], eax

	add	edi, 8
	add	esi, ebx

	dec	edx
	jnz	load_or_mask_sup32_ret	

load_or_mask_end:
	pop	ds
	popad
	pop	ebp
	ret

; void HWCURSOR_ShiftMask(unsigned long pSrcMask, unsigned long pDstMask, unsigned long dwHeight, unsigned long dwXShift, unsigned long dwYShift)
HWCURSOR_ShiftMask:
	push	ebp
	mov	ebp, esp

	;;; Load registers

	pushad
	push	ds

	mov	eax, [ebp + 24]		; Y shift
	mov ebx, [ebp + 20]		; X shift
	mov	edx, [ebp + 16]		; EDX = Height
	mov	esi, [ebp + 8]		; ESI = Source mask
	mov	edi, [ebp + 12]		; EDI = Destination mask

	;;; Load registers end

	sub	edx, eax		; y shift
	shl	eax, 3
	add	esi, eax

	sub	esp, 8			; book 4 dword in stack

	cmp	ebx, 32
	jbe	shift_mask_inf32
	jmp	shift_mask_sup32

shift_mask_inf32:
	cmp	ebx, 32
	jne	shift_mask_not32
	jmp	shift_mask_32

shift_mask_not32:
	cmp	ebx, 0
	je	shift_mask_y

	; Prepare mask for unused bits
	mov	eax, 0ffffffffh
	mov	ecx, 32
	sub	ecx, ebx
	shr	eax, cl

	mov	dword [esp], eax	; Second dword unused bits
	not	eax
	mov	dword [esp + 4], eax	; Second dword unused bits inverted

	mov	ecx, ebx

shift_mask_ret:

	; And mask
	mov eax, [esi]
	mov ebx, [esi + 4]	; load one mask line

	bswap eax
	bswap ebx			; compensate little indian effect

	shld eax, ebx, cl
	shl ebx, cl	; shift mask

	or ebx, [esp]	; Set to 1 unused bits of and mask

	bswap eax
	bswap ebx			; compensate little indian effect

	mov [edi], eax
	mov [edi + 4], ebx

	; Xor mask
	mov eax, [esi + 512]
	mov ebx, [esi + 516]; load one mask line

	bswap eax
	bswap ebx			; compensate little indian effect

	shld eax, ebx, cl
	shld ebx, esi, cl	; Shift the line

	and ebx, [esp + 4]	; Set to 0 unused bits

	bswap eax
	bswap ebx			; compensate little indian effect

	mov [edi + 512], eax
	mov [edi + 516], ebx

	add	esi, 8
	add	edi, 8
		
	dec	edx
	jnz	shift_mask_ret

	jmp shift_mask_end

shift_mask_y:
	mov	ecx, ebx

shift_mask_y_ret:

	; And mask
	mov eax, [esi]
	mov ebx, [esi + 4]	; load one mask line
	mov [edi], eax
	mov [edi + 4], ebx

	; Xor mask
	mov eax, [esi + 512]
	mov ebx, [esi + 516]; load one mask line
	mov [edi + 512], eax
	mov [edi + 516], ebx

	add	esi, 8
	add	edi, 8

	dec	edx
	jnz	shift_mask_y_ret

	jmp shift_mask_end

shift_mask_32:
	mov	ecx, ebx

shift_mask_32_ret:

	; And mask
	mov eax, [esi + 4]	; load one mask line
	mov [edi], eax
	mov dword [edi + 4], 0ffffffffh

	; Xor mask
	mov eax, [esi + 516]; load one mask line
	mov [edi + 512], eax
	mov dword [edi + 516], 0

	add	esi, 8
	add	edi, 8

	dec	edx
	jnz	shift_mask_32_ret

	jmp shift_mask_end

shift_mask_sup32:
	mov	eax, 0ffffffffh
	mov	ecx, 64
	sub	ecx, ebx
	shr	eax, cl

	mov	dword [esp], eax		; First dword unused bits
	not	eax
	mov	dword [esp + 4], eax		; First dword unused bits inverted

	mov	ecx, ebx
	sub	ecx, 32

shift_mask_sup32_ret:

	; And mask
	mov eax, [esi + 4]	; load one mask line

	bswap eax			; compensate little indian effect
	shl eax, cl
	or eax, [esp]		; Set to 1 unused bits of and mask
	bswap eax			; compensate little indian effect

	mov [edi], eax
	mov [edi + 4], dword 0ffffffffh

	; Xor mask
	mov eax, [esi + 516]; load one mask line

	bswap eax			; compensate little indian effect
	shl eax, cl
	and eax, [esp + 4]	; Set to 0 unused bits
	bswap eax			; compensate little indian effect

	mov [edi + 512], eax
	mov dword [edi + 516], 0

	add	esi, 8
	add	edi, 8
		
	dec	edx
	jnz	shift_mask_sup32_ret

shift_mask_end:
	add	esp, 8

	pop	ds
	popad
	pop	ebp
	ret
