;/* ------------------------------------------------------------
;// Copyright 2000 ST Microelectronics
;// ------------------------------------------------------------
;// This software is the property of ST Microelectronics which
;// specifically grants the user the right to modify, use and
;// distribute this software provided this notice is not removed
;// or altered. All other rights are reserved by ST.
;// ------------------------------------------------------------
;// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
;// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
;// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
;// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
;// SOFTWARE.
;// ------------------------------------------------------------
;// So that all may benefit from your experience, please report
;// any problems or suggestions about this software back to ST.
;// Please include your name, company, telephone number, email,
;// ST product requiring support and question or problem
;// encountered.
;// ------------------------------------------------------------
;// ST Microelectronics
;// Technoparc du pays de gex
;// 165 rue Edouard Branly bp112
;// 06637 St Genis Pouilly cedex
;// France
;// ------------------------------------------------------------
;// To contact us : http://www.st.com/stpc
;// ------------------------------------------------------------*/

;/* ------------------------------------------------------------
;// Copyright (C) 2000 STMicroelectronics
;// ------------------------------------------------------------
;// NVG/SICL/System Solution
;// ------------------------------------------------------------
;// FILE NAME	: geasm.asm
;// LIBRARY		: STPC Graphics Development Kit
;// MODULE		: libhard\ge
;// PORTABILITY	: NASM under Linux OS
;// ------------------------------------------------------------
;// REVISION	: Rev 1.00
;// DATE:		: March, 1st 2000
;// AUTHOR:		: Thierry Vuillaume
;// ------------------------------------------------------------
;// DESCRIPTION	: Functions to manage graphic 2D acceleration,
;//				  assembly language part
;// ------------------------------------------------------------
;// VARIABLES	: -
;// ------------------------------------------------------------
;// MACROS		: -
;// ------------------------------------------------------------
;// FUNCTIONS	: HWGE_SendDataToDataPort
;//				  HWGE_SendMonoDataToDataPort
;// ------------------------------------------------------------*/

;/* ------------------------------------------------------------
;// REVISION	: Rev 1.00
;// DATE		: March, 1st 2000
;// AUTHOR		: Thierry Vuillaume
;// ------------------------------------------------------------
;// MODIFICATIONS: Initial source code
;// ------------------------------------------------------------*/

GLOBAL HWGE_SendPackedData
GLOBAL HWGE_SendDataToDataPort
GLOBAL HWGE_SendMonoDataToDataPort

; void HWGE_SendPackedData(char far *pSrc, char far *pDst, unsigned long ulSizeInDword)
HWGE_SendPackedData:
	push	ebp
	mov	ebp, esp
	push	esi
	push	edi
	push	ecx

	;;; Load registers
	mov	esi, [ebp + 8]
	mov	edi, [ebp + 12]
	mov	ecx, [ebp + 16]
	cld
	rep	movsd

	pop	ecx
	pop	edi
	pop	esi
	pop	ebp
	ret	

; void HWGE_SendDataToDataPort(unsigned long ulDstXY, char far *pData, unsigned long ulPitch, unsigned long ulGERegister, unsigned long ulDataPort)
HWGE_SendDataToDataPort:
	push	ebp
	mov	ebp, esp

	;;; Load registers

	pushad
	push	ds
	push	es

	mov	ax, ds
	mov	es, ax

	mov	ebx, [ebp + 24]		; EBX = Base address of the data port
	mov	edi, [ebp + 20]		; EDI = Base pointer on GE registers
	mov	esi, [ebp + 12]		; ESI = Pointer on source data
	mov	edx, [ebp + 16]		; EDX = Pitch
	mov	ebp, [ebp + 8]		; EBP = Dst XY register
	; !! Starting from here ebp modified, no more access to stack

	;;; Load registers end

	sub	esp, 20

	; Save data port address
	mov	[esp], ebx

	; load width and height and check validity

	mov	eax, [edi + 048h] 		; Height register
	add	eax, 1
	jnz	sdtdp_continue1				; Nothing to transfer
	jmp	sdtdp_end

sdtdp_continue1:
	mov	ebx, [edi + 0c8h] 		; Width register
	add	ebx, 1
	jnz	sdtdp_continue2				; Nothing to transfer
	jmp	sdtdp_end

sdtdp_continue2:
	; Analyse the best way to transfer

;	cmp	edx, ebx
;	je	sdtdp_packed			; Pitch = Width <=> packed mode

	test	edx, 3
	jz	sdtdp_aligned		; Pitch is not 32 bits aligned
	jmp	sdtdp_notaligned

sdtdp_aligned:
	mov	ecx, esi
	and	ecx, 3
	add	ebx, ecx
	mov	[edi + 0bch], ecx
	and	esi, 0fffffffch			; Adjust source pointer to be 32 bits aligned

	add	ebx, 3
	shr	ebx, 2				; Convert width from byte to dword

	mov	ecx, ebx
	shl	ecx, 2				; Width in byte
	sub	edx, ecx
	jz	sdtdp_bloc			; Pitch = Width <=> no jump between two lines

	;
	; FIRST CASE : Pitch 32 bits aligned
	;

	mov	[edi + 10000h], ebp		; Start bitblit

	mov	edi, [esp]		; Data port address
	mov	ecx, ebx
	cld

sdtdp_return:
	rep     movsd
	add	esi, edx			; jump to next line
	mov	ecx, ebx			; load width
	dec	eax
	jnz	sdtdp_return

	jmp sdtdp_end

	;
	; SECOND CASE : Copy in one block (pitch = width)
	;

sdtdp_bloc:

	mov	[edi + 10000h], ebp		; Start bitblit

	mul	ebx				; size of the bloc to copy
	mov	ecx, eax
	cld
	mov	edi, [esp]		; Data port address
	rep	movsd

	jmp sdtdp_end

	;
	; THIRD CASE : Packed mode
	;

sdtdp_packed:
	mov	edx, edi		; Now, pointer on registers = EDX
	mov	edi, [edx + 0ach]	; Save old value and put in packed mode
	mov     [edx + 0ach], dword 1

	mov	ecx, esi
	and	ecx, 3
	add	ebx, ecx
	mov	[edx + 0bch], ecx
	and	esi, 0fffffffch		; Adjust source pointer to be 32 bits aligned

	mov	[edx + 10000h], ebp	; Start bitblit
	mov	ebp, edi		; Now EBP is free

	mul	ebx			; size of the bloc to copy
	mov	ecx, eax
	add	ecx, 3
	shr	ecx, 2			; In dword
	cld
	mov	edi, [esp]		; Data port address
	rep	movsd

	mov	[edx + 0ach], ebp

	jmp sdtdp_end

	;
	; FOURTH CASE : Pitch is not multiple of 32 bits
	;

sdtdp_notaligned:
	mov	[esp + 4], ebx		; store width in the stack

	mov	ebx, edi
	mov	edi, [esp]

	mov	ecx, edx
	and	ecx, 0fffffffch
	mov	[esp + 8], ecx		; store 32 bits aligned part of the pitch
	and	edx, 3
	mov	[esp + 12], edx		; store rest of pitch

	mov	edx, esi
	and	edx, 3			; edx = reste

	and	esi, 0fffffffch		; esi = 32 bits aligned

sdtdp_notaligned_loop:
	mov	[ebx + 0bch], edx	; number of pixel to drop
	mov	[ebx + 18000h], ebp	; next line

	mov	ecx, [esp + 4]		; load width
	add	ecx, edx
	add	ecx, 3
	shr	ecx, 2			; ecx = number of 32 bits to transfer


	mov	[esp + 16], esi		;
	rep     movsd
	mov	esi, [esp + 16]		;

	add	esi, [esp + 8]		; add 32 bits aligned part of the pitch
	add	edx, [esp + 12]		; add pitch rest to the rest

	add	esi, edx		; rest reach 32 bits limit ?
	and	esi, 0fffffffch

	and	edx, 3			; rest modulo 32 bits

	add	ebp, 10000h		; next value of Y coordinate

	dec	eax
	jnz	sdtdp_notaligned_loop

sdtdp_end:
	add	esp, 20

	pop	es
	pop	ds
	popad
	pop	ebp
	ret

; void HWGE_SendMonoDataToDataPort(unsigned long ulDstXY, char far *pData, unsigned long ulOffsetInBits, unsigned long ulPitchInBits, unsigned long ulGERegister, unsigned long ulDataPort)
HWGE_SendMonoDataToDataPort:
	push	ebp
	mov	ebp, esp

	;;; Load registers

	pushad
	push	ds
	push	es

	mov	ax, ds
	mov	es, ax

	mov	ecx, [ebp + 32]		; ECX = Base address of the data port
	mov	edi, [ebp + 28]		; EDI = Base pointer on GE registers
	mov	esi, [ebp + 12]		; ESI = Pointer on source data
	mov	eax, [ebp + 16]     ; EAX = Offset from pointer in bits
	mov	ebx, [ebp + 24]     ; EBX = Width in bits
	mov	edx, [ebp + 20]		; EDX = Pitch
	mov	ebp, [ebp + 8]		; EBP = Dst XY register
	; !! Starting from here ebp modified, no more access to stack
	; !! stored value

	;;; Load registers end

	sub	esp, 24

	; Save data port address
	mov	[esp], ecx

	; transfer 8 bits aligned part of offset in base pointer (ESI)
	mov	ecx, eax
	sar	ecx, 3
	add	esi, ecx
	and	eax, 7
	mov	[esp + 4], eax

	; load width and height and check validity
	mov	eax, [edi + 048h] 		; Height register
	add	eax, 1
	jnz	smdtdp_continue1			; Nothing to transfer
	jmp	smdtdp_end

smdtdp_continue1:
	and	ebx, ebx
	jnz	smdtdp_continue2			; Nothing to transfer
	jmp	smdtdp_end

smdtdp_continue2:
	; Analyse the best way to transfer

	; Negative pitch <=> use standard algorithm anyway
	test	edx, 80000000h
	jnz	smdtdp_aligned

;	cmp	edx, ebx
;	je	smdtdp_packed			; Pitch = Width <=> packed mode

	test	edx, 1fh
	jz	smdtdp_aligned		; Pitch is not 32 bits aligned
	jmp	smdtdp_notaligned

smdtdp_aligned:
	sar	edx, 3				; Pitch is now in bytes

	mov	ecx, esi
	and	ecx, 3
	shl	ecx, 3
	add	ecx, [esp + 4]
	add	ebx, ecx
	mov	[edi + 0bch], ecx
	and	esi, 0fffffffch			; Adjust source pointer to be 32 bits aligned

	add	ebx, 1fh
	shr	ebx, 5				; Convert width from bits to dword

	mov	ecx, ebx
	shl	ecx, 2				; Width in byte
	sub	edx, ecx
	jz	smdtdp_bloc			; Pitch = Width <=> no jump between two lines

	;
	; FIRST CASE : Pitch 32 bits aligned
	;

	mov	[edi + 10000h], ebp		; Start bitblit

	mov	edi, [esp]			; Data port address
	mov	ecx, ebx
	cld

smdtdp_return:
	rep     movsd
	add	esi, edx			; jump to next line
	mov	ecx, ebx			; load width
	dec	eax
	jnz	smdtdp_return

	jmp smdtdp_end

	;
	; SECOND CASE : Copy in one block (pitch = width)
	;

smdtdp_bloc:

	mov	[edi + 10000h], ebp		; Start bitblit

	mul	ebx				; size of the bloc to copy
	mov	ecx, eax
	cld
	mov	edi, [esp]		; Data port address
	rep	movsd

	jmp smdtdp_end

	;
	; THIRD CASE : Packed mode
	;

smdtdp_packed:
	mov	edx, edi		; Now, pointer on registers = EDX
	mov	edi, [edx + 08ch]	; Save old value and put in packed mode
	or	[edx + 08ch], dword 800h

	mov	ecx, esi
	and	ecx, 3
	shl	ecx, 3
	add	ecx, [esp + 4]
	add	ebx, ecx
	mov	[edx + 0bch], ecx
	and	esi, 0fffffffch		; Adjust source pointer to be 32 bits aligned

	mov	[edx + 10000h], ebp	; Start bitblit
	mov	ebp, edi		; Now EBP is free

	mul	ebx			; size of the bloc to copy
	mov	ecx, eax
	add	ecx, 1fh
	shr	ecx, 5			; In dword
	cld
	mov	edi, [esp]		; Data port address
	rep	movsd

	mov	[edx + 08ch], ebp

	jmp smdtdp_end

	;
	; FOURTH CASE : Pitch is not multiple of 32 bits
	;

smdtdp_notaligned:
	mov	[esp + 8], ebx		; store width in the stack

	mov	ebx, edi
	mov	edi, [esp]

	mov	ecx, edx
	and	ecx, 0ffffffe0h
	mov	[esp + 12], ecx		; store 32 bits aligned part of the pitch
	and	edx, 1fh
	mov	[esp + 16], edx		; store rest of pitch

	mov	edx, esi
	and	edx, 3			; edx = reste
	shl	edx, 3
	add     edx, [esp + 4]

	and	esi, 0fffffffch		; esi = 32 bits aligned

smdtdp_notaligned_loop:
	mov	[ebx + 0bch], edx	; number of pixel to drop
	mov	[ebx + 18000h], ebp	; next line

	mov	ecx, [esp + 8]		; load width
	add	ecx, edx
	add	ecx, 1fh
	shr	ecx, 5			; ecx = number of 32 bits to transfer


	mov	[esp + 20], esi		;
	rep     movsd
	mov	esi, [esp + 20]		;

	add	esi, [esp + 12]		; add 32 bits aligned part of the pitch
	add	edx, [esp + 16]		; add pitch rest to the rest

	mov	ecx, edx
	shr	ecx, 3
	add	esi, ecx		; rest reach 32 bits limit ?
	and	esi, 0fffffffch

	and	edx, 1fh		; rest modulo 32 bits

	add	ebp, 10000h		; next value of Y coordinate

	dec	eax
	jnz	smdtdp_notaligned_loop

smdtdp_end:
	add	esp, 24

	pop	es
	pop	ds
	popad
	pop	ebp
	ret
