/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: vip.c
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libhard\vip
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.02
// DATE:		: May 29, 2001
// AUTHOR:		: joel Gouiller
// ------------------------------------------------------------
// DESCRIPTION	: Functions to manage the Video Input Port
// ------------------------------------------------------------
// VARIABLES	: -
// ------------------------------------------------------------
// MACROS		: -
// ------------------------------------------------------------
// FUNCTIONS	: HWVIP_SetSource
//				  HWVIP_GetSourceSize
//				  HWVIP_SetDestination
//				  HWVIP_Enable
//				  HWVIP_Disable
//				  HWVIP_UrgentStop
//				  HWVIP_IsField0
//				  HWVIP_WaitEndField0
//				  HWVIP_WaitEndField1
//				  HWVIP_FlipOdd
//				  HWVIP_FlipEven
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.01
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/
/* ------------------------------------------------------------
// REVISION		: Rev 1.02
// DATE			: May 29, 2001
// AUTHOR		: joel Gouiller
// ------------------------------------------------------------
// MODIFICATIONS: Change setting for CCIR 601 for to run 
//					VPX encoder of micronas
// ------------------------------------------------------------*/

#include "trace.h"
#include "vip.h"

/* ------------------------------------------------------------
// FUNCTION		: HWVIP_GetSourceSize
// ------------------------------------------------------------
// DESCRIPTION	: Get the size of the input datas
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulSourceType	: video source type : HWVIP_FORMAT_...
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ulWidth		: Number of pixels per lines
// ulHeight		: Number lines
// -----------------------------------------------------s-------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVIP_GetSourceSize(unsigned long ulSourceType, unsigned long *ulWidth, unsigned long *ulHeight)
{
	DBGTRACE_BEGINFUNCTION("HWVIP_GetSourceSize");
	DBGTRACE_NUMERICPARAM(ulSourceType);

	/* Size depend on format */
	switch(ulSourceType & 0xffff)
	{
		case HWVIP_FORMAT_NTSC:
		case HWVIP_FORMAT_NTSCM:
			DBGTRACE_BRANCH("NTSC");
			*ulWidth = 720;
			*ulHeight = 480;
			break;

		case HWVIP_FORMAT_SECAM:
		case HWVIP_FORMAT_PAL:
		case HWVIP_FORMAT_PALBGHI:
			DBGTRACE_BRANCH("PAL/SECAM");
			*ulWidth = 720;
			*ulHeight = 576;
			break;

		default:
			DBGTRACE_WARNING("Unknown format");
	}

	DBGTRACE_ENDFUNCTION("HWVIP_GetSourceSize");
	DBGTRACE_NUMERICPARAM(*ulWidth);
	DBGTRACE_NUMERICPARAM(*ulHeight);
}

/* ------------------------------------------------------------
// FUNCTION		: HWVIP_SetSource
// ------------------------------------------------------------
// DESCRIPTION	: Set the type of video data
//				  This function initialize the video only in
//				  CCIR656 mode
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulSourceType	: video source type : HWVIP_FORMAT_...
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// -----------------------------------------------------s-------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVIP_SetSource(unsigned long ulSourceType)
{
	DBGTRACE_BEGINFUNCTION("HWVIP_SetSource");
	DBGTRACE_NUMERICPARAM(ulSourceType);

	/* Initialize the horizontal and vertical timings */
	if(ulSourceType & HWVIP_FORMAT_CCIR601)
	{
		switch(ulSourceType & 0xffff)
		{
			case HWVIP_FORMAT_NTSC:
			case HWVIP_FORMAT_NTSCM:
				DBGTRACE_BRANCH("NTSC");
				REGHOST_SET_VIP_HORIZONTALTIMINGS_REG(0x1018bb33L);
				REGHOST_SET_VIP_VERTICALTIMINGS_REG(0x04c81906L);
				break;
	
			case HWVIP_FORMAT_SECAM:
			case HWVIP_FORMAT_PAL:
			case HWVIP_FORMAT_PALBGHI:
				DBGTRACE_BRANCH("PAL/SECAM");
				REGHOST_SET_VIP_HORIZONTALTIMINGS_REG(0x1018bb33L);
				REGHOST_SET_VIP_VERTICALTIMINGS_REG(0x4c81938L);
				break;
		}
	
		/* Initialize the timing generator in CCIR601 */
		REGHOST_SET_VIP_TIMINGS1_REG(0x802110a0L);
		REGHOST_SET_VIP_TIMINGS2_REG(0x00008b7bL);

	}
	else
	{
		switch(ulSourceType & 0xffff)
		{
			case HWVIP_FORMAT_NTSC:
			case HWVIP_FORMAT_NTSCM:
				DBGTRACE_BRANCH("NTSC");
				REGHOST_SET_VIP_HORIZONTALTIMINGS_REG(0x269ffbffL);
				REGHOST_SET_VIP_VERTICALTIMINGS_REG(0x048ffbffL);
				break;
	
			case HWVIP_FORMAT_SECAM:
				DBGTRACE_BRANCH("SECAM");
				REGHOST_SET_VIP_HORIZONTALTIMINGS_REG(0x23BffbffL);
				REGHOST_SET_VIP_VERTICALTIMINGS_REG(0x060ffbffL);
				break;
	
			case HWVIP_FORMAT_PAL:
			case HWVIP_FORMAT_PALBGHI:
				DBGTRACE_BRANCH("PAL");
				REGHOST_SET_VIP_HORIZONTALTIMINGS_REG(0x23BffbffL);
				REGHOST_SET_VIP_VERTICALTIMINGS_REG(0x061ffbffL);
				break;
		}
	
		/* Initialize the timing generator in CCIR656 */
		REGHOST_SET_VIP_TIMINGS1_REG(0x804110a1L);
		REGHOST_SET_VIP_TIMINGS2_REG(0x00008b7bL);
	}

	DBGTRACE_ENDFUNCTION("HWVIP_SetSource");
}

/* ------------------------------------------------------------
// FUNCTION		: HWVIP_SetDestination
// ------------------------------------------------------------
// DESCRIPTION	: Set the location of captured video in the
//				  frame buffer
//
//				  To set a full size video :
//				  ulPitch = 2 * pitch of video
//				  ulOdd = location of video in frame buffer
//				  ulEven = location of video in frame buffer + 
//				  width of video
//				  bDecimator = 0
//
//				  To set a quarter size video :
//				  ulPitch = pitch of video
//				  ulOdd = location of video in frame buffer
//				  ulEven = HWVIP_NOEVEN
//				  bDecimator = 1
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulOdd		: offset of odd frame in frame buffer, 
//				  HWVIP_NOODD <=> odd frame disabled
// ulEven		: offset of even frame in frame buffer, 
//				  HWVIP_NOEVEN <=> even frame disabled
// ulPitch		: Number of bytes between two lines of a frame
// ulDecimator	: TRUE to divide by two the width of the video
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVIP_SetDestination(unsigned long ulOdd, unsigned long ulEven, unsigned long ulPitch, unsigned long ulDecimator)
{
	unsigned long ulConfigReg;

	DBGTRACE_BEGINFUNCTION("HWVIP_SetDestination");
	DBGTRACE_NUMERICPARAM(ulOdd);
	DBGTRACE_NUMERICPARAM(ulEven);
	DBGTRACE_NUMERICPARAM(ulPitch);
	DBGTRACE_NUMERICPARAM(ulDecimator);

	/* CCIR656 video */
	ulConfigReg = 0x0008000eL;

	/* Need to enable odd frame? */ 
	if(ulOdd != HWVIP_NOODD)
	{
		ulConfigReg |= 0x80;
	}

	/* Need to enable even frame? */ 
	if(ulEven != HWVIP_NOEVEN)
	{
		ulConfigReg |= 0x100;
	}

	/* If both odd and even frame are displayed, enable */
	/* use of 2 buffer */
	if((ulConfigReg & 0x180) == 0x180)
	{
		ulConfigReg |= 0x40;
	}

	/* Enable decimator? */
	if(ulDecimator)
	{
		ulConfigReg |= 0x100000L;
	}

	/* Initialize Video Input Port */
	REGHOST_SET_VIP_CONFIG_REG(ulConfigReg);
	REGHOST_SET_VIP_ODDBASE_REG(ulOdd);
	REGHOST_SET_VIP_EVENBASE_REG(ulEven);
	REGHOST_SET_VIP_PITCH_REG(ulPitch);

	DBGTRACE_ENDFUNCTION("HWVIP_SetDestination");
}

/* ------------------------------------------------------------
// FUNCTION		: HWVIP_Enable
// ------------------------------------------------------------
// DESCRIPTION	: Start the Video Input Port
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVIP_Enable(void)
{
	unsigned long ulConfigReg;

	DBGTRACE_BEGINFUNCTION("HWVIP_Enable");

	ulConfigReg = REGHOST_GET_VIP_CONFIG_REG();

	/* Set clock on GCLK before make change in config register */
	ulConfigReg &= ~0xc00000L;
	REGHOST_SET_VIP_CONFIG_REG(ulConfigReg);

	/* Set enable bit with GCLK for VCLK */
	ulConfigReg |= 0x11;
	REGHOST_SET_VIP_CONFIG_REG(ulConfigReg);

	/* Enable external clock */
	ulConfigReg |= 0x400000L;
	REGHOST_SET_VIP_CONFIG_REG(ulConfigReg);

	DBGTRACE_ENDFUNCTION("HWVIP_Enable");
}

/* ------------------------------------------------------------
// FUNCTION		: HWVIP_Disable
// ------------------------------------------------------------
// DESCRIPTION	: Stop the Video Input Port at end of the frame
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVIP_Disable(void)
{
	DBGTRACE_BEGINFUNCTION("HWVIP_Disable");

	REGHOST_SET_VIP_CONFIG_REG(REGHOST_GET_VIP_CONFIG_REG() & ~0x10);

	DBGTRACE_ENDFUNCTION("HWVIP_Disable");
}

/* ------------------------------------------------------------
// FUNCTION		: HWVIP_UrgentStop
// ------------------------------------------------------------
// DESCRIPTION	: Stop the Video Input Port without wait for
//				  the end of the frame
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVIP_UrgentStop(void)
{
	DBGTRACE_BEGINFUNCTION("HWVIP_UrgentStop");

	REGHOST_SET_VIP_CONFIG_REG(REGHOST_GET_VIP_CONFIG_REG() & ~0x11);

	DBGTRACE_ENDFUNCTION("HWVIP_UrgentStop");
}

/* ------------------------------------------------------------
// FUNCTION		: HWVIP_IsField0
// ------------------------------------------------------------
// DESCRIPTION	: Field 0 is currently processed
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: <> 0 if field 0 in progress
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWVIP_IsField0(void)
{
	return REGHOST_GET_VIP_STATUS_REG() & 0x8; 
}

/* ------------------------------------------------------------
// FUNCTION		: HWVIP_WaitEndField0
// ------------------------------------------------------------
// DESCRIPTION	: Wait for the end of the odd frame
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVIP_WaitEndField0(void)
{
	DBGTRACE_BEGINFUNCTION("HWVIP_WaitEndField0");

	while(HWVIP_IsField0());
	while(!HWVIP_IsField0());

	DBGTRACE_ENDFUNCTION("HWVIP_WaitEndField0");
}

/* ------------------------------------------------------------
// FUNCTION		: HWVIP_WaitEndField1
// ------------------------------------------------------------
// DESCRIPTION	: Wait for the end of the even frame
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVIP_WaitEndField1(void)
{
	DBGTRACE_BEGINFUNCTION("HWVIP_WaitEndField1");

	while(!HWVIP_IsField0());
	while(HWVIP_IsField0());

	DBGTRACE_ENDFUNCTION("HWVIP_WaitEndField1");
}

/* ------------------------------------------------------------
// FUNCTION		: HWVIP_FlipOdd
// ------------------------------------------------------------
// DESCRIPTION	: Change the location of odd frame in frame
//				  buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulOdd		: Offset of odd frame in frame buffer
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVIP_FlipOdd(unsigned long ulOdd)
{
	DBGTRACE_BEGINFUNCTION("HWVIP_FlipOdd");
	DBGTRACE_NUMERICPARAM(ulOdd);

	REGHOST_SET_VIP_ODDBASE_REG(ulOdd);

	DBGTRACE_ENDFUNCTION("HWVIP_FlipOdd");
}

/* ------------------------------------------------------------
// FUNCTION		: HWVIP_FlipEven
// ------------------------------------------------------------
// DESCRIPTION	: Change the location of even frame in frame
//				  buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulOdd		: Offset of even frame in frame buffer
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVIP_FlipEven(unsigned long ulEven)
{
	DBGTRACE_BEGINFUNCTION("HWVIP_FlipEven");
	DBGTRACE_NUMERICPARAM(ulEven);

	REGHOST_SET_VIP_EVENBASE_REG(ulEven);

	DBGTRACE_ENDFUNCTION("HWVIP_FlipEven");
}
