/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: vip.h
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libhard\vip
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE:		: March, 1st 2000
// AUTHOR:		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Functions to manage the Video Input Port
// ------------------------------------------------------------
// VARIABLES	: -
// ------------------------------------------------------------
// MACROS		: HWVIP_FORMAT_...
//				  HWVIP_NOODD
//				  HWVIP_NOEVEN
// ------------------------------------------------------------
// FUNCTIONS	: HWVIP_SetSource
//				  HWVIP_GetSourceSize
//				  HWVIP_SetDestination
//				  HWVIP_Enable
//				  HWVIP_Disable
//				  HWVIP_UrgentStop
//				  HWVIP_WaitEndField0
//				  HWVIP_WaitEndField1
//				  HWVIP_FlipOdd
//				  HWVIP_FlipEven
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

#ifndef __VIP_H
#define __VIP_H

#include "host.h"

/* ------------------------------------------------------------
// DEFINE		: HWVIP_FORMAT_... 
// ------------------------------------------------------------
// DESCRIPTION	: Video input format
// ------------------------------------------------------------*/
#define	HWVIP_FORMAT_NTSC		0
#define	HWVIP_FORMAT_NTSCM		1
#define	HWVIP_FORMAT_SECAM		2
#define	HWVIP_FORMAT_PAL		3
#define	HWVIP_FORMAT_PALBGHI	4

#define HWVIP_FORMAT_CCIR656	0x00000
#define HWVIP_FORMAT_CCIR601	0x10000

/* ------------------------------------------------------------
// DEFINE		: HWVIP_NOODD
//				  HWVIP_NOEVEN 
// ------------------------------------------------------------
// DESCRIPTION	: Doesn't capture the odd or the even frame
// ------------------------------------------------------------*/
#define	HWVIP_NOODD			0xffffffff
#define	HWVIP_NOEVEN		0xffffffff

/* ------------------------------------------------------------
// FUNCTION		: HWVIP_SetSource
// ------------------------------------------------------------
// DESCRIPTION	: Set the type of video data
//				  This function initialize the video only in
//				  CCIR656 mode
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulSourceType	: video source type : HWVIP_FORMAT_...
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// -----------------------------------------------------s-------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVIP_SetSource(unsigned long ulSourceType);

/* ------------------------------------------------------------
// FUNCTION     : HWVIP_GetSourceSize
// ------------------------------------------------------------
// DESCRIPTION  : Get the size of the input datas
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulSourceType : video source type : HWVIP_FORMAT_...
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ulWidth      : Number of pixels per lines
// ulHeight     : Number lines
// -----------------------------------------------------s-------
// RETURN       : -
// ------------------------------------------------------------
// MODIFIED     : -
// ------------------------------------------------------------*/
void HWVIP_GetSourceSize(unsigned long ulSourceType, unsigned long *ulWidth, unsigned long *ulHeight);

/* ------------------------------------------------------------
// FUNCTION		: HWVIP_SetDestination
// ------------------------------------------------------------
// DESCRIPTION	: Set the location of captured video in the
//				  frame buffer
//
//				  To set a full size video :
//				  ulPitch = 2 * pitch of video
//				  ulOdd = location of video in frame buffer
//				  ulEven = location of video in frame buffer + 
//				  width of video
//				  bDecimator = 0
//
//				  To set a quarter size video :
//				  ulPitch = pitch of video
//				  ulOdd = location of video in frame buffer
//				  ulEven = HWVIP_NOEVEN
//				  bDecimator = 1
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulOdd		: offset of odd frame in frame buffer, 
//				  HWVIP_NOODD <=> odd frame disabled
// ulEven		: offset of even frame in frame buffer, 
//				  HWVIP_NOEVEN <=> even frame disabled
// ulPitch		: Number of bytes between two lines of a frame
// ulDecimator	: TRUE to divide by two the width of the video
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVIP_SetDestination(unsigned long ulOdd, unsigned long ulEven, unsigned long ulPitch, unsigned long ulDecimator);

/* ------------------------------------------------------------
// FUNCTION		: HWVIP_Enable
// ------------------------------------------------------------
// DESCRIPTION	: Start the Video Input Port
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVIP_Enable(void);

/* ------------------------------------------------------------
// FUNCTION		: HWVIP_Disable
// ------------------------------------------------------------
// DESCRIPTION	: Stop the Video Input Port at end of the frame
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVIP_Disable(void);

/* ------------------------------------------------------------
// FUNCTION		: HWVIP_UrgentStop
// ------------------------------------------------------------
// DESCRIPTION	: Stop the Video Input Port without wait for
//				  the end of the frame
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVIP_UrgentStop(void);

/* ------------------------------------------------------------
// FUNCTION		: HWVIP_WaitEndField0
// ------------------------------------------------------------
// DESCRIPTION	: Wait for the end of the odd frame
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVIP_WaitEndField0(void);

/* ------------------------------------------------------------
// FUNCTION		: HWVIP_WaitEndField1
// ------------------------------------------------------------
// DESCRIPTION	: Wait for the end of the even frame
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVIP_WaitEndField1(void);

/* ------------------------------------------------------------
// FUNCTION		: HWVIP_FlipOdd
// ------------------------------------------------------------
// DESCRIPTION	: Change the location of odd frame in frame
//				  buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulOdd		: Offset of odd frame in frame buffer
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVIP_FlipOdd(unsigned long ulOdd);

/* ------------------------------------------------------------
// FUNCTION		: HWVIP_FlipEven
// ------------------------------------------------------------
// DESCRIPTION	: Change the location of even frame in frame
//				  buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulOdd		: Offset of even frame in frame buffer
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVIP_FlipEven(unsigned long ulEven);

#endif


