/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: vpipe.h
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libhard\vpipe
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE:		: July, 18th 2000
// AUTHOR:		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Functions to manage the Video Pipeline
// ------------------------------------------------------------
// VARIABLES	: -
// ------------------------------------------------------------
// MACROS		: HWVPIPE_...
// ------------------------------------------------------------
// FUNCTIONS	: HWVPIPE_Init
//				  HWVPIPE_NotifyGraphicsModeChange
//				  HWVPIPE_InitDestination
//				  HWVPIPE_VerticalFlip
//				  HWVPIPE_InitChromaKey
//				  HWVPIPE_SetFilter
//				  HWVPIPE_InitSource
//				  HWVPIPE_SetSource
//				  HWVPIPE_SetDestination
//				  HWVPIPE_SetColorKey
//				  HWVPIPE_Flip
//				  HWVPIPE_Enable
//				  HWVPIPE_Disable
//				  HWVPIPE_SetClippingArea
//				  HWVPIPE_ExtendedSetDestination
//				  HWVPIPE_ExtendedSetSource
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE:		: July, 18th 2000
// AUTHOR:		: Thierry Vuillaume
// MODIFICATIONS: Add automatic clipping on the physical screen
//				  area. HWCRTC_NotifyGraphicsModeChange to
//				  be call after any graphics mode change
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

#ifndef VPIPE_H
#define VPIPE_H

#include "host.h"
#include "crtc.h"

/* ------------------------------------------------------------
// DEFINE		: HWVPIPE_FORMAT_... 
// ------------------------------------------------------------
// DESCRIPTION	: Format of source data
// ------------------------------------------------------------
// HWVPIPE_FORMAT_RGB555	: RGB 555
// HWVPIPE_FORMAT_RGB565	: RGB 565
// HWVPIPE_FORMAT_YUV422	: YUV 422
// ------------------------------------------------------------*/
#define HWVPIPE_FORMAT_RGB555			1
#define HWVPIPE_FORMAT_RGB565			2
#define HWVPIPE_FORMAT_YUV422			3

/* ------------------------------------------------------------
// DEFINE		: HWVPIPE_COLORKEY_... 
// ------------------------------------------------------------
// DESCRIPTION	: Different type of color key
// ------------------------------------------------------------
// HWVPIPE_COLORKEY_DISABLE	: No color key 
// HWVPIPE_COLORKEY_VIDEO	: Color key on video
// HWVPIPE_COLORKEY_CHROMA	: Chroma key
// ------------------------------------------------------------*/
#define HWVPIPE_COLORKEY_DISABLE		0x0fffffffL
#define HWVPIPE_COLORKEY_VIDEO			0x80000000L
#define HWVPIPE_COLORKEY_CHROMA			0x40000000L

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_Init
// ------------------------------------------------------------
// DESCRIPTION	: Init the videop pipeline cursor library
//				  This functions is necessary only when using
//				  HWVPIPE_Extended... function
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: HWVPIPE_sScreenInfo
// ------------------------------------------------------------*/
void HWVPIPE_Init(void);

/* ------------------------------------------------------------
// FUNCTIONS	: HWVPIPE_NotifyGraphicsModeChange
// ------------------------------------------------------------
// DESCRIPTION	: To be send after a graphics mode change
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_NotifyGraphicsModeChange(void);

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_InitDestination
// ------------------------------------------------------------
// DESCRIPTION	: Init the way Video Pipeline is displayed
//				  on screen
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulColorKey	: Video is displayed only in screen area filled
//				  with this color, in the pixel format
//				  of the screen (8 bit used only in 256 colors...)
//				  HWVPIPE_NOCOLORKEY <=> video always displayed
//				  HWVPIPE_CHROMAKEY  <=> use chroma key,
//				  have to call HWVPIPE_InitChromaKey function
//				  If the color key value is mixed with the
//				  HWVPIPE_VIDEOCOLORKEY flag by an OR operation
//				  a specific color of the Video Pipeline
//				  image is replaced by the graphic (color key
//				  on video in place of graphic)
// ulSourceFormat: HWVPIPE_RGB555, HWVPIPE_RGB565 or HWVPIPE_YUV422
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_InitDestination(unsigned long ulColorKey, unsigned long ulSourceFormat);

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_SetColorKey
// ------------------------------------------------------------
// DESCRIPTION	: Set the color key
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulColorKey	: Color key value
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_SetColorKey(unsigned long ulColorKey);

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_VerticalFlip
// ------------------------------------------------------------
// DESCRIPTION	: Flip verticaly the Video Pipeline image
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_VerticalFlip(void);

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_InitChromaKey
// ------------------------------------------------------------
// DESCRIPTION	: Use chroma key mix mode (replace a range of
//				  video color by the graphic pixels)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulCSC		: TRUE if compare after the color space converter
//				  <=> in RGB in case of YUV input format
// ulLow		: Lowest value of the range
// ulHigh		: Highest value of the range
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_InitChromaKey(unsigned long ulCSC, unsigned long ulLow, unsigned long ulHigh);

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_SetFilter
// ------------------------------------------------------------
// DESCRIPTION	: Enable or disable scaler interpolators
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// In YUV input format :
// ulHorizontalY	: TRUE to enable horizontal luminance filter
// ulHorizontalUV	: TRUE to enable horizontal chrominance filter
// ulVerticalY		: TRUE to enable vertical luminance filter
// ulVerticalUV		: TRUE to enable vertical chrominance filter
//
// In RGB input format :
// ulHorizontalY and bHorizontalUV : TRUE to enable horizontal filter
// ulVerticalY and bVerticalUV     : TRUE to enable vertical filter
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_SetFilter(unsigned long ulHorizontalY, unsigned long ulHorizontalUV, unsigned long ulVerticalY, unsigned long ulVerticalUV);

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_InitSource
// ------------------------------------------------------------
// DESCRIPTION	: Initialize the source image
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulSrcBase	: Offset of the source location in frame buffer
// ulSrcPitch	: Number of bytes between to lines of source data
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_InitSource(unsigned long ulSrcBase, unsigned long ulSrcPitch);

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_SetSource
// ------------------------------------------------------------
// DESCRIPTION	: Initialize the source rectangle
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulSrcX		: X coordinate of the portion of the source
//				  image to display
// ulSrcY		: Y coordinate of the portion of the source
//				  image to display
// ulSrcWidth	: Width of the portion of the source
//				  image to display
// ulSrcHeight	: Height of the portion of the source
//				  image to display
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_SetSource(unsigned long ulSrcX, unsigned long ulSrcY, unsigned long ulSrcWidth, unsigned long ulSrcHeight);

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_SetDestination
// ------------------------------------------------------------
// DESCRIPTION	: Initialize the destination rectangle
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulDstX		: X coordinate of the location of the video
//				  pipeline on screen
// ulDstY		: Y coordinate of the location of the video
//				  pipeline on screen
// ulDstWidth	: Width of the video pipeline on screen
// ulDstHeight	: Height of the video pipeline on screen
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_SetDestination(unsigned long ulDstX, unsigned long ulDstY, unsigned long ulDstWidth, unsigned long ulDstHeight);

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_Flip
// ------------------------------------------------------------
// DESCRIPTION	: Change the source data of video pipeline
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBase		: New base address
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_Flip(unsigned long ulBase);

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_Enable
// ------------------------------------------------------------
// DESCRIPTION	: Enable the Video Pipeline
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_Enable(void);

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_Disable
// ------------------------------------------------------------
// DESCRIPTION	: Disable the Video Pipeline
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_Disable(void);

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_SetClippingArea
// ------------------------------------------------------------
// DESCRIPTION	: Set clipping rectangle for Video Pipeline
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulLeft		: Left side of the clipping area
// ulRight		: Right side of the clipping area
// ulTop		: Top side of the clipping area
// ulBottom		: Bottom side of the clipping area
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_SetClippingArea(unsigned long ulLeft, unsigned long ulRight, unsigned long ulTop, unsigned long ulBottom);

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_ExtendedSetDestination
// ------------------------------------------------------------
// DESCRIPTION	: Initialize the destination rectangle. This 
//				  function is similar to HWVPIPE_SetDestination
//				  but manage clipping
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// lDstX		: X coordinate of the location of the video
//				  pipeline on screen
// lDstY		: Y coordinate of the location of the video
//				  pipeline on screen
// ulDstWidth	: Width of the video pipeline on screen
// ulDstHeight	: Height of the video pipeline on screen
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_ExtendedSetDestination(long lDstX, long lDstY, unsigned long ulDstWidth, unsigned long ulDstHeight);

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_ExtendedSetSource
// ------------------------------------------------------------
// DESCRIPTION	: Initialize the source rectangle. This function
//				  is similar to HWVPIPE_SetSource but manage
//				  clipping
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// lSrcX		: X coordinate of the portion of the source
//				  image to display
// lSrcY		: Y coordinate of the portion of the source
//				  image to display
// ulSrcWidth	: Width of the portion of the source
//				  image to display
// ulSrcHeight	: Height of the portion of the source
//				  image to display
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_ExtendedSetSource(unsigned long ulSrcX, unsigned long ulSrcY, unsigned long ulSrcWidth, unsigned long ulSrcHeight);

#endif
